@@ -41,12 +41,14 @@
          attach_receiver_link/4,
          attach_receiver_link/5,
          attach_receiver_link/6,
+         attach_receiver_link/7,
          attach_link/2,
          detach_link/1,
          send_msg/2,
          accept_msg/2,
          flow_link_credit/3,
          flow_link_credit/4,
+         echo/1,
          link_handle/1,
          get_msg/1,
          get_msg/2,
@@ -66,6 +68,7 @@
 -type attach_role() :: amqp10_client_session:attach_role().
 -type attach_args() :: amqp10_client_session:attach_args().
 -type filter() :: amqp10_client_session:filter().
+-type properties() :: amqp10_client_session:properties().
 
 -type connection_config() :: amqp10_client_connection:connection_config().
 
@@ -257,12 +260,24 @@ attach_receiver_link(Session, Name, Source, SettleMode, Durability) ->
                            snd_settle_mode(), terminus_durability(), filter()) ->
     {ok, link_ref()}.
 attach_receiver_link(Session, Name, Source, SettleMode, Durability, Filter) ->
+    attach_receiver_link(Session, Name, Source, SettleMode, Durability, Filter, #{}).
+
+%% @doc Attaches a receiver link to a source.
+%% This is asynchronous and will notify completion of the attach request to the
+%% caller using an amqp10_event of the following format:
+%% {amqp10_event, {link, LinkRef, attached | {detached, Why}}}
+-spec attach_receiver_link(pid(), binary(), binary(),
+                           snd_settle_mode(), terminus_durability(), filter(),
+                           properties()) ->
+    {ok, link_ref()}.
+attach_receiver_link(Session, Name, Source, SettleMode, Durability, Filter, Properties) ->
     AttachArgs = #{name => Name,
                    role => {receiver, #{address => Source,
                                         durable => Durability}, self()},
                    snd_settle_mode => SettleMode,
                    rcv_settle_mode => first,
-                   filter => Filter},
+                   filter => Filter,
+                   properties => Properties},
     amqp10_client_session:attach(Session, AttachArgs).
 
 -spec attach_link(pid(), attach_args()) -> {ok, link_ref()}.
@@ -299,6 +314,15 @@ flow_link_credit(#link_ref{role = receiver, session = Session,
                         drain = Drain},
     ok = amqp10_client_session:flow(Session, Handle, Flow, RenewWhenBelow).
 
+%% @doc Request that the sender's flow state is echoed back
+%% This may be used to determine when the Link has finally quiesced.
+%% see §2.6.10 of the spec
+echo(#link_ref{role = receiver, session = Session,
+               link_handle = Handle}) ->
+    Flow = #'v1_0.flow'{link_credit = {uint, 0},
+                        echo = true},
+    ok = amqp10_client_session:flow(Session, Handle, Flow, 0).
+
 %%% messages
 
 %% @doc Send a message on a the link referred to be the 'LinkRef'.
