@@ -94,6 +94,7 @@
 
 % http://www.amqp.org/specification/1.0/filters
 -type filter() :: #{binary() => binary() | map() | list(binary())}.
+-type properties() :: #{binary() => tuple()}.
 
 -type attach_args() :: #{name => binary(),
                          role => attach_role(),
@@ -111,6 +112,7 @@
               attach_role/0,
               target_def/0,
               source_def/0,
+              properties/0,
               filter/0]).
 
 -record(link,
@@ -653,6 +655,19 @@ make_target(#{role := {sender, #{address := Address} = Target}}) ->
     #'v1_0.target'{address = {utf8, Address},
                    durable = {uint, Durable}}.
 
+make_properties(#{properties := Properties}) ->
+    translate_properties(Properties);
+make_properties(_) ->
+    undefined.
+
+translate_properties(Properties) when is_map(Properties) andalso map_size(Properties) =< 0 ->
+    undefined;
+translate_properties(Properties) when is_map(Properties) ->
+    {map, maps:fold(fun translate_property/3, [], Properties)}.
+
+translate_property(K, V, Acc) when is_tuple(V) ->
+    [{{symbol, K}, V} | Acc].
+
 translate_terminus_durability(none) -> 0;
 translate_terminus_durability(configuration) -> 1;
 translate_terminus_durability(unsettled_state) -> 2.
@@ -662,15 +677,15 @@ translate_filters(Filters) when is_map(Filters) -> {
     map,
     maps:fold(
         fun(<<"apache.org:legacy-amqp-direct-binding:string">> = K, V, Acc) when is_binary(V) ->
-            [{{symbol, K}, {utf8, V}} | Acc];
+            [{{symbol, K}, {described, {symbol, K}, {utf8, V}}} | Acc];
         (<<"apache.org:legacy-amqp-topic-binding:string">> = K, V, Acc) when is_binary(V) ->
-            [{{symbol, K}, {utf8, V}} | Acc];
+            [{{symbol, K}, {described, {symbol, K}, {utf8, V}}} | Acc];
         (<<"apache.org:legacy-amqp-headers-binding:map">> = K, V, Acc) when is_map(V) ->
-            [{{symbol, K}, translate_legacy_amqp_headers_binding(V)} | Acc];
+            [{{symbol, K}, {described, {symbol, K}, translate_legacy_amqp_headers_binding(V)}} | Acc];
         (<<"apache.org:no-local-filter:list">> = K, V, Acc) when is_list(V) ->
-            [{{symbol, K}, lists:map(fun(Id) -> {utf8, Id} end, V)} | Acc];
+            [{{symbol, K}, {described, {symbol, K}, lists:map(fun(Id) -> {utf8, Id} end, V)}} | Acc];
         (<<"apache.org:selector-filter:string">> = K, V, Acc) when is_binary(V) ->
-            [{{symbol, K}, {utf8, V}} | Acc]
+                [{{symbol, K}, {described, {symbol, K}, {utf8, V}}} | Acc]
         end,
         [],
         Filters)
@@ -723,6 +738,7 @@ send_attach(Send, #{name := Name, role := Role} = Args, {FromPid, _},
 
     Source = make_source(Args),
     Target = make_target(Args),
+    Properties = make_properties(Args),
 
     {LinkTarget, RoleAsBool} = case Role of
                                    {receiver, _, Pid} ->
@@ -736,6 +752,7 @@ send_attach(Send, #{name := Name, role := Role} = Args, {FromPid, _},
                             role = RoleAsBool,
                             handle = {uint, OutHandle},
                             source = Source,
+                            properties = Properties,
                             initial_delivery_count =
                                 {uint, ?INITIAL_DELIVERY_COUNT},
                             snd_settle_mode = snd_settle_mode(Args),
@@ -1070,7 +1087,7 @@ translate_filters_legacy_amqp_direct_binding_filter_test() ->
     {map,
         [{
             {symbol,<<"apache.org:legacy-amqp-direct-binding:string">>},
-            {utf8,<<"my topic">>}
+            {described, {symbol, <<"apache.org:legacy-amqp-direct-binding:string">>}, {utf8,<<"my topic">>}}
         }]
     } = translate_filters(#{<<"apache.org:legacy-amqp-direct-binding:string">> => <<"my topic">>}).
 
@@ -1078,20 +1095,21 @@ translate_filters_legacy_amqp_topic_binding_filter_test() ->
     {map,
         [{
             {symbol, <<"apache.org:legacy-amqp-topic-binding:string">>},
-            {utf8, <<"*.stock.#">>}
+            {described, {symbol, <<"apache.org:legacy-amqp-topic-binding:string">>}, {utf8, <<"*.stock.#">>}}
         }]
     } = translate_filters(#{<<"apache.org:legacy-amqp-topic-binding:string">> => <<"*.stock.#">>}).
 
 translate_filters_legacy_amqp_headers_binding_filter_test() ->
     {map,
         [{
-            {symbol,<<"apache.org:legacy-amqp-headers-binding:map">>},
-            {map, [
+            {symbol, <<"apache.org:legacy-amqp-headers-binding:map">>},
+            {described, {symbol, <<"apache.org:legacy-amqp-headers-binding:map">>},
+             {map, [
                     {{utf8, <<"x-match">>}, {utf8, <<"all">>}},
                     {{utf8, <<"foo">>}, {utf8, <<"bar">>}},
                     {{utf8, <<"bar">>}, {utf8, <<"baz">>}}
                 ]
-            }
+            }}
         }]
     } = translate_filters(#{<<"apache.org:legacy-amqp-headers-binding:map">> => #{
             <<"x-match">> => <<"all">>,
@@ -1104,7 +1122,7 @@ translate_filters_legacy_amqp_no_local_filter_test() ->
     {map,
         [{
             {symbol, <<"apache.org:no-local-filter:list">>},
-            [{utf8, <<"foo">>}, {utf8, <<"bar">>}]
+            {described, {symbol, <<"apache.org:no-local-filter:list">>}, [{utf8, <<"foo">>}, {utf8, <<"bar">>}]}
         }]
     } = translate_filters(#{<<"apache.org:no-local-filter:list">> => [<<"foo">>, <<"bar">>]}).
 
@@ -1112,15 +1130,19 @@ translate_filters_selector_filter_test() ->
     {map,
         [{
             {symbol, <<"apache.org:selector-filter:string">>},
-            {utf8, <<"amqp.annotation.x-opt-enqueuedtimeutc > 123456789">>}
+            {described, {symbol, <<"apache.org:selector-filter:string">>},
+             {utf8, <<"amqp.annotation.x-opt-enqueuedtimeutc > 123456789">>}}
         }]
     } = translate_filters(#{<<"apache.org:selector-filter:string">> => <<"amqp.annotation.x-opt-enqueuedtimeutc > 123456789">>}).
 
 translate_filters_multiple_filters_test() ->
     {map,
         [
-            {{symbol, <<"apache.org:selector-filter:string">>}, {utf8, <<"amqp.annotation.x-opt-enqueuedtimeutc > 123456789">>}},
-            {{symbol, <<"apache.org:legacy-amqp-direct-binding:string">>}, {utf8,<<"my topic">>}}
+            {{symbol, <<"apache.org:selector-filter:string">>},
+             {described, {symbol, <<"apache.org:selector-filter:string">>},
+              {utf8, <<"amqp.annotation.x-opt-enqueuedtimeutc > 123456789">>}}},
+            {{symbol, <<"apache.org:legacy-amqp-direct-binding:string">>},
+             {described, {symbol, <<"apache.org:legacy-amqp-direct-binding:string">>}, {utf8,<<"my topic">>}}}
         ]
     } = translate_filters(#{
             <<"apache.org:legacy-amqp-direct-binding:string">> => <<"my topic">>,
