@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -73,7 +73,7 @@
 -export([start/1, start/2, close/1, close/2, close/3, close/4]).
 -export([error_atom/1]).
 -export([info/2, info_keys/1, info_keys/0]).
--export([connection_name/1]).
+-export([connection_name/1, update_secret/3]).
 -export([socket_adapter_info/2]).
 
 -define(DEFAULT_CONSUMER, {amqp_selective_consumer, []}).
@@ -128,8 +128,8 @@
 %%     client properties to be sent to the server, defaults to []</li>
 %% <li>socket_options :: [any()] - Extra socket options.  These are
 %%     appended to the default options.  See
-%%     <a href="http://www.erlang.org/doc/man/inet.html#setopts-2">inet:setopts/2</a>
-%%     and <a href="http://www.erlang.org/doc/man/gen_tcp.html#connect-4">
+%%     <a href="https://www.erlang.org/doc/man/inet.html#setopts-2">inet:setopts/2</a>
+%%     and <a href="https://www.erlang.org/doc/man/gen_tcp.html#connect-4">
 %%     gen_tcp:connect/4</a> for descriptions of the available options.</li>
 %% </ul>
 
@@ -307,6 +307,16 @@ close(ConnectionPid, Code, Text, Timeout) ->
 register_blocked_handler(ConnectionPid, BlockHandler) ->
     amqp_gen_connection:register_blocked_handler(ConnectionPid, BlockHandler).
 
+-spec update_secret(pid(), term(), binary()) ->
+    {'ok', rabbit_types:auth_user()} |
+    {'refused', string(), [any()]} |
+    {'error', any()}.
+
+update_secret(ConnectionPid, NewSecret, Reason) ->
+  Update = #'connection.update_secret'{new_secret = NewSecret,
+                                       reason = Reason},
+  amqp_gen_connection:update_secret(ConnectionPid, Update).
+
 %%---------------------------------------------------------------------------
 %% Other functions
 %%---------------------------------------------------------------------------
