@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -88,7 +88,7 @@ handle_info({Tag, Sock}, State = #state{sock = Sock})
 handle_info({Tag, Sock, Reason}, State = #state{sock = Sock})
             when Tag =:= tcp_error; Tag =:= ssl_error ->
     handle_error(Reason, State);
-handle_info({timeout, TimerRef, idle_timeout}, State = #state{timer = TimerRef}) ->
+handle_info({timeout, _TimerRef, idle_timeout}, State) ->
     handle_error(timeout, State).
 
 handle_data(<<Type:8, Channel:16, Length:32, Payload:Length/binary, ?FRAME_END,
