@@ -4,7 +4,7 @@ use Mix.Project
   def project do
     [
       app: :jsx,
-      version: "2.8.2",
+      version: "2.9.0",
       description: "an erlang application for consuming, producing and manipulating json. inspired by yajl",
       deps: deps(Mix.env),
       package: package(),
@@ -35,8 +35,8 @@ use Mix.Project
         "README.md",
         "src"
       ],
-      contributors: ["alisdair sullivan"],
-      maintainers: ["@talentdeficit"],
+      contributors: [ "@talentdeficit" ],
+      maintainers: [ "@talentdeficit" ],
       links: %{"github" => "https://github.com/talentdeficit/jsx"},
       licenses: ["MIT"]
     ]
