@@ -23,6 +23,7 @@
          extract_headers/1, extract_timestamp/1, map_headers/2, delivery/4,
          header_routes/1, parse_expiration/1, header/2, header/3]).
 -export([build_content/2, from_content/1, msg_size/1, maybe_gc_large_msg/1]).
+-export([add_header/4]).
 
 %%----------------------------------------------------------------------------
 
@@ -36,61 +37,27 @@
 -type exchange_input() :: rabbit_types:exchange() | rabbit_exchange:name().
 -type body_input() :: binary() | [binary()].
 
--spec publish
-        (exchange_input(), rabbit_router:routing_key(), properties_input(),
-         body_input()) ->
-            publish_result().
--spec publish
-        (exchange_input(), rabbit_router:routing_key(), boolean(),
-         properties_input(), body_input()) ->
-            publish_result().
--spec publish(rabbit_types:delivery()) -> publish_result().
--spec delivery
-        (boolean(), boolean(), rabbit_types:message(), undefined | integer()) ->
-            rabbit_types:delivery().
--spec message
-        (rabbit_exchange:name(), rabbit_router:routing_key(), properties_input(),
-         binary()) ->
-            rabbit_types:message().
--spec message
-        (rabbit_exchange:name(), rabbit_router:routing_key(),
-         rabbit_types:decoded_content()) ->
-            rabbit_types:ok_or_error2(rabbit_types:message(), any()).
--spec properties
-        (properties_input()) -> rabbit_framing:amqp_property_record().
-
--spec prepend_table_header
-        (binary(), rabbit_framing:amqp_table(), headers()) -> headers().
-
--spec header(header(), headers()) -> 'undefined' | any().
--spec header(header(), headers(), any()) -> 'undefined' | any().
-
--spec extract_headers(rabbit_types:content()) -> headers().
-
--spec map_headers
-        (fun((headers()) -> headers()), rabbit_types:content()) ->
-            rabbit_types:content().
-
--spec header_routes(undefined | rabbit_framing:amqp_table()) -> [string()].
--spec build_content
-        (rabbit_framing:amqp_property_record(), binary() | [binary()]) ->
-            rabbit_types:content().
--spec from_content
-        (rabbit_types:content()) ->
-            {rabbit_framing:amqp_property_record(), binary()}.
--spec parse_expiration
-        (rabbit_framing:amqp_property_record()) ->
-            rabbit_types:ok_or_error2('undefined' | non_neg_integer(), any()).
-
 %%----------------------------------------------------------------------------
 
 %% Convenience function, for avoiding round-trips in calls across the
 %% erlang distributed network.
+
+-spec publish
+        (exchange_input(), rabbit_router:routing_key(), properties_input(),
+         body_input()) ->
+            publish_result().
+
 publish(Exchange, RoutingKeyBin, Properties, Body) ->
     publish(Exchange, RoutingKeyBin, false, Properties, Body).
 
 %% Convenience function, for avoiding round-trips in calls across the
 %% erlang distributed network.
+
+-spec publish
+        (exchange_input(), rabbit_router:routing_key(), boolean(),
+         properties_input(), body_input()) ->
+            publish_result().
+
 publish(X = #exchange{name = XName}, RKey, Mandatory, Props, Body) ->
     Message = message(XName, RKey, properties(Props), Body),
     publish(X, delivery(Mandatory, false, Message, undefined));
@@ -98,6 +65,8 @@ publish(XName, RKey, Mandatory, Props, Body) ->
     Message = message(XName, RKey, properties(Props), Body),
     publish(delivery(Mandatory, false, Message, undefined)).
 
+-spec publish(rabbit_types:delivery()) -> publish_result().
+
 publish(Delivery = #delivery{
           message = #basic_message{exchange_name = XName}}) ->
     case rabbit_exchange:lookup(XName) of
@@ -107,13 +76,20 @@ publish(Delivery = #delivery{
 
 publish(X, Delivery) ->
     Qs = rabbit_amqqueue:lookup(rabbit_exchange:route(X, Delivery)),
-    _QPids = rabbit_amqqueue:deliver(Qs, Delivery),
-    ok.
+    rabbit_amqqueue:deliver(Qs, Delivery).
+
+-spec delivery
+        (boolean(), boolean(), rabbit_types:message(), undefined | integer()) ->
+            rabbit_types:delivery().
 
 delivery(Mandatory, Confirm, Message, MsgSeqNo) ->
     #delivery{mandatory = Mandatory, confirm = Confirm, sender = self(),
               message = Message, msg_seq_no = MsgSeqNo, flow = noflow}.
 
+-spec build_content
+        (rabbit_framing:amqp_property_record(), binary() | [binary()]) ->
+            rabbit_types:content().
+
 build_content(Properties, BodyBin) when is_binary(BodyBin) ->
     build_content(Properties, [BodyBin]);
 
@@ -127,6 +103,10 @@ build_content(Properties, PFR) ->
              protocol = none,
              payload_fragments_rev = PFR}.
 
+-spec from_content
+        (rabbit_types:content()) ->
+            {rabbit_framing:amqp_property_record(), binary()}.
+
 from_content(Content) ->
     #content{class_id = ClassId,
              properties = Props,
@@ -152,6 +132,11 @@ strip_header(#content{properties = Props = #'P_basic'{headers = Headers}}
                                              headers = Headers0}})
     end.
 
+-spec message
+        (rabbit_exchange:name(), rabbit_router:routing_key(),
+         rabbit_types:decoded_content()) ->
+            rabbit_types:ok_or_error2(rabbit_types:message(), any()).
+
 message(XName, RoutingKey, #content{properties = Props} = DecodedContent) ->
     try
         {ok, #basic_message{
@@ -165,12 +150,20 @@ message(XName, RoutingKey, #content{properties = Props} = DecodedContent) ->
         {error, _Reason} = Error -> Error
     end.
 
+-spec message
+        (rabbit_exchange:name(), rabbit_router:routing_key(), properties_input(),
+         binary()) ->
+            rabbit_types:message().
+
 message(XName, RoutingKey, RawProperties, Body) ->
     Properties = properties(RawProperties),
     Content = build_content(Properties, Body),
     {ok, Msg} = message(XName, RoutingKey, Content),
     Msg.
 
+-spec properties
+        (properties_input()) -> rabbit_framing:amqp_property_record().
+
 properties(P = #'P_basic'{}) ->
     P;
 properties(P) when is_list(P) ->
@@ -184,6 +177,9 @@ properties(P) when is_list(P) ->
                         end
                 end, #'P_basic'{}, P).
 
+-spec prepend_table_header
+        (binary(), rabbit_framing:amqp_table(), headers()) -> headers().
+
 prepend_table_header(Name, Info, undefined) ->
     prepend_table_header(Name, Info, []);
 prepend_table_header(Name, Info, Headers) ->
@@ -223,6 +219,8 @@ update_invalid(Name, Value, ExistingHdr, Header) ->
     NewHdr = rabbit_misc:set_table_value(ExistingHdr, Name, array, Values),
     set_invalid(NewHdr, Header).
 
+-spec header(header(), headers()) -> 'undefined' | any().
+
 header(_Header, undefined) ->
     undefined;
 header(_Header, []) ->
@@ -230,12 +228,16 @@ header(_Header, []) ->
 header(Header, Headers) ->
     header(Header, Headers, undefined).
 
+-spec header(header(), headers(), any()) -> 'undefined' | any().
+
 header(Header, Headers, Default) ->
     case lists:keysearch(Header, 1, Headers) of
         false        -> Default;
         {value, Val} -> Val
     end.
 
+-spec extract_headers(rabbit_types:content()) -> headers().
+
 extract_headers(Content) ->
     #content{properties = #'P_basic'{headers = Headers}} =
         rabbit_binary_parser:ensure_content_decoded(Content),
@@ -246,6 +248,10 @@ extract_timestamp(Content) ->
         rabbit_binary_parser:ensure_content_decoded(Content),
     Timestamp.
 
+-spec map_headers
+        (fun((headers()) -> headers()), rabbit_types:content()) ->
+            rabbit_types:content().
+
 map_headers(F, Content) ->
     Content1 = rabbit_binary_parser:ensure_content_decoded(Content),
     #content{properties = #'P_basic'{headers = Headers} = Props} = Content1,
@@ -269,6 +275,9 @@ is_message_persistent(#content{properties = #'P_basic'{
     end.
 
 %% Extract CC routes from headers
+
+-spec header_routes(undefined | rabbit_framing:amqp_table()) -> [string()].
+
 header_routes(undefined) ->
     [];
 header_routes(HeadersTable) ->
@@ -280,6 +289,10 @@ header_routes(HeadersTable) ->
                                              binary_to_list(HeaderKey), Type}})
        end || HeaderKey <- ?ROUTING_HEADERS]).
 
+-spec parse_expiration
+        (rabbit_framing:amqp_property_record()) ->
+            rabbit_types:ok_or_error2('undefined' | non_neg_integer(), any()).
+
 parse_expiration(#'P_basic'{expiration = undefined}) ->
     {ok, undefined};
 parse_expiration(#'P_basic'{expiration = Expiration}) ->
@@ -300,3 +313,12 @@ maybe_gc_large_msg(Content) ->
 
 msg_size(Content) ->
     rabbit_writer:msg_size(Content).
+
+add_header(Name, Type, Value, #basic_message{content = Content0} = Msg) ->
+    Content = rabbit_basic:map_headers(
+                fun(undefined) ->
+                        rabbit_misc:set_table_value([], Name, Type, Value);
+                   (Headers) ->
+                        rabbit_misc:set_table_value(Headers, Name, Type, Value)
+                end, Content0),
+    Msg#basic_message{content = Content}.
