@@ -21,19 +21,20 @@
 %% Internal API
 -export([local/0]).
 
--spec node(node(), timeout()) -> ok | {badrpc, term()} | {error_string, string()}.
--spec local() -> ok | {error_string, string()}.
-
 %%----------------------------------------------------------------------------
 %% External functions
 %%----------------------------------------------------------------------------
 
+-spec node(node(), timeout()) -> ok | {badrpc, term()} | {error_string, string()}.
+
 node(Node) ->
     %% same default as in CLI
     node(Node, 70000).
 node(Node, Timeout) ->
     rabbit_misc:rpc_call(Node, rabbit_health_check, local, [], Timeout).
 
+-spec local() -> ok | {error_string, string()}.
+
 local() ->
     run_checks([list_channels, list_queues, alarms, rabbit_node_monitor]).
 
@@ -57,12 +58,15 @@ node_health_check(list_channels) ->
     end;
 
 node_health_check(list_queues) ->
-    health_check_queues(rabbit_vhost:list());
+    health_check_queues(rabbit_vhost:list_names());
 
 node_health_check(rabbit_node_monitor) ->
     case rabbit_node_monitor:partitions() of
-        L when is_list(L) ->
-            ok
+        [] ->
+            ok;
+        L when is_list(L), length(L) > 0 ->
+            ErrorMsg = io_lib:format("cluster partition in effect: ~p", [L]),
+            {error_string, ErrorMsg}
     end;
 
 node_health_check(alarms) ->
