@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2010-2015 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2010-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mirror_queue_coordinator).
@@ -26,7 +26,8 @@
 -behaviour(gen_server2).
 -behaviour(gm).
 
--include("rabbit.hrl").
+-include_lib("rabbit_common/include/rabbit.hrl").
+-include("amqqueue.hrl").
 -include("gm_specs.hrl").
 
 -record(state, { q,
@@ -36,14 +37,6 @@
                  depth_fun
                }).
 
--spec start_link
-        (rabbit_types:amqqueue(), pid() | 'undefined',
-         rabbit_mirror_queue_master:death_fun(),
-         rabbit_mirror_queue_master:depth_fun()) ->
-            rabbit_types:ok_pid_or_error().
--spec get_gm(pid()) -> pid().
--spec ensure_monitoring(pid(), [pid()]) -> 'ok'.
-
 %%----------------------------------------------------------------------------
 %%
 %% Mirror Queues
@@ -306,12 +299,22 @@
 %%
 %%----------------------------------------------------------------------------
 
+-spec start_link
+        (amqqueue:amqqueue(), pid() | 'undefined',
+         rabbit_mirror_queue_master:death_fun(),
+         rabbit_mirror_queue_master:depth_fun()) ->
+            rabbit_types:ok_pid_or_error().
+
 start_link(Queue, GM, DeathFun, DepthFun) ->
     gen_server2:start_link(?MODULE, [Queue, GM, DeathFun, DepthFun], []).
 
+-spec get_gm(pid()) -> pid().
+
 get_gm(CPid) ->
     gen_server2:call(CPid, get_gm, infinity).
 
+-spec ensure_monitoring(pid(), [pid()]) -> 'ok'.
+
 ensure_monitoring(CPid, Pids) ->
     gen_server2:cast(CPid, {ensure_monitoring, Pids}).
 
@@ -319,7 +322,8 @@ ensure_monitoring(CPid, Pids) ->
 %% gen_server
 %% ---------------------------------------------------------------------------
 
-init([#amqqueue { name = QueueName } = Q, GM, DeathFun, DepthFun]) ->
+init([Q, GM, DeathFun, DepthFun]) when ?is_amqqueue(Q) ->
+    QueueName = amqqueue:get_name(Q),
     ?store_proc_name(QueueName),
     GM1 = case GM of
               undefined ->
@@ -345,9 +349,9 @@ init([#amqqueue { name = QueueName } = Q, GM, DeathFun, DepthFun]) ->
 handle_call(get_gm, _From, State = #state { gm = GM }) ->
     reply(GM, State).
 
-handle_cast({gm_deaths, DeadGMPids},
-            State = #state { q  = #amqqueue { name = QueueName, pid = MPid } })
-  when node(MPid) =:= node() ->
+handle_cast({gm_deaths, DeadGMPids}, State = #state{q = Q}) when ?amqqueue_pid_runs_on_local_node(Q) ->
+    QueueName = amqqueue:get_name(Q),
+    MPid = amqqueue:get_pid(Q),
     case rabbit_mirror_queue_misc:remove_from_queue(
            QueueName, MPid, DeadGMPids) of
         {ok, MPid, DeadPids, ExtraNodes} ->
@@ -373,14 +377,15 @@ handle_cast({gm_deaths, DeadGMPids},
             error(unexpected_mirrored_state)
     end;
 
-handle_cast(request_depth, State = #state { depth_fun = DepthFun,
-					    q  = #amqqueue { name = QName, pid = MPid }}) ->
+handle_cast(request_depth, State = #state{depth_fun = DepthFun, q = QArg}) when ?is_amqqueue(QArg) ->
+    QName = amqqueue:get_name(QArg),
+    MPid = amqqueue:get_pid(QArg),
     case rabbit_amqqueue:lookup(QName) of
-	{ok, #amqqueue{ pid = MPid }} ->
-	    ok = DepthFun(),
-	    noreply(State);
-	_ ->
-	    {stop, shutdown, State}
+        {ok, QFound} when ?amqqueue_pid_equals(QFound, MPid) ->
+            ok = DepthFun(),
+            noreply(State);
+        _ ->
+            {stop, shutdown, State}
     end;
 
 handle_cast({ensure_monitoring, Pids}, State = #state { monitors = Mons }) ->
