@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2010-2015 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2010-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mirror_queue_slave).
@@ -35,8 +35,9 @@
 -behaviour(gen_server2).
 -behaviour(gm).
 
--include("rabbit.hrl").
+-include_lib("rabbit_common/include/rabbit.hrl").
 
+-include("amqqueue.hrl").
 -include("gm_specs.hrl").
 
 %%----------------------------------------------------------------------------
@@ -76,8 +77,9 @@ set_maximum_since_use(QPid, Age) ->
 
 info(QPid) -> gen_server2:call(QPid, info, infinity).
 
-init(Q) ->
-    ?store_proc_name(Q#amqqueue.name),
+init(Q) when ?is_amqqueue(Q) ->
+    QName = amqqueue:get_name(Q),
+    ?store_proc_name(QName),
     {ok, {not_started, Q}, hibernate,
      {backoff, ?HIBERNATE_AFTER_MIN, ?HIBERNATE_AFTER_MIN,
       ?DESIRED_HIBERNATE}, ?MODULE}.
@@ -85,7 +87,8 @@ init(Q) ->
 go(SPid, sync)  -> gen_server2:call(SPid, go, infinity);
 go(SPid, async) -> gen_server2:cast(SPid, go).
 
-handle_go(Q = #amqqueue{name = QName}) ->
+handle_go(Q) when ?is_amqqueue(Q) ->
+    QName = amqqueue:get_name(Q),
     %% We join the GM group before we add ourselves to the amqqueue
     %% record. As a result:
     %% 1. We can receive msgs from GM that correspond to messages we will
@@ -119,10 +122,10 @@ handle_go(Q = #amqqueue{name = QName}) ->
             ok = rabbit_memory_monitor:register(
                    Self, {rabbit_amqqueue, set_ram_duration_target, [Self]}),
             {ok, BQ} = application:get_env(backing_queue_module),
-            Q1 = Q #amqqueue { pid = QPid },
+            QPid = amqqueue:get_pid(Q),
             _ = BQ:delete_crashed(Q), %% For crash recovery
-            BQS = bq_init(BQ, Q1, new),
-            State = #state { q                   = Q1,
+            BQS = bq_init(BQ, Q, new),
+            State = #state { q                   = Q,
                              gm                  = GM,
                              backing_queue       = BQ,
                              backing_queue_state = BQS,
@@ -139,7 +142,7 @@ handle_go(Q = #amqqueue{name = QName}) ->
                    },
             ok = gm:broadcast(GM, request_depth),
             ok = gm:validate_members(GM, [GM | [G || {G, _} <- GMPids]]),
-            rabbit_mirror_queue_misc:maybe_auto_sync(Q1),
+            rabbit_mirror_queue_misc:maybe_auto_sync(Q),
             {ok, State};
         {stale, StalePid} ->
             rabbit_mirror_queue_misc:log_warning(
@@ -163,8 +166,11 @@ handle_go(Q = #amqqueue{name = QName}) ->
 
 init_it(Self, GM, Node, QName) ->
     case mnesia:read({rabbit_queue, QName}) of
-        [Q = #amqqueue { pid = QPid, slave_pids = SPids, gm_pids = GMPids,
-                         slave_pids_pending_shutdown = PSPids}] ->
+        [Q] when ?is_amqqueue(Q) ->
+            QPid = amqqueue:get_pid(Q),
+            SPids = amqqueue:get_slave_pids(Q),
+            GMPids = amqqueue:get_gm_pids(Q),
+            PSPids = amqqueue:get_slave_pids_pending_shutdown(Q),
             case [Pid || Pid <- [QPid | SPids], node(Pid) =:= Node] of
                 []     -> stop_pending_slaves(QName, PSPids),
                           add_slave(Q, Self, GM),
@@ -175,12 +181,11 @@ init_it(Self, GM, Node, QName) ->
                           end;
                 [SPid] -> case rabbit_mnesia:is_process_alive(SPid) of
                               true  -> existing;
-                              false -> GMPids1 = [T || T = {_, S} <- GMPids,
-                                                       S =/= SPid],
-                                       Q1 = Q#amqqueue{
-                                              slave_pids = SPids -- [SPid],
-                                              gm_pids    = GMPids1},
-                                       add_slave(Q1, Self, GM),
+                              false -> GMPids1 = [T || T = {_, S} <- GMPids, S =/= SPid],
+                                       SPids1 = SPids -- [SPid],
+                                       Q1 = amqqueue:set_slave_pids(Q, SPids1),
+                                       Q2 = amqqueue:set_gm_pids(Q1, GMPids1),
+                                       add_slave(Q2, Self, GM),
                                        {new, QPid, GMPids1}
                           end
             end;
@@ -212,9 +217,14 @@ stop_pending_slaves(QName, Pids) ->
 
 %% Add to the end, so they are in descending order of age, see
 %% rabbit_mirror_queue_misc:promote_slave/1
-add_slave(Q = #amqqueue { slave_pids = SPids, gm_pids = GMPids }, New, GM) ->
-    rabbit_mirror_queue_misc:store_updated_slaves(
-      Q#amqqueue{slave_pids = SPids ++ [New], gm_pids = [{GM, New} | GMPids]}).
+add_slave(Q0, New, GM) when ?is_amqqueue(Q0) ->
+    SPids = amqqueue:get_slave_pids(Q0),
+    GMPids = amqqueue:get_gm_pids(Q0),
+    SPids1 = SPids ++ [New],
+    GMPids1 = [{GM, New} | GMPids],
+    Q1 = amqqueue:set_slave_pids(Q0, SPids1),
+    Q2 = amqqueue:set_gm_pids(Q1, GMPids1),
+    rabbit_mirror_queue_misc:store_updated_slaves(Q2).
 
 handle_call(go, _From, {not_started, Q} = NotStarted) ->
     case handle_go(Q) of
@@ -223,10 +233,11 @@ handle_call(go, _From, {not_started, Q} = NotStarted) ->
     end;
 
 handle_call({gm_deaths, DeadGMPids}, From,
-            State = #state{ gm = GM,
-                            q = Q = #amqqueue{ name = QName, pid = MPid },
-                            backing_queue       = BQ,
-                            backing_queue_state = BQS}) ->
+            State = #state{ gm = GM, q = Q,
+                            backing_queue = BQ,
+                            backing_queue_state = BQS}) when ?is_amqqueue(Q) ->
+    QName = amqqueue:get_name(Q),
+    MPid = amqqueue:get_pid(Q),
     Self = self(),
     case rabbit_mirror_queue_misc:remove_from_queue(QName, Self, DeadGMPids) of
         {error, not_found} ->
@@ -269,7 +280,9 @@ handle_call({gm_deaths, DeadGMPids}, From,
                     %% death. That is all process_death does, create
                     %% some traffic.
                     ok = gm:broadcast(GM, process_death),
-                    noreply(State #state { q = Q #amqqueue { pid = Pid } })
+                    Q1 = amqqueue:set_pid(Q, Pid),
+                    State1 = State#state{q = Q1},
+                    noreply(State1)
             end
     end;
 
@@ -285,20 +298,22 @@ handle_cast(go, {not_started, Q} = NotStarted) ->
 handle_cast({run_backing_queue, Mod, Fun}, State) ->
     noreply(run_backing_queue(Mod, Fun, State));
 
-handle_cast({gm, Instruction}, State = #state{q = #amqqueue { name = QName }}) ->
+handle_cast({gm, Instruction}, State = #state{q = Q0}) when ?is_amqqueue(Q0) ->
+    QName = amqqueue:get_name(Q0),
     case rabbit_amqqueue:lookup(QName) of
-	{ok, #amqqueue{slave_pids = SPids}} ->
-	    case lists:member(self(), SPids) of
-		true ->
-		    handle_process_result(process_instruction(Instruction, State));
-		false ->
-		    %% Potentially a duplicated slave caused by a partial partition,
-		    %% will stop as a new slave could start unaware of our presence
-		    {stop, shutdown, State}
-	    end;
-	{error, not_found} ->
-	    %% Would not expect this to happen after fixing #953
-	    {stop, shutdown, State}
+       {ok, Q1} when ?is_amqqueue(Q1) ->
+           SPids = amqqueue:get_slave_pids(Q1),
+           case lists:member(self(), SPids) of
+               true ->
+                   handle_process_result(process_instruction(Instruction, State));
+               false ->
+                   %% Potentially a duplicated slave caused by a partial partition,
+                   %% will stop as a new slave could start unaware of our presence
+                   {stop, shutdown, State}
+           end;
+       {error, not_found} ->
+           %% Would not expect this to happen after fixing #953
+           {stop, shutdown, State}
     end;
 
 handle_cast({deliver, Delivery = #delivery{sender = Sender, flow = Flow}, true},
@@ -521,11 +536,16 @@ handle_terminate([_SPid], _Reason) ->
 
 infos(Items, State) -> [{Item, i(Item, State)} || Item <- Items].
 
-i(pid,             _State)                                   -> self();
-i(name,            #state { q = #amqqueue { name = Name } }) -> Name;
-i(master_pid,      #state { q = #amqqueue { pid  = MPid } }) -> MPid;
-i(is_synchronised, #state { depth_delta = DD })              -> DD =:= 0;
-i(Item,            _State) -> throw({bad_argument, Item}).
+i(pid, _State) ->
+    self();
+i(name, #state{q = Q}) when ?is_amqqueue(Q) ->
+    amqqueue:get_name(Q);
+i(master_pid, #state{q = Q}) when ?is_amqqueue(Q) ->
+    amqqueue:get_pid(Q);
+i(is_synchronised, #state{depth_delta = DD}) ->
+    DD =:= 0;
+i(Item, _State) ->
+    throw({bad_argument, Item}).
 
 bq_init(BQ, Q, Recover) ->
     Self = self(),
@@ -542,6 +562,11 @@ run_backing_queue(Mod, Fun, State = #state { backing_queue       = BQ,
                                              backing_queue_state = BQS }) ->
     State #state { backing_queue_state = BQ:invoke(Mod, Fun, BQS) }.
 
+%% This feature was used by `rabbit_amqqueue_process` and
+%% `rabbit_mirror_queue_slave` up-to and including RabbitMQ 3.7.x. It is
+%% unused in 3.8.x and thus deprecated. We keep it to support in-place
+%% upgrades to 3.8.x (i.e. mixed-version clusters), but it is a no-op
+%% starting with that version.
 send_mandatory(#delivery{mandatory  = false}) ->
     ok;
 send_mandatory(#delivery{mandatory  = true,
@@ -557,7 +582,7 @@ send_or_record_confirm(published, #delivery { sender     = ChPid,
                                               message    = #basic_message {
                                                 id            = MsgId,
                                                 is_persistent = true } },
-                       MS, #state { q = #amqqueue { durable = true } }) ->
+                       MS, #state{q = Q}) when ?amqqueue_is_durable(Q) ->
     maps:put(MsgId, {published, ChPid, MsgSeqNo} , MS);
 send_or_record_confirm(_Status, #delivery { sender     = ChPid,
                                             confirm    = true,
@@ -602,7 +627,7 @@ handle_process_result({stop, State}) -> {stop, normal, State}.
 
 -spec promote_me({pid(), term()}, #state{}) -> no_return().
 
-promote_me(From, #state { q                   = Q = #amqqueue { name = QName },
+promote_me(From, #state { q                   = Q0,
                           gm                  = GM,
                           backing_queue       = BQ,
                           backing_queue_state = BQS,
@@ -610,13 +635,14 @@ promote_me(From, #state { q                   = Q = #amqqueue { name = QName },
                           sender_queues       = SQ,
                           msg_id_ack          = MA,
                           msg_id_status       = MS,
-                          known_senders       = KS }) ->
+                          known_senders       = KS}) when ?is_amqqueue(Q0) ->
+    QName = amqqueue:get_name(Q0),
     rabbit_mirror_queue_misc:log_info(QName, "Promoting slave ~s to master~n",
                                       [rabbit_misc:pid_to_string(self())]),
-    Q1 = Q #amqqueue { pid = self() },
-    {ok, CPid} = rabbit_mirror_queue_coordinator:start_link(
-                   Q1, GM, rabbit_mirror_queue_master:sender_death_fun(),
-                   rabbit_mirror_queue_master:depth_fun()),
+    Q1 = amqqueue:set_pid(Q0, self()),
+    DeathFun = rabbit_mirror_queue_master:sender_death_fun(),
+    DepthFun = rabbit_mirror_queue_master:depth_fun(),
+    {ok, CPid} = rabbit_mirror_queue_coordinator:start_link(Q1, GM, DeathFun, DepthFun),
     true = unlink(GM),
     gen_server2:reply(From, {promote, CPid}),
 
@@ -691,10 +717,10 @@ promote_me(From, #state { q                   = Q = #amqqueue { name = QName },
                     QName, CPid, BQ, BQS, GM, AckTags, SS, MPids),
 
     MTC = maps:fold(fun (MsgId, {published, ChPid, MsgSeqNo}, MTC0) ->
-                            gb_trees:insert(MsgId, {ChPid, MsgSeqNo}, MTC0);
+                            maps:put(MsgId, {ChPid, MsgSeqNo}, MTC0);
                         (_Msgid, _Status, MTC0) ->
                             MTC0
-                    end, gb_trees:empty(), MS),
+                    end, #{}, MS),
     Deliveries = [promote_delivery(Delivery) ||
                    {_ChPid, {PubQ, _PendCh, _ChState}} <- maps:to_list(SQ),
                    Delivery <- queue:to_list(PubQ)],
@@ -1050,19 +1076,22 @@ update_ram_duration(BQ, BQS) ->
         rabbit_memory_monitor:report_ram_duration(self(), RamDuration),
     BQ:set_ram_duration_target(DesiredDuration, BQS1).
 
-record_synchronised(#amqqueue { name = QName }) ->
+record_synchronised(Q0) when ?is_amqqueue(Q0) ->
+    QName = amqqueue:get_name(Q0),
     Self = self(),
-    case rabbit_misc:execute_mnesia_transaction(
-           fun () ->
-                   case mnesia:read({rabbit_queue, QName}) of
-                       [] ->
-                           ok;
-                       [Q1 = #amqqueue { sync_slave_pids = SSPids }] ->
-                           Q2 = Q1#amqqueue{sync_slave_pids = [Self | SSPids]},
-                           rabbit_mirror_queue_misc:store_updated_slaves(Q2),
-                           {ok, Q2}
-                   end
-           end) of
-        ok      -> ok;
-        {ok, Q} -> rabbit_mirror_queue_misc:maybe_drop_master_after_sync(Q)
+    F = fun () ->
+            case mnesia:read({rabbit_queue, QName}) of
+                [] ->
+                    ok;
+                [Q1] when ?is_amqqueue(Q1) ->
+                    SSPids = amqqueue:get_sync_slave_pids(Q1),
+                    SSPids1 = [Self | SSPids],
+                    Q2 = amqqueue:set_sync_slave_pids(Q1, SSPids1),
+                    rabbit_mirror_queue_misc:store_updated_slaves(Q2),
+                    {ok, Q2}
+            end
+        end,
+    case rabbit_misc:execute_mnesia_transaction(F) of
+        ok -> ok;
+        {ok, Q2} -> rabbit_mirror_queue_misc:maybe_drop_master_after_sync(Q2)
     end.
