@@ -19,30 +19,27 @@
 
 -export([names/1, diagnostics/1, make/1, parts/1, cookie_hash/0,
          is_running/2, is_process_running/2,
-         cluster_name/0, set_cluster_name/2, ensure_epmd/0,
+         cluster_name/0, set_cluster_name/1, set_cluster_name/2, ensure_epmd/0,
          all_running/0, name_type/0, running_count/0,
-         await_running_count/2]).
+         await_running_count/2,
+         boot/0]).
 
 -include_lib("kernel/include/inet.hrl").
+-include_lib("rabbit_common/include/rabbit.hrl").
 
 -define(SAMPLING_INTERVAL, 1000).
 
 %%----------------------------------------------------------------------------
-%% Specs
+%% API
 %%----------------------------------------------------------------------------
 
--spec names(string()) ->
-          rabbit_types:ok_or_error2([{string(), integer()}], term()).
--spec diagnostics([node()]) -> string().
--spec cookie_hash() -> string().
--spec is_running(node(), atom()) -> boolean().
--spec is_process_running(node(), atom()) -> boolean().
--spec cluster_name() -> binary().
--spec set_cluster_name(binary(), rabbit_types:username()) -> 'ok'.
--spec all_running() -> [node()].
--spec running_count() -> integer().
-
-%%----------------------------------------------------------------------------
+boot() ->
+  case application:get_env(rabbit, cluster_name) of
+    undefined  -> ok;
+    {ok, Name} ->
+      rabbit_log:info("Setting cluster name to '~s' as configured", [Name]),
+      set_cluster_name(rabbit_data_coercion:to_binary(Name))
+  end.
 
 name_type() ->
     case os:getenv("RABBITMQ_USE_LONGNAME") of
@@ -50,9 +47,14 @@ name_type() ->
         _      -> shortnames
     end.
 
+-spec names(string()) ->
+          rabbit_types:ok_or_error2([{string(), integer()}], term()).
+
 names(Hostname) ->
     rabbit_nodes_common:names(Hostname).
 
+-spec diagnostics([node()]) -> string().
+
 diagnostics(Nodes) ->
     rabbit_nodes_common:diagnostics(Nodes).
 
@@ -62,15 +64,23 @@ make(NodeStr) ->
 parts(NodeStr) ->
     rabbit_nodes_common:parts(NodeStr).
 
+-spec cookie_hash() -> string().
+
 cookie_hash() ->
     rabbit_nodes_common:cookie_hash().
 
+-spec is_running(node(), atom()) -> boolean().
+
 is_running(Node, Application) ->
     rabbit_nodes_common:is_running(Node, Application).
 
+-spec is_process_running(node(), atom()) -> boolean().
+
 is_process_running(Node, Process) ->
     rabbit_nodes_common:is_process_running(Node, Process).
 
+-spec cluster_name() -> binary().
+
 cluster_name() ->
     rabbit_runtime_parameters:value_global(
       cluster_name, cluster_name_default()).
@@ -80,6 +90,13 @@ cluster_name_default() ->
     FQDN = rabbit_net:hostname(),
     list_to_binary(atom_to_list(make({ID, FQDN}))).
 
+-spec set_cluster_name(binary()) -> 'ok'.
+
+set_cluster_name(Name) ->
+    set_cluster_name(Name, ?INTERNAL_USER).
+
+-spec set_cluster_name(binary(), rabbit_types:username()) -> 'ok'.
+
 set_cluster_name(Name, Username) ->
     %% Cluster name should be binary
     BinaryName = rabbit_data_coercion:to_binary(Name),
@@ -88,8 +105,12 @@ set_cluster_name(Name, Username) ->
 ensure_epmd() ->
     rabbit_nodes_common:ensure_epmd().
 
+-spec all_running() -> [node()].
+
 all_running() -> rabbit_mnesia:cluster_nodes(running).
 
+-spec running_count() -> integer().
+
 running_count() -> length(all_running()).
 
 -spec await_running_count(integer(), integer()) -> 'ok' | {'error', atom()}.
