@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2010-2015 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2010-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_prequeue).
@@ -29,7 +29,8 @@
 
 -behaviour(gen_server2).
 
--include("rabbit.hrl").
+-include_lib("rabbit_common/include/rabbit.hrl").
+-include("amqqueue.hrl").
 
 %%----------------------------------------------------------------------------
 
@@ -37,11 +38,11 @@
 
 -type start_mode() :: 'declare' | 'recovery' | 'slave'.
 
--spec start_link(rabbit_types:amqqueue(), start_mode(), pid())
-                      -> rabbit_types:ok_pid_or_error().
-
 %%----------------------------------------------------------------------------
 
+-spec start_link(amqqueue:amqqueue(), start_mode(), pid())
+                      -> rabbit_types:ok_pid_or_error().
+
 start_link(Q, StartMode, Marker) ->
     gen_server2:start_link(?MODULE, {Q, StartMode, Marker}, []).
 
@@ -57,20 +58,22 @@ init({Q, StartMode, Marker}) ->
 init(Q, master) -> rabbit_amqqueue_process:init(Q);
 init(Q, slave)  -> rabbit_mirror_queue_slave:init(Q);
 
-init(#amqqueue{name = QueueName}, restart) ->
-    {ok, Q = #amqqueue{pid        = QPid,
-                       slave_pids = SPids}} = rabbit_amqqueue:lookup(QueueName),
+init(Q0, restart) when ?is_amqqueue(Q0) ->
+    QueueName = amqqueue:get_name(Q0),
+    {ok, Q1} = rabbit_amqqueue:lookup(QueueName),
+    QPid = amqqueue:get_pid(Q1),
+    SPids = amqqueue:get_slave_pids(Q1),
     LocalOrMasterDown = node(QPid) =:= node()
         orelse not rabbit_mnesia:on_running_node(QPid),
     Slaves = [SPid || SPid <- SPids, rabbit_mnesia:is_process_alive(SPid)],
     case rabbit_mnesia:is_process_alive(QPid) of
         true  -> false = LocalOrMasterDown, %% assertion
                  rabbit_mirror_queue_slave:go(self(), async),
-                 rabbit_mirror_queue_slave:init(Q); %% [1]
+                 rabbit_mirror_queue_slave:init(Q1); %% [1]
         false -> case LocalOrMasterDown andalso Slaves =:= [] of
-                     true  -> crash_restart(Q);     %% [2]
+                     true  -> crash_restart(Q1);     %% [2]
                      false -> timer:sleep(25),
-                              init(Q, restart)      %% [3]
+                              init(Q1, restart)      %% [3]
                  end
     end.
 %% [1] There is a master on another node. Regardless of whether we
@@ -83,10 +86,12 @@ init(#amqqueue{name = QueueName}, restart) ->
 %%     not a stable situation. Sleep and wait for somebody else to make a
 %%     move.
 
-crash_restart(Q = #amqqueue{name = QueueName}) ->
+crash_restart(Q0) when ?is_amqqueue(Q0) ->
+    QueueName = amqqueue:get_name(Q0),
     rabbit_log:error("Restarting crashed ~s.~n", [rabbit_misc:rs(QueueName)]),
     gen_server2:cast(self(), init),
-    rabbit_amqqueue_process:init(Q#amqqueue{pid = self()}).
+    Q1 = amqqueue:set_pid(Q0, self()),
+    rabbit_amqqueue_process:init(Q1).
 
 %%----------------------------------------------------------------------------
 
@@ -102,4 +107,3 @@ handle_info(_Msg, _State)            -> exit(unreachable).
 terminate(_Reason, _State)           -> exit(unreachable).
 -spec code_change(_, _, _) -> no_return().
 code_change(_OldVsn, _State, _Extra) -> exit(unreachable).
-
