@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -11,12 +11,13 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_queue_master_location_misc).
 
--include("rabbit.hrl").
+-include_lib("rabbit_common/include/rabbit.hrl").
+-include("amqqueue.hrl").
 
 -export([lookup_master/2,
          lookup_queue/2,
@@ -28,22 +29,25 @@
 
 lookup_master(QueueNameBin, VHostPath) when is_binary(QueueNameBin),
                                             is_binary(VHostPath) ->
-    Queue = rabbit_misc:r(VHostPath, queue, QueueNameBin),
-    case rabbit_amqqueue:lookup(Queue) of
-        {ok, #amqqueue{pid = Pid}} when is_pid(Pid) ->
+    QueueR = rabbit_misc:r(VHostPath, queue, QueueNameBin),
+    case rabbit_amqqueue:lookup(QueueR) of
+        {ok, Queue} when ?amqqueue_has_valid_pid(Queue) ->
+            Pid = amqqueue:get_pid(Queue),
             {ok, node(Pid)};
         Error -> Error
     end.
 
 lookup_queue(QueueNameBin, VHostPath) when is_binary(QueueNameBin),
                                            is_binary(VHostPath) ->
-    Queue = rabbit_misc:r(VHostPath, queue, QueueNameBin),
-    case rabbit_amqqueue:lookup(Queue) of
-        Reply = {ok, #amqqueue{}} -> Reply;
-        Error                     -> Error
+    QueueR = rabbit_misc:r(VHostPath, queue, QueueNameBin),
+    case rabbit_amqqueue:lookup(QueueR) of
+        Reply = {ok, Queue} when ?is_amqqueue(Queue) ->
+            Reply;
+        Error ->
+            Error
     end.
 
-get_location(Queue=#amqqueue{})->
+get_location(Queue) when ?is_amqqueue(Queue) ->
     Reply1 = case get_location_mod_by_args(Queue) of
                  _Err1 = {error, _} ->
                      case get_location_mod_by_policy(Queue) of
@@ -62,7 +66,8 @@ get_location(Queue=#amqqueue{})->
         Error    -> Error
     end.
 
-get_location_mod_by_args(#amqqueue{arguments=Args}) ->
+get_location_mod_by_args(Queue) when ?is_amqqueue(Queue) ->
+    Args = amqqueue:get_arguments(Queue),
     case rabbit_misc:table_lookup(Args, <<"x-queue-master-locator">>) of
         {_Type, Strategy}  ->
             case rabbit_queue_location_validator:validate_strategy(Strategy) of
@@ -72,7 +77,7 @@ get_location_mod_by_args(#amqqueue{arguments=Args}) ->
         _ -> {error, "x-queue-master-locator undefined"}
     end.
 
-get_location_mod_by_policy(Queue=#amqqueue{}) ->
+get_location_mod_by_policy(Queue) when ?is_amqqueue(Queue) ->
     case rabbit_policy:get(<<"queue-master-locator">> , Queue) of
         undefined ->  {error, "queue-master-locator policy undefined"};
         Strategy  ->
@@ -82,7 +87,7 @@ get_location_mod_by_policy(Queue=#amqqueue{}) ->
             end
     end.
 
-get_location_mod_by_config(#amqqueue{}) ->
+get_location_mod_by_config(Queue) when ?is_amqqueue(Queue) ->
     case application:get_env(rabbit, queue_master_locator) of
         {ok, Strategy} ->
             case rabbit_queue_location_validator:validate_strategy(Strategy) of
@@ -92,7 +97,7 @@ get_location_mod_by_config(#amqqueue{}) ->
         _ -> {error, "queue_master_locator undefined"}
     end.
 
-all_nodes(Queue = #amqqueue{}) ->
+all_nodes(Queue) when ?is_amqqueue(Queue) ->
     handle_is_mirrored_ha_nodes(rabbit_mirror_queue_misc:is_mirrored_ha_nodes(Queue), Queue).
 
 handle_is_mirrored_ha_nodes(false, _Queue) ->
