@@ -32,11 +32,8 @@
 -dialyzer(no_missing_calls).
 -ignore_xref([{ssl_cipher_format, suite_legacy, 1},
               {ssl_cipher_format, suite, 1},
-              {ssl_cipher, suite, 1},
               {ssl_cipher_format, suite_to_str, 1},
-              {ssl_cipher, erl_suite_definition, 1},
               {ssl_cipher_format, erl_suite_definition, 1},
-              {ssl_cipher, openssl_suite_name, 1},
               {ssl_cipher_format, suite_map_to_openssl_str, 1},
               {ssl_cipher_format, suite_map_to_bin, 1}]).
 
@@ -81,19 +78,13 @@ cipher_suites_openssl(Mode, Version) ->
     end,
     ssl:cipher_suites(Mode, Version)).
 
-%% OTP-20.3 and OTP-21 have different modules containing cipher format functions
-%% This is not on the hot code path and `function_exported` should not slow things down much.
+
 format_cipher_erlang(Cipher) ->
   case erlang:function_exported(ssl_cipher_format, suite_map_to_bin, 1) of
       true ->
-        format_cipher_erlang22(Cipher);
+          format_cipher_erlang22(Cipher);
       false ->
-        case erlang:function_exported(ssl_cipher_format, erl_suite_definition, 1) of
-          true ->
-            format_cipher_erlang21(Cipher);
-          false ->
-            format_cipher_erlang20(Cipher)
-        end
+          format_cipher_erlang21(Cipher)
   end.
 
 format_cipher_erlang22(Cipher) ->
@@ -102,22 +93,13 @@ format_cipher_erlang22(Cipher) ->
 format_cipher_erlang21(Cipher) ->
   ssl_cipher_format:erl_suite_definition(ssl_cipher_format:suite(Cipher)).
 
-format_cipher_erlang20(Cipher) ->
-  ssl_cipher:erl_suite_definition(ssl_cipher:suite(Cipher)).
-
 
 format_cipher_openssl(Cipher) ->
-    case erlang:function_exported(ssl_cipher_format, suite_map_to_openssl_str, 1) of
+    case erlang:function_exported(ssl_cipher_format, suite_map_to_bin, 1) of
       true ->
         format_cipher_openssl22(Cipher);
       false ->
-
-        case erlang:function_exported(ssl_cipher_format, suite_to_str, 1) of
-            true ->
-                format_cipher_openssl21(Cipher);
-            false ->
-                format_cipher_openssl20(Cipher)
-        end
+        format_cipher_openssl21(Cipher)
     end.
 
 format_cipher_openssl22(Cipher) ->
@@ -126,10 +108,6 @@ format_cipher_openssl22(Cipher) ->
 format_cipher_openssl21(Cipher) ->
     ssl_cipher_format:suite_to_str(Cipher).
 
-format_cipher_openssl20(Cipher) ->
-    ssl_cipher:openssl_suite_name(ssl_cipher:suite(Cipher)).
-
-
 -spec get_highest_protocol_version() -> tls_record:tls_version().
 get_highest_protocol_version() ->
     tls_record:highest_protocol_version([]).
