@@ -1,5 +1,5 @@
 %% Copyright (c) 2011 Basho Technologies, Inc.  All Rights Reserved.
-%% Copyright (c) 2018 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2018-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 %% This file is provided to you under the Apache License,
 %% Version 2.0 (the "License"); you may not use this file
@@ -25,10 +25,6 @@
 
 -behaviour(gen_event).
 
-%% Transitional step until we can require Erlang/OTP 21 and
-%% use the now recommended try/catch syntax for obtaining the stack trace.
--compile(nowarn_deprecated_function).
-
 %% API
 -export([add_handler/0]).
 
@@ -179,11 +175,9 @@ format_pretty_proc_or_port_info(PidOrPort) ->
             Res ->
                 Res
         end
-    catch C:E ->
-        S = erlang:get_stacktrace(),
-        Fmt = "Pid ~w, ~W ~W at ~w\n",
-        Args = [PidOrPort, C, 20, E, 20, S],
-        {Fmt, Args}
+    catch C:E:S ->
+        {"Pid ~w, ~W ~W at ~w\n",
+            [PidOrPort, C, 20, E, 20, S]}
     end.
 
 get_pretty_proc_or_port_info(Pid) when is_pid(Pid) ->
