@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_upgrade_functions).
@@ -64,48 +64,12 @@
 
 %% -------------------------------------------------------------------
 
--spec remove_user_scope() -> 'ok'.
--spec hash_passwords() -> 'ok'.
--spec add_ip_to_listener() -> 'ok'.
--spec add_opts_to_listener() -> 'ok'.
--spec internal_exchanges() -> 'ok'.
--spec user_to_internal_user() -> 'ok'.
--spec topic_trie() -> 'ok'.
--spec semi_durable_route() -> 'ok'.
--spec exchange_event_serial() -> 'ok'.
--spec trace_exchanges() -> 'ok'.
--spec user_admin_to_tags() -> 'ok'.
--spec ha_mirrors() -> 'ok'.
--spec gm() -> 'ok'.
--spec exchange_scratch() -> 'ok'.
--spec mirrored_supervisor() -> 'ok'.
--spec topic_trie_node() -> 'ok'.
--spec runtime_parameters() -> 'ok'.
--spec policy() -> 'ok'.
--spec sync_slave_pids() -> 'ok'.
--spec no_mirror_nodes() -> 'ok'.
--spec gm_pids() -> 'ok'.
--spec exchange_decorators() -> 'ok'.
--spec policy_apply_to() -> 'ok'.
--spec queue_decorators() -> 'ok'.
--spec internal_system_x() -> 'ok'.
--spec cluster_name() -> 'ok'.
--spec down_slave_nodes() -> 'ok'.
--spec queue_state() -> 'ok'.
--spec recoverable_slaves() -> 'ok'.
--spec user_password_hashing() -> 'ok'.
--spec vhost_limits() -> 'ok'.
--spec operator_policies() -> 'ok'.
--spec queue_vhost_field() -> 'ok'.
--spec queue_options() -> 'ok'.
--spec exchange_options() -> 'ok'.
-
-
-%%--------------------------------------------------------------------
-
 %% replaces vhost.dummy (used to avoid having a single-field record
 %% which Mnesia doesn't like) with vhost.limits (which is actually
 %% used)
+
+-spec vhost_limits() -> 'ok'.
+
 vhost_limits() ->
     transform(
       rabbit_vhost,
@@ -120,6 +84,8 @@ vhost_limits() ->
 %% would be messy to have to go back and fix old transforms at that
 %% point.
 
+-spec remove_user_scope() -> 'ok'.
+
 remove_user_scope() ->
     transform(
       rabbit_user_permission,
@@ -132,6 +98,9 @@ remove_user_scope() ->
 %% only relevant to those migrating from 2.1.1.
 %% all users created after in 3.6.0 or later will use SHA-256 (unless configured
 %% otherwise)
+
+-spec hash_passwords() -> 'ok'.
+
 hash_passwords() ->
     transform(
       rabbit_user,
@@ -141,6 +110,8 @@ hash_passwords() ->
       end,
       [username, password_hash, is_admin]).
 
+-spec add_ip_to_listener() -> 'ok'.
+
 add_ip_to_listener() ->
     transform(
       rabbit_listener,
@@ -149,6 +120,8 @@ add_ip_to_listener() ->
       end,
       [node, protocol, host, ip_address, port]).
 
+-spec add_opts_to_listener() -> 'ok'.
+
 add_opts_to_listener() ->
     transform(
       rabbit_listener,
@@ -157,6 +130,8 @@ add_opts_to_listener() ->
       end,
       [node, protocol, host, ip_address, port, opts]).
 
+-spec internal_exchanges() -> 'ok'.
+
 internal_exchanges() ->
     Tables = [rabbit_exchange, rabbit_durable_exchange],
     AddInternalFun =
@@ -169,6 +144,8 @@ internal_exchanges() ->
       || T <- Tables ],
     ok.
 
+-spec user_to_internal_user() -> 'ok'.
+
 user_to_internal_user() ->
     transform(
       rabbit_user,
@@ -177,6 +154,8 @@ user_to_internal_user() ->
       end,
       [username, password_hash, is_admin], internal_user).
 
+-spec topic_trie() -> 'ok'.
+
 topic_trie() ->
     create(rabbit_topic_trie_edge, [{record_name, topic_trie_edge},
                                     {attributes, [trie_edge, node_id]},
@@ -185,20 +164,28 @@ topic_trie() ->
                                        {attributes, [trie_binding, value]},
                                        {type, ordered_set}]).
 
+-spec semi_durable_route() -> 'ok'.
+
 semi_durable_route() ->
     create(rabbit_semi_durable_route, [{record_name, route},
                                        {attributes, [binding, value]}]).
 
+-spec exchange_event_serial() -> 'ok'.
+
 exchange_event_serial() ->
     create(rabbit_exchange_serial, [{record_name, exchange_serial},
                                     {attributes, [name, next]}]).
 
+-spec trace_exchanges() -> 'ok'.
+
 trace_exchanges() ->
     [declare_exchange(
        rabbit_misc:r(VHost, exchange, <<"amq.rabbitmq.trace">>), topic) ||
-        VHost <- rabbit_vhost:list()],
+        VHost <- rabbit_vhost:list_names()],
     ok.
 
+-spec user_admin_to_tags() -> 'ok'.
+
 user_admin_to_tags() ->
     transform(
       rabbit_user,
@@ -209,6 +196,8 @@ user_admin_to_tags() ->
       end,
       [username, password_hash, tags], internal_user).
 
+-spec ha_mirrors() -> 'ok'.
+
 ha_mirrors() ->
     Tables = [rabbit_queue, rabbit_durable_queue],
     AddMirrorPidsFun =
@@ -223,10 +212,14 @@ ha_mirrors() ->
       || T <- Tables ],
     ok.
 
+-spec gm() -> 'ok'.
+
 gm() ->
     create(gm_group, [{record_name, gm_group},
                       {attributes, [name, version, members]}]).
 
+-spec exchange_scratch() -> 'ok'.
+
 exchange_scratch() ->
     ok = exchange_scratch(rabbit_exchange),
     ok = exchange_scratch(rabbit_durable_exchange).
@@ -239,17 +232,23 @@ exchange_scratch(Table) ->
       end,
       [name, type, durable, auto_delete, internal, arguments, scratch]).
 
+-spec mirrored_supervisor() -> 'ok'.
+
 mirrored_supervisor() ->
     create(mirrored_sup_childspec,
            [{record_name, mirrored_sup_childspec},
             {attributes, [key, mirroring_pid, childspec]}]).
 
+-spec topic_trie_node() -> 'ok'.
+
 topic_trie_node() ->
     create(rabbit_topic_trie_node,
            [{record_name, topic_trie_node},
             {attributes, [trie_node, edge_count, binding_count]},
             {type, ordered_set}]).
 
+-spec runtime_parameters() -> 'ok'.
+
 runtime_parameters() ->
     create(rabbit_runtime_parameters,
            [{record_name, runtime_parameters},
@@ -273,6 +272,8 @@ exchange_scratches(Table) ->
       end,
       [name, type, durable, auto_delete, internal, arguments, scratches]).
 
+-spec policy() -> 'ok'.
+
 policy() ->
     ok = exchange_policy(rabbit_exchange),
     ok = exchange_policy(rabbit_durable_exchange),
@@ -299,6 +300,8 @@ queue_policy(Table) ->
       [name, durable, auto_delete, exclusive_owner, arguments, pid,
        slave_pids, mirror_nodes, policy]).
 
+-spec sync_slave_pids() -> 'ok'.
+
 sync_slave_pids() ->
     Tables = [rabbit_queue, rabbit_durable_queue],
     AddSyncSlavesFun =
@@ -311,6 +314,8 @@ sync_slave_pids() ->
      || T <- Tables],
     ok.
 
+-spec no_mirror_nodes() -> 'ok'.
+
 no_mirror_nodes() ->
     Tables = [rabbit_queue, rabbit_durable_queue],
     RemoveMirrorNodesFun =
@@ -323,6 +328,8 @@ no_mirror_nodes() ->
      || T <- Tables],
     ok.
 
+-spec gm_pids() -> 'ok'.
+
 gm_pids() ->
     Tables = [rabbit_queue, rabbit_durable_queue],
     AddGMPidsFun =
@@ -335,6 +342,8 @@ gm_pids() ->
      || T <- Tables],
     ok.
 
+-spec exchange_decorators() -> 'ok'.
+
 exchange_decorators() ->
     ok = exchange_decorators(rabbit_exchange),
     ok = exchange_decorators(rabbit_durable_exchange).
@@ -350,6 +359,8 @@ exchange_decorators(Table) ->
       [name, type, durable, auto_delete, internal, arguments, scratches, policy,
        decorators]).
 
+-spec policy_apply_to() -> 'ok'.
+
 policy_apply_to() ->
     transform(
       rabbit_runtime_parameters,
@@ -372,6 +383,8 @@ apply_to(Def) ->
         [_,         _]         -> <<"all">>
     end.
 
+-spec queue_decorators() -> 'ok'.
+
 queue_decorators() ->
     ok = queue_decorators(rabbit_queue),
     ok = queue_decorators(rabbit_durable_queue).
@@ -387,6 +400,8 @@ queue_decorators(Table) ->
       [name, durable, auto_delete, exclusive_owner, arguments, pid, slave_pids,
        sync_slave_pids, policy, gm_pids, decorators]).
 
+-spec internal_system_x() -> 'ok'.
+
 internal_system_x() ->
     transform(
       rabbit_durable_exchange,
@@ -400,6 +415,8 @@ internal_system_x() ->
       [name, type, durable, auto_delete, internal, arguments, scratches, policy,
        decorators]).
 
+-spec cluster_name() -> 'ok'.
+
 cluster_name() ->
     {atomic, ok} = mnesia:transaction(fun cluster_name_tx/0),
     ok.
@@ -426,6 +443,8 @@ cluster_name_tx() ->
     [mnesia:delete(T, K, write) || K <- Ks],
     ok.
 
+-spec down_slave_nodes() -> 'ok'.
+
 down_slave_nodes() ->
     ok = down_slave_nodes(rabbit_queue),
     ok = down_slave_nodes(rabbit_durable_queue).
@@ -441,6 +460,8 @@ down_slave_nodes(Table) ->
       [name, durable, auto_delete, exclusive_owner, arguments, pid, slave_pids,
        sync_slave_pids, down_slave_nodes, policy, gm_pids, decorators]).
 
+-spec queue_state() -> 'ok'.
+
 queue_state() ->
     ok = queue_state(rabbit_queue),
     ok = queue_state(rabbit_durable_queue).
@@ -457,6 +478,8 @@ queue_state(Table) ->
       [name, durable, auto_delete, exclusive_owner, arguments, pid, slave_pids,
        sync_slave_pids, down_slave_nodes, policy, gm_pids, decorators, state]).
 
+-spec recoverable_slaves() -> 'ok'.
+
 recoverable_slaves() ->
     ok = recoverable_slaves(rabbit_queue),
     ok = recoverable_slaves(rabbit_durable_queue).
@@ -504,6 +527,8 @@ slave_pids_pending_shutdown(Table) ->
        sync_slave_pids, recoverable_slaves, policy, gm_pids, decorators, state,
        policy_version, slave_pids_pending_shutdown]).
 
+-spec operator_policies() -> 'ok'.
+
 operator_policies() ->
     ok = exchange_operator_policies(rabbit_exchange),
     ok = exchange_operator_policies(rabbit_durable_exchange),
@@ -535,6 +560,7 @@ queue_operator_policies(Table) ->
        sync_slave_pids, recoverable_slaves, policy, operator_policy,
        gm_pids, decorators, state, policy_version, slave_pids_pending_shutdown]).
 
+-spec queue_vhost_field() -> 'ok'.
 
 queue_vhost_field() ->
     ok = queue_vhost_field(rabbit_queue),
@@ -557,6 +583,8 @@ queue_vhost_field(Table) ->
        sync_slave_pids, recoverable_slaves, policy, operator_policy,
        gm_pids, decorators, state, policy_version, slave_pids_pending_shutdown, vhost]).
 
+-spec queue_options() -> 'ok'.
+
 queue_options() ->
     ok = queue_options(rabbit_queue),
     ok = queue_options(rabbit_durable_queue),
@@ -580,6 +608,9 @@ queue_options(Table) ->
 %% existing records with said default.  Users created with 3.6.0+ will
 %% have internal_user.hashing_algorithm populated by the internal
 %% authn backend.
+
+-spec user_password_hashing() -> 'ok'.
+
 user_password_hashing() ->
     transform(
       rabbit_user,
@@ -594,6 +625,8 @@ topic_permission() ->
          {attributes, [topic_permission_key, permission]},
          {disc_copies, [node()]}]).
 
+-spec exchange_options() -> 'ok'.
+
 exchange_options() ->
     ok = exchange_options(rabbit_exchange),
     ok = exchange_options(rabbit_durable_exchange).
