@@ -21,8 +21,9 @@
 -behaviour(rabbit_authz_backend).
 
 -export([user/0]).
--export([user_login_authentication/2, user_login_authorization/1,
-         check_vhost_access/3, check_resource_access/3, check_topic_access/4]).
+-export([user_login_authentication/2, user_login_authorization/2,
+         check_vhost_access/3, check_resource_access/4, check_topic_access/4]).
+-export([state_can_expire/0]).
 
 -spec user() -> rabbit_types:user().
 
@@ -37,9 +38,11 @@ user() -> #user{username       = <<"none">>,
 user_login_authentication(_, _) ->
     {refused, "cannot log in conventionally as dummy user", []}.
 
-user_login_authorization(_) ->
+user_login_authorization(_, _) ->
     {refused, "cannot log in conventionally as dummy user", []}.
 
-check_vhost_access(#auth_user{}, _VHostPath, _Sock) -> true.
-check_resource_access(#auth_user{}, #resource{}, _Permission) -> true.
+check_vhost_access(#auth_user{}, _VHostPath, _AuthzData) -> true.
+check_resource_access(#auth_user{}, #resource{}, _Permission, _Context) -> true.
 check_topic_access(#auth_user{}, #resource{}, _Permission, _Context) -> true.
+
+state_can_expire() -> false.
