@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -83,6 +83,9 @@ build_content_frames(FragsRev, FrameMax, ChannelInt) ->
 build_content_frames(SizeAcc, FramesAcc, _FragSizeRem, [],
                      [], _BodyPayloadMax, _ChannelInt) ->
     {SizeAcc, lists:reverse(FramesAcc)};
+build_content_frames(SizeAcc, FramesAcc, _FragSizeRem, [],
+                     [<<>>], _BodyPayloadMax, _ChannelInt) ->
+    {SizeAcc, lists:reverse(FramesAcc)};
 build_content_frames(SizeAcc, FramesAcc, FragSizeRem, FragAcc,
                      Frags, BodyPayloadMax, ChannelInt)
   when FragSizeRem == 0 orelse Frags == [] ->
