@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_core_metrics).
@@ -35,7 +35,8 @@
          channel_queue_exchange_down/1,
          channel_exchange_down/1]).
 
--export([consumer_created/7,
+-export([consumer_created/9,
+         consumer_updated/9,
          consumer_deleted/3]).
 
 -export([queue_stats/2,
@@ -66,6 +67,8 @@
 
 -type(channel_stats_type() :: queue_exchange_stats | queue_stats |
 			      exchange_stats | reductions).
+
+-type(activity_status() :: up | single_active | waiting | suspected_down).
 %%----------------------------------------------------------------------------
 %% Specs
 %%----------------------------------------------------------------------------
@@ -84,7 +87,9 @@
                                    rabbit_exchange:name()}}) -> ok.
 -spec channel_exchange_down({pid(), rabbit_exchange:name()}) -> ok.
 -spec consumer_created(pid(), binary(), boolean(), boolean(),
-                       rabbit_amqqueue:name(), integer(), list()) -> ok.
+                       rabbit_amqqueue:name(), integer(), boolean(), activity_status(), list()) -> ok.
+-spec consumer_updated(pid(), binary(), boolean(), boolean(),
+                       rabbit_amqqueue:name(), integer(), boolean(), activity_status(), list()) -> ok.
 -spec consumer_deleted(pid(), binary(), rabbit_amqqueue:name()) -> ok.
 -spec queue_stats(rabbit_amqqueue:name(), rabbit_types:infos()) -> ok.
 -spec queue_stats(rabbit_amqqueue:name(), integer(), integer(), integer(),
@@ -161,15 +166,19 @@ channel_stats(reductions, Id, Value) ->
 
 channel_stats(exchange_stats, publish, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_exchange_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(exchange_stats, confirm, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_exchange_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(exchange_stats, return_unroutable, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_exchange_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0, 0}),
+    ok;
+channel_stats(exchange_stats, drop_unroutable, Id, Value) ->
+    %% Includes delete marker
+    _ = ets:update_counter(channel_exchange_metrics, Id, {5, Value}, {Id, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_exchange_stats, publish, Id, Value) ->
     %% Includes delete marker
@@ -177,27 +186,31 @@ channel_stats(queue_exchange_stats, publish, Id, Value) ->
     ok;
 channel_stats(queue_stats, get, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, get_no_ack, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, deliver, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, deliver_no_ack, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {5, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {5, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, redeliver, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {6, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {6, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, ack, Id, Value) ->
     %% Includes delete marker
-    _ = ets:update_counter(channel_queue_metrics, Id, {7, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {7, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
+    ok;
+channel_stats(queue_stats, get_empty, Id, Value) ->
+    %% Includes delete marker
+    _ = ets:update_counter(channel_queue_metrics, Id, {8, Value}, {Id, 0, 0, 0, 0, 0, 0, 0, 0}),
     ok.
 
 delete(Table, Key) ->
@@ -206,7 +219,7 @@ delete(Table, Key) ->
 
 channel_queue_down(Id) ->
     %% Delete marker
-    ets:update_element(channel_queue_metrics, Id, {8, 1}),
+    ets:update_element(channel_queue_metrics, Id, {9, 1}),
     ok.
 
 channel_queue_exchange_down(Id) ->
@@ -216,13 +229,19 @@ channel_queue_exchange_down(Id) ->
 
 channel_exchange_down(Id) ->
     %% Delete marker
-    ets:update_element(channel_exchange_metrics, Id, {5, 1}),
+    ets:update_element(channel_exchange_metrics, Id, {6, 1}),
     ok.
 
 consumer_created(ChPid, ConsumerTag, ExclusiveConsume, AckRequired, QName,
-                 PrefetchCount, Args) ->
+                 PrefetchCount, Active, ActivityStatus, Args) ->
     ets:insert(consumer_created, {{QName, ChPid, ConsumerTag}, ExclusiveConsume,
-                                   AckRequired, PrefetchCount, Args}),
+                                   AckRequired, PrefetchCount, Active, ActivityStatus, Args}),
+    ok.
+
+consumer_updated(ChPid, ConsumerTag, ExclusiveConsume, AckRequired, QName,
+                 PrefetchCount, Active, ActivityStatus, Args) ->
+    ets:insert(consumer_created, {{QName, ChPid, ConsumerTag}, ExclusiveConsume,
+                                   AckRequired, PrefetchCount, Active, ActivityStatus, Args}),
     ok.
 
 consumer_deleted(ChPid, ConsumerTag, QName) ->
@@ -263,7 +282,7 @@ queue_deleted(Name) ->
                   end, CQX),
     CQ = ets:select(channel_queue_metrics, match_spec_cq(Name)),
     lists:foreach(fun(Key) ->
-                          ets:update_element(channel_queue_metrics, Key, {8, 1})
+                          ets:update_element(channel_queue_metrics, Key, {9, 1})
                   end, CQ).
 
 queues_deleted(Queues) ->
@@ -310,14 +329,14 @@ delete_channel_queue_metrics(MatchSpecCondition) ->
         channel_queue_metrics,
         [
             {
-                {{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_'},
+                {{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_', '_'},
                 [MatchSpecCondition],
                 [{{'$2', '$1'}}]
             }
         ]
     ),
     lists:foreach(fun(Key) ->
-        ets:update_element(channel_queue_metrics, Key, {8, 1})
+        ets:update_element(channel_queue_metrics, Key, {9, 1})
     end, ChannelQueueMetricsToUpdate).
 
 % [{'orelse',
@@ -357,7 +376,7 @@ match_spec_cqx(Id) ->
     [{{{'$2', {'$1', '$3'}}, '_', '_'}, [{'==', {Id}, '$1'}], [{{'$2', {{'$1', '$3'}}}}]}].
 
 match_spec_cq(Id) ->
-    [{{{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_'}, [{'==', {Id}, '$1'}], [{{'$2', '$1'}}]}].
+    [{{{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_', '_'}, [{'==', {Id}, '$1'}], [{{'$2', '$1'}}]}].
 
 gen_server2_stats(Pid, BufferLength) ->
     ets:insert(gen_server2_metrics, {Pid, BufferLength}),
