@@ -31,7 +31,7 @@
 -include("rabbit_log.hrl").
 %%----------------------------------------------------------------------------
 
--type category() :: connection | channel | mirroring | queue | federation | upgrade.
+-type category() :: connection | channel | ldap | mirroring | queue | federation | upgrade | ra.
 
 -spec debug(string()) -> 'ok'.
 -spec debug(string(), [any()]) -> 'ok'.
@@ -82,6 +82,7 @@ make_internal_sink_name(queue)      -> rabbit_log_queue_lager_event;
 make_internal_sink_name(federation) -> rabbit_log_federation_lager_event;
 make_internal_sink_name(shovel)     -> rabbit_log_shovel_lager_event;
 make_internal_sink_name(upgrade)    -> rabbit_log_upgrade_lager_event;
+make_internal_sink_name(ra)         -> rabbit_log_ra_lager_event;
 make_internal_sink_name(default)    -> rabbit_log_lager_event;
 make_internal_sink_name(Category)   -> erlang:error({unknown_category, Category}).
 
