@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_queue_collector).
@@ -33,14 +33,12 @@
 %%----------------------------------------------------------------------------
 
 -spec start_link(rabbit_types:proc_name()) -> rabbit_types:ok_pid_or_error().
--spec register(pid(), pid()) -> 'ok'.
--spec delete_all(pid()) -> 'ok'.
-
-%%----------------------------------------------------------------------------
 
 start_link(ProcName) ->
     gen_server:start_link(?MODULE, [ProcName], []).
 
+-spec register(pid(), pid()) -> 'ok'.
+
 register(CollectorPid, Q) ->
     gen_server:call(CollectorPid, {register, Q}, infinity).
 
@@ -50,6 +48,7 @@ delete_all(CollectorPid) ->
 %%----------------------------------------------------------------------------
 
 init([ProcName]) ->
+    ?LG_PROCESS_TYPE(queue_collector),
     ?store_proc_name(ProcName),
     {ok, #state{monitors = pmon:new(), delete_from = undefined}}.
 
@@ -59,7 +58,7 @@ handle_call({register, QPid}, _From,
             State = #state{monitors = QMons, delete_from = Deleting}) ->
     case Deleting of
         undefined -> ok;
-        _         -> ok = rabbit_amqqueue:delete_immediately([QPid])
+        _         -> ok = rabbit_amqqueue_common:delete_exclusive([QPid], Deleting)
     end,
     {reply, ok, State#state{monitors = pmon:monitor(QPid, QMons)}};
 
@@ -67,7 +66,7 @@ handle_call(delete_all, From, State = #state{monitors    = QMons,
                                              delete_from = undefined}) ->
     case pmon:monitored(QMons) of
         []    -> {reply, ok, State#state{delete_from = From}};
-        QPids -> ok = rabbit_amqqueue:delete_immediately(QPids),
+        QPids -> ok = rabbit_amqqueue_common:delete_exclusive(QPids, From),
                  {noreply, State#state{delete_from = From}}
     end.
 
