@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 -module('Elixir.RabbitMQ.CLI.Ctl.Commands.ListAmqp10ConnectionsCommand').
 
@@ -33,9 +33,7 @@
          description/0]).
 
 formatter() -> 'Elixir.RabbitMQ.CLI.Formatters.Table'.
-
 scopes() -> [ctl, diagnostics].
-
 switches() -> [{verbose, boolean}].
 aliases() -> [{'V', verbose}].
 
@@ -61,15 +59,14 @@ usage_additional() ->
       {<<"<column>">>, <<Prefix/binary, InfoItems/binary>>}
     ].
 
-description() ->
-    <<"Lists AMQP 1.0 connections">>.
+description() -> <<"Lists AMQP 1.0 connections on the target node">>.
 
 help_section() ->
     {plugin, 'amqp1.0'}.
 
 run(Args, #{node := NodeName,
-                    timeout := Timeout,
-                    verbose := Verbose}) ->
+            timeout := Timeout,
+            verbose := Verbose}) ->
     InfoKeys = case Verbose of
         true  -> ?INFO_ITEMS;
         false -> 'Elixir.RabbitMQ.CLI.Ctl.InfoKeys':prepare_info_keys(Args)
