@@ -23,9 +23,14 @@
 
 -define(EXCHANGE_SUB_LIFETIME, "delete-on-close").
 -define(DEFAULT_OUTCOME, #'v1_0.released'{}).
+-define(SUPPORTED_OUTCOMES, [?V_1_0_SYMBOL_ACCEPTED,
+                             ?V_1_0_SYMBOL_REJECTED,
+                             ?V_1_0_SYMBOL_RELEASED]).
+
 -define(OUTCOMES, [?V_1_0_SYMBOL_ACCEPTED,
                    ?V_1_0_SYMBOL_REJECTED,
-                   ?V_1_0_SYMBOL_RELEASED]).
+                   ?V_1_0_SYMBOL_RELEASED,
+                   ?V_1_0_SYMBOL_MODIFIED]).
 
 outcomes(Source) ->
     {DefaultOutcome, Outcomes} =
@@ -39,8 +44,8 @@ outcomes(Source) ->
                           _         -> DO
                       end,
                 Os1 = case Os of
-                          undefined    -> ?OUTCOMES;
-                          {list, Syms} -> Syms;
+                          undefined    -> ?SUPPORTED_OUTCOMES;
+                          {array, symbol, Syms} -> Syms;
                           Bad1         -> rabbit_amqp1_0_util:protocol_error(
                                             ?V_1_0_AMQP_ERROR_NOT_IMPLEMENTED,
                                             "Outcomes not supported: ~p",
@@ -48,10 +53,10 @@ outcomes(Source) ->
                       end,
                 {DO1, Os1};
             _ ->
-                {?DEFAULT_OUTCOME, ?OUTCOMES}
+                {?DEFAULT_OUTCOME, ?SUPPORTED_OUTCOMES}
         end,
     case [O || O <- Outcomes, not lists:member(O, ?OUTCOMES)] of
-        []  -> {DefaultOutcome, {array, symbol, [X || {symbol, X} <- Outcomes]}};
+        []  -> {DefaultOutcome, {array, symbol, Outcomes}};
         Bad -> rabbit_amqp1_0_util:protocol_error(
                  ?V_1_0_AMQP_ERROR_NOT_IMPLEMENTED,
                  "Outcomes not supported: ~p", [Bad])
