@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_auth_backend_cache).
@@ -20,10 +20,11 @@
 -behaviour(rabbit_authn_backend).
 -behaviour(rabbit_authz_backend).
 
--export([user_login_authentication/2, user_login_authorization/1,
-         check_vhost_access/3, check_resource_access/3, check_topic_access/4]).
+-export([user_login_authentication/2, user_login_authorization/2,
+         check_vhost_access/3, check_resource_access/4, check_topic_access/4,
+         state_can_expire/0]).
 
-%% Implementation of rabbit_auth_backend
+%% API
 
 user_login_authentication(Username, AuthProps) ->
     with_cache(authn, {user_login_authentication, [Username, AuthProps]},
@@ -33,8 +34,8 @@ user_login_authentication(Username, AuthProps) ->
            (_)                -> unknown
         end).
 
-user_login_authorization(Username) ->
-    with_cache(authz, {user_login_authorization, [Username]},
+user_login_authorization(Username, AuthProps) ->
+    with_cache(authz, {user_login_authorization, [Username, AuthProps]},
         fun({ok, _})      -> success;
            ({ok, _, _})   -> success;
            ({refused, _, _})  -> refusal;
@@ -42,11 +43,8 @@ user_login_authorization(Username) ->
            (_)                -> unknown
         end).
 
-check_vhost_access(#auth_user{} = AuthUser, VHostPath, Sock) ->
-    ClientAddress = extract_address(Sock),
-    % the underlying backend uses the socket, but only
-    % the client address is used in the key of the cache entry
-    with_cache(authz, {check_vhost_access, [AuthUser, VHostPath, Sock], [AuthUser, VHostPath, ClientAddress]},
+check_vhost_access(#auth_user{} = AuthUser, VHostPath, AuthzData) ->
+    with_cache(authz, {check_vhost_access, [AuthUser, VHostPath, AuthzData]},
         fun(true)  -> success;
            (false) -> refusal;
            ({error, _} = Err) -> Err;
@@ -54,8 +52,8 @@ check_vhost_access(#auth_user{} = AuthUser, VHostPath, Sock) ->
         end).
 
 check_resource_access(#auth_user{} = AuthUser,
-                      #resource{} = Resource, Permission) ->
-    with_cache(authz, {check_resource_access, [AuthUser, Resource, Permission]},
+                      #resource{} = Resource, Permission, AuthzContext) ->
+    with_cache(authz, {check_resource_access, [AuthUser, Resource, Permission, AuthzContext]},
         fun(true)  -> success;
            (false) -> refusal;
            ({error, _} = Err) -> Err;
@@ -71,21 +69,25 @@ check_topic_access(#auth_user{} = AuthUser,
             (_)                -> unknown
         end).
 
+state_can_expire() -> false.
+
+%%
+%% Implementation
+%%
+
 with_cache(BackendType, {F, A}, Fun) ->
-    with_cache(BackendType, {F, A, A}, Fun);
-with_cache(BackendType, {F, OriginalArguments, ArgumentsForCache}, Fun) ->
     {ok, AuthCache} = application:get_env(rabbitmq_auth_backend_cache,
                                           cache_module),
-    case AuthCache:get({F, ArgumentsForCache}) of
+    case AuthCache:get({F, A}) of
         {ok, Result} ->
             Result;
         {error, not_found} ->
             Backend = get_cached_backend(BackendType),
             {ok, TTL} = application:get_env(rabbitmq_auth_backend_cache,
                                             cache_ttl),
-            BackendResult = apply(Backend, F, OriginalArguments),
+            BackendResult = apply(Backend, F, A),
             case should_cache(BackendResult, Fun) of
-                true  -> ok = AuthCache:put({F, ArgumentsForCache}, BackendResult, TTL);
+                true  -> ok = AuthCache:put({F, A}, BackendResult, TTL);
                 false -> ok
             end,
             BackendResult
@@ -112,16 +114,3 @@ should_cache(Result, Fun) ->
         {refusal, true} -> true;
         _               -> false
     end.
-
-extract_address(undefined) ->
-    undefined;
-extract_address(none) ->
-    undefined;
-% for native direct connections the address is set to unknown
-extract_address(#authz_socket_info{peername = {unknown, _Port}}) ->
-    undefined;
-extract_address(#authz_socket_info{peername = {Address, _Port}}) ->
-    Address;
-extract_address(Sock) ->
-    {ok, {Address, _Port}} = rabbit_net:peername(Sock),
-    Address.
