@@ -22,14 +22,17 @@
 -behaviour(rabbit_authz_backend).
 
 -export([description/0, p/1, q/1, join_tags/1]).
--export([user_login_authentication/2, user_login_authorization/1,
-         check_vhost_access/3, check_resource_access/3, check_topic_access/4]).
+-export([user_login_authentication/2, user_login_authorization/2,
+         check_vhost_access/3, check_resource_access/4, check_topic_access/4,
+         state_can_expire/0]).
 
 %% If keepalive connection is closed, retry N times before failing.
 -define(RETRY_ON_KEEPALIVE_CLOSED, 3).
 
 -define(RESOURCE_REQUEST_PARAMETERS, [username, vhost, resource, name, permission]).
 
+-define(SUCCESSFUL_RESPONSE_CODES, [200, 201]).
+
 %%--------------------------------------------------------------------
 
 description() ->
@@ -50,27 +53,31 @@ user_login_authentication(Username, AuthProps) ->
         Other           -> {error, {bad_response, Other}}
     end.
 
-user_login_authorization(Username) ->
-    case user_login_authentication(Username, []) of
+user_login_authorization(Username, AuthProps) ->
+    case user_login_authentication(Username, AuthProps) of
         {ok, #auth_user{impl = Impl}} -> {ok, Impl};
         Else                          -> Else
     end.
 
-check_vhost_access(#auth_user{username = Username, tags = Tags}, VHost, Sock) ->
+check_vhost_access(#auth_user{username = Username, tags = Tags}, VHost, AuthzData = #{peeraddr := PeerAddr}) ->
+    AuthzData1 = maps:remove(peeraddr, AuthzData),
+    OptionsParameters = context_as_parameters(AuthzData1),
     bool_req(vhost_path, [{username, Username},
                           {vhost,    VHost},
-                          {ip,       extract_address(Sock)},
-                          {tags, join_tags(Tags)}]).
+                          {ip,       parse_peeraddr(PeerAddr)},
+                          {tags,     join_tags(Tags)}] ++ OptionsParameters).
 
 check_resource_access(#auth_user{username = Username, tags = Tags},
                       #resource{virtual_host = VHost, kind = Type, name = Name},
-                      Permission) ->
+                      Permission,
+                      AuthzContext) ->
+    OptionsParameters = context_as_parameters(AuthzContext),
     bool_req(resource_path, [{username,   Username},
                              {vhost,      VHost},
                              {resource,   Type},
                              {name,       Name},
                              {permission, Permission},
-                             {tags, join_tags(Tags)}]).
+                             {tags, join_tags(Tags)}] ++ OptionsParameters).
 
 check_topic_access(#auth_user{username = Username, tags = Tags},
                    #resource{virtual_host = VHost, kind = topic = Type, name = Name},
@@ -84,6 +91,8 @@ check_topic_access(#auth_user{username = Username, tags = Tags},
         {permission, Permission},
         {tags, join_tags(Tags)}] ++ OptionsParameters).
 
+state_can_expire() -> false.
+
 %%--------------------------------------------------------------------
 
 context_as_parameters(Options) when is_map(Options) ->
@@ -138,12 +147,12 @@ do_http_req(PathName, Query) ->
     end,
     case httpc:request(Method, Request, HttpOpts, []) of
         {ok, {{_HTTP, Code, _}, _Headers, Body}} ->
-            case Code of
-                200 -> case parse_resp(Body) of
-                           {error, _} = E -> E;
-                           Resp           -> Resp
-                       end;
-                _   -> {error, {Code, Body}}
+            case lists:member(Code, ?SUCCESSFUL_RESPONSE_CODES) of
+                true  -> case parse_resp(Body) of
+                             {error, _} = E -> E;
+                             Resp           -> Resp
+                         end;
+                false -> {error, {Code, Body}}
             end;
         {error, _} = E ->
             E
@@ -169,14 +178,10 @@ join_tags(Tags) ->
   Strings = [rabbit_data_coercion:to_list(T) || T <- Tags],
   string:join(Strings, " ").
 
-%%--------------------------------------------------------------------
+parse_peeraddr(PeerAddr) ->
+    handle_inet_ntoa_peeraddr(inet:ntoa(PeerAddr), PeerAddr).
 
-extract_address(undefined) -> undefined;
-% for native direct connections the address is set to unknown
-extract_address(#authz_socket_info{peername = {unknown, _Port}}) -> undefined;
-extract_address(#authz_socket_info{peername = {Address, _Port}}) -> inet_parse:ntoa(Address);
-extract_address(Sock) ->
-    {ok, {Address, _Port}} = rabbit_net:peername(Sock),
-    inet_parse:ntoa(Address).
-
-%%--------------------------------------------------------------------
+handle_inet_ntoa_peeraddr({error, einval}, PeerAddr) ->
+    rabbit_data_coercion:to_list(PeerAddr);
+handle_inet_ntoa_peeraddr(PeerAddrStr, _PeerAddr0) ->
+    PeerAddrStr.
