@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -24,8 +24,9 @@
 -behaviour(rabbit_authn_backend).
 -behaviour(rabbit_authz_backend).
 
--export([user_login_authentication/2, user_login_authorization/1,
-         check_vhost_access/3, check_resource_access/3, check_topic_access/4]).
+-export([user_login_authentication/2, user_login_authorization/2,
+         check_vhost_access/3, check_resource_access/4, check_topic_access/4,
+         state_can_expire/0]).
 
 -export([get_connections/0]).
 
@@ -89,48 +90,55 @@ user_login_authentication(Username, AuthProps) when is_list(AuthProps) ->
 user_login_authentication(Username, AuthProps) ->
     exit({unknown_auth_props, Username, AuthProps}).
 
-user_login_authorization(Username) ->
-    case user_login_authentication(Username, []) of
+user_login_authorization(Username, AuthProps) ->
+    case user_login_authentication(Username, AuthProps) of
         {ok, #auth_user{impl = Impl, tags = Tags}} -> {ok, Impl, Tags};
         Else                                       -> Else
     end.
 
 check_vhost_access(User = #auth_user{username = Username,
                                      impl     = #impl{user_dn = UserDN}},
-                   VHost, _Sock) ->
+                   VHost, AuthzData) ->
+    OptionsArgs = context_as_options(AuthzData, undefined),
     ADArgs = rabbit_auth_backend_ldap_util:get_active_directory_args(Username),
     Args = [{username, Username},
             {user_dn,  UserDN},
-            {vhost,    VHost}] ++ ADArgs,
+            {vhost,    VHost}] ++ OptionsArgs ++ ADArgs,
     ?L("CHECK: ~s for ~s", [log_vhost(Args), log_user(User)]),
-    R = evaluate_ldap(env(vhost_access_query), Args, User),
-    ?L("DECISION: ~s for ~s: ~p",
-       [log_vhost(Args), log_user(User), log_result(R)]),
-    R.
+    R0 = evaluate_ldap(env(vhost_access_query), Args, User),
+    R1 = ensure_rabbit_authz_backend_result(R0),
+    ?L("DECISION: ~s for ~s: ~p (~p)",
+       [log_vhost(Args), log_user(User),
+        log_result(R0), log_result(R1)]),
+    R1.
 
 check_resource_access(User = #auth_user{username = Username,
                                         impl     = #impl{user_dn = UserDN}},
                       #resource{virtual_host = VHost, kind = Type, name = Name},
-                      Permission) ->
+                      Permission,
+                      AuthzContext) ->
+    OptionsArgs = context_as_options(AuthzContext, undefined),
     ADArgs = rabbit_auth_backend_ldap_util:get_active_directory_args(Username),
     Args = [{username,   Username},
             {user_dn,    UserDN},
             {vhost,      VHost},
             {resource,   Type},
             {name,       Name},
-            {permission, Permission}] ++ ADArgs,
+            {permission, Permission}] ++ OptionsArgs ++ ADArgs,
     ?L("CHECK: ~s for ~s", [log_resource(Args), log_user(User)]),
-    R = evaluate_ldap(env(resource_access_query), Args, User),
-    ?L("DECISION: ~s for ~s: ~p",
-       [log_resource(Args), log_user(User), log_result(R)]),
-    R.
+    R0 = evaluate_ldap(env(resource_access_query), Args, User),
+    R1 = ensure_rabbit_authz_backend_result(R0),
+    ?L("DECISION: ~s for ~s: ~p (~p)",
+       [log_resource(Args), log_user(User),
+        log_result(R0), log_result(R1)]),
+    R1.
 
 check_topic_access(User = #auth_user{username = Username,
                                      impl     = #impl{user_dn = UserDN}},
                    #resource{virtual_host = VHost, kind = topic = Resource, name = Name},
                    Permission,
                    Context) ->
-    OptionsArgs = topic_context_as_options(Context, undefined),
+    OptionsArgs = context_as_options(Context, undefined),
     ADArgs = rabbit_auth_backend_ldap_util:get_active_directory_args(Username),
     Args = [{username,   Username},
             {user_dn,    UserDN},
@@ -139,20 +147,38 @@ check_topic_access(User = #auth_user{username = Username,
             {name,       Name},
             {permission, Permission}] ++ OptionsArgs ++ ADArgs,
     ?L("CHECK: ~s for ~s", [log_resource(Args), log_user(User)]),
-    R = evaluate_ldap(env(topic_access_query), Args, User),
-    ?L("DECISION: ~s for ~s: ~p",
-        [log_resource(Args), log_user(User), log_result(R)]),
-    R.
+    R0 = evaluate_ldap(env(topic_access_query), Args, User),
+    R1 = ensure_rabbit_authz_backend_result(R0),
+    ?L("DECISION: ~s for ~s: ~p (~p)",
+        [log_resource(Args), log_user(User),
+         log_result(R0), log_result(R1)]),
+    R1.
+
+state_can_expire() -> false.
 
 %%--------------------------------------------------------------------
 
-topic_context_as_options(Context, Namespace) when is_map(Context) ->
+ensure_rabbit_authz_backend_result(true) ->
+    true;
+ensure_rabbit_authz_backend_result(false) ->
+    false;
+ensure_rabbit_authz_backend_result({error, _}=Error) ->
+    Error;
+% rabbitmq/rabbitmq-auth-backend-ldap#116
+ensure_rabbit_authz_backend_result({refused, _, _}) ->
+    false;
+ensure_rabbit_authz_backend_result({ok, _}) ->
+    true;
+ensure_rabbit_authz_backend_result({ok, _, _}) ->
+    true.
+
+context_as_options(Context, Namespace) when is_map(Context) ->
     % filter keys that would erase fixed variables
     lists:flatten([begin
          Value = maps:get(Key, Context),
          case Value of
              MapOfValues when is_map(MapOfValues) ->
-                 topic_context_as_options(MapOfValues, Key);
+                 context_as_options(MapOfValues, Key);
              SimpleValue ->
                  case Namespace of
                      undefined ->
@@ -164,7 +190,7 @@ topic_context_as_options(Context, Namespace) when is_map(Context) ->
      end || Key <- maps:keys(Context), lists:member(
                     rabbit_data_coercion:to_atom(Key),
                     ?RESOURCE_ACCESS_QUERY_VARIABLES) =:= false]);
-topic_context_as_options(_, _) ->
+context_as_options(_, _) ->
     [].
 
 create_option_name_with_namespace(Namespace, Key) ->
@@ -219,9 +245,11 @@ evaluate0({in_group_nested, DNPattern, Desc}, Args, User, LDAP) ->
 evaluate0({in_group_nested, DNPattern, Desc, Scope}, Args,
           #auth_user{impl = #impl{user_dn = UserDN}}, LDAP) ->
     GroupsBase = case env(group_lookup_base) of
-        none -> env(dn_lookup_base);
-        B    -> B
-    end,
+                     none ->
+                         get_expected_env_str(dn_lookup_base, none);
+                     B ->
+                         B
+                 end,
     GroupDN = fill(DNPattern, Args),
     EldapScope =
         case Scope of
@@ -374,7 +402,7 @@ do_match(S1, S2) ->
             R
     end.
 
-%% In some cases when fetching regular expressions, LDAP evalution() 
+%% In some cases when fetching regular expressions, LDAP evalution()
 %% returns a list of strings, so we need to wrap guards around that.
 %% If a list of strings is returned, loop and match versus each element.
 do_match_multi(S1, []) ->
@@ -692,6 +720,20 @@ ssl_options() ->
 at_least(Ver) ->
     rabbit_misc:version_compare(erlang:system_info(version), Ver) =/= lt.
 
+% Note:
+% Default is configured in the Makefile
+get_expected_env_str(Key, Default) ->
+    V = case env(Key) of
+            Default ->
+                rabbit_log_ldap:warning("rabbitmq_auth_backend_ldap configuration key '~p' is set to "
+                                        "the default value of '~p', expected to get a non-default value~n",
+                                        [Key, Default]),
+                Default;
+            V0 ->
+                V0
+        end,
+    rabbit_data_coercion:to_list(V).
+
 env(F) ->
     {ok, V} = application:get_env(rabbitmq_auth_backend_ldap, F),
     V.
@@ -740,14 +782,19 @@ vhost_if_defined([])    -> [];
 vhost_if_defined(<<>>)  -> [];
 vhost_if_defined(VHost) -> [{vhost, VHost}].
 
-dn_lookup_when() -> case {env(dn_lookup_attribute), env(dn_lookup_bind)} of
-                        {none, _}       -> never;
-                        {_,    as_user}   -> postbind;
-                        %% make it more obvious what the invariants are,
-                        %% see rabbitmq/rabbitmq-auth-backend-ldap#94. MK.
-                        {_,    anon}      -> prebind;
-                        {_,    _}         -> prebind
-                    end.
+dn_lookup_when() ->
+    case {env(dn_lookup_attribute), env(dn_lookup_bind)} of
+        {none, DnLookupBind} ->
+            rabbit_log_ldap:warning(
+              "dn_lookup_attribute is 'none' but dn_lookup_bind is ~p. No DN lookup bind will be performed~n",
+              [DnLookupBind]),
+            never;
+        {_, as_user} -> postbind;
+        %% make it more obvious what the invariants are,
+        %% see rabbitmq/rabbitmq-auth-backend-ldap#94. MK.
+        {_, anon} -> prebind;
+        {_, _} -> prebind
+    end.
 
 username_to_dn_prebind(Username) ->
     with_ldap({ok, env(dn_lookup_bind)},
@@ -758,10 +805,11 @@ username_to_dn(Username, _LDAP, _When)    -> fill_user_dn_pattern(Username).
 
 dn_lookup(Username, LDAP) ->
     Filled = fill_user_dn_pattern(Username),
+    DnLookupBase = get_expected_env_str(dn_lookup_base, none),
+    DnLookupAttribute = get_expected_env_str(dn_lookup_attribute, none),
     case eldap:search(LDAP,
-                      [{base, env(dn_lookup_base)},
-                       {filter, eldap:equalityMatch(
-                                  env(dn_lookup_attribute), Filled)},
+                      [{base, DnLookupBase},
+                       {filter, eldap:equalityMatch(DnLookupAttribute, Filled)},
                        {attributes, ["distinguishedName"]}]) of
         {ok, {referral, Referrals}} ->
             {error, {referrals_not_supported, Referrals}};
