@@ -32,6 +32,7 @@ defmodule RabbitMQ.CLI.CommandBehaviour do
               | {:ok, any}
               | {:stream, Enum.t()}
               | {:error, RabbitMQ.CLI.Core.ExitCodes.exit_code(), [String.t()]}
+
   @optional_callbacks formatter: 0,
                       printer: 0,
                       scopes: 0,
@@ -64,6 +65,60 @@ defmodule RabbitMQ.CLI.CommandBehaviour do
   ## {:fun, fun} - use a custom function to start distribution
   @callback distribution(map()) :: :cli | :none | {:fun, (map() -> :ok | {:error, any()})}
 
+  defmacro defcmd(map) do
+    usage_q = case map[:usage] do
+      nil -> :ok
+      usage ->
+        quote do def usage(), do: unquote(usage) end
+    end
+    scopes_q = case map[:scopes] do
+      nil -> :ok
+      scopes ->
+        quote do def scopes(), do: unquote(scopes) end
+    end
+    description_q = case map[:description] do
+      nil -> :ok
+      description ->
+        quote do def description(), do: unquote(description) end
+    end
+    help_section_q = case map[:help_section] do
+      nil -> :ok
+      help_section ->
+        quote do def help_section(), do: unquote(help_section) end
+    end
+    usage_additional_q = case map[:usage_additional] do
+      nil -> :ok
+      usage_additional ->
+        quote do def usage_additional(), do: unquote(usage_additional) end
+    end
+    formatter_q = case map[:formatter] do
+      nil -> :ok
+      formatter ->
+        quote do def formatter(), do: unquote(formatter) end
+    end
+    switches_q = case map[:switches] do
+      nil -> :ok
+      switches ->
+        quote do def switches(), do: unquote(switches) end
+    end
+    aliases_q = case map[:aliases] do
+      nil -> :ok
+      aliases ->
+        quote do def aliases(), do: unquote(aliases) end
+    end
+
+    quote do
+      unquote(usage_q)
+      unquote(scopes_q)
+      unquote(description_q)
+      unquote(help_section_q)
+      unquote(usage_additional_q)
+      unquote(formatter_q)
+      unquote(switches_q)
+      unquote(aliases_q)
+    end
+  end
+
   def usage(cmd) do
     cmd.usage()
   end
