@@ -52,6 +52,7 @@ defmodule RabbitMQ.CLI.Core.DocGuide do
   Macros.defguide("connections")
   Macros.defguide("configuration", path_segment: "configure")
   Macros.defguide("consumers")
+  Macros.defguide("definitions")
   Macros.defguide("erlang_versions", path_segment: "which-erlang")
   Macros.defguide("feature_flags", domain: "next.rabbitmq.com")
   Macros.defguide("firehose")
@@ -60,7 +61,9 @@ defmodule RabbitMQ.CLI.Core.DocGuide do
   Macros.defguide("management")
   Macros.defguide("memory_use")
   Macros.defguide("monitoring")
+  Macros.defguide("networking")
   Macros.defguide("parameters")
+  Macros.defguide("publishers")
   Macros.defguide("plugins")
   Macros.defguide("queues")
   Macros.defguide("quorum_queues", domain: "next.rabbitmq.com")
@@ -68,4 +71,5 @@ defmodule RabbitMQ.CLI.Core.DocGuide do
   Macros.defguide("tls", path_segment: "ssl")
   Macros.defguide("troubleshooting")
   Macros.defguide("virtual_hosts", path_segments: "vhosts")
+  Macros.defguide("upgrade")
 end
