@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,15 +11,14 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ClearParameterCommand do
-
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
   use RabbitMQ.CLI.DefaultOutput
+
   def merge_defaults(args, opts) do
     {args, Map.merge(%{vhost: "/"}, opts)}
   end
@@ -27,22 +26,42 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ClearParameterCommand do
   def validate(args, _) when is_list(args) and length(args) < 2 do
     {:validation_failure, :not_enough_args}
   end
-  def validate([_|_] = args, _) when length(args) > 2 do
+
+  def validate([_ | _] = args, _) when length(args) > 2 do
     {:validation_failure, :too_many_args}
   end
-  def validate([_,_], _), do: :ok
+
+  def validate([_, _], _), do: :ok
 
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([component_name, key], %{node: node_name, vhost: vhost}) do
-    :rabbit_misc.rpc_call(node_name,
+    :rabbit_misc.rpc_call(
+      node_name,
       :rabbit_runtime_parameters,
       :clear,
-      [vhost, component_name, key, Helpers.cli_acting_user()])
+      [vhost, component_name, key, Helpers.cli_acting_user()]
+    )
   end
 
-  def usage, do: "clear_parameter [-p <vhost>] <component_name> <key>"
+  def usage, do: "clear_parameter [--vhost <vhost>] <component_name> <name>"
 
+  def usage_additional() do
+    [
+      ["<component_name>", "component name"],
+      ["<name>", "parameter name (identifier)"]
+    ]
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.parameters()
+    ]
+  end
+
+  def help_section(), do: :parameters
+
+  def description(), do: "Clears a runtime parameter."
 
   def banner([component_name, key], %{vhost: vhost}) do
     "Clearing runtime parameter \"#{key}\" for component \"#{component_name}\" on vhost \"#{vhost}\" ..."
