@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,31 +11,45 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ClearPasswordCommand do
-
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
-
-  def merge_defaults(args, opts), do: {args, opts}
-
-  def validate([], _), do: {:validation_failure, :not_enough_args}
-  def validate([_|_] = args, _) when length(args) > 1, do: {:validation_failure, :too_many_args}
-  def validate([_], _), do: :ok
 
+  use RabbitMQ.CLI.Core.MergesNoDefaults
+  use RabbitMQ.CLI.Core.AcceptsOnePositionalArgument
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([_user] = args, %{node: node_name}) do
-    :rabbit_misc.rpc_call(node_name, :rabbit_auth_backend_internal, :clear_password,
-      args ++ [Helpers.cli_acting_user()])
+    :rabbit_misc.rpc_call(
+      node_name,
+      :rabbit_auth_backend_internal,
+      :clear_password,
+      args ++ [Helpers.cli_acting_user()]
+    )
   end
 
+  use RabbitMQ.CLI.DefaultOutput
+
   def usage, do: "clear_password <username>"
 
-  def banner([user], _), do: "Clearing password for user \"#{user}\" ..."
+  def usage_additional() do
+    [
+      ["<username>", "Name of the user whose password should be cleared"]
+    ]
+  end
 
+  def usage_doc_guides() do
+    [
+      DocGuide.access_control()
+    ]
+  end
+
+  def help_section(), do: :user_management
+
+  def description(), do: "Clears (resets) password and disables password login for a user"
+
+  def banner([user], _), do: "Clearing password for user \"#{user}\" ..."
 end
