@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,35 +11,46 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ClearPolicyCommand do
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{Helpers, DocGuide}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
   use RabbitMQ.CLI.DefaultOutput
+
   def merge_defaults(args, opts) do
     {args, Map.merge(%{vhost: "/"}, opts)}
   end
 
-  def validate([], _) do
-    {:validation_failure, :not_enough_args}
-  end
-  def validate([_,_|_], _) do
-    {:validation_failure, :too_many_args}
-  end
-  def validate([_], _), do: :ok
-
+  use RabbitMQ.CLI.Core.AcceptsOnePositionalArgument
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([key], %{node: node_name, vhost: vhost}) do
-    :rabbit_misc.rpc_call(node_name,
-      :rabbit_policy, :delete, [vhost, key, Helpers.cli_acting_user()])
+    :rabbit_misc.rpc_call(node_name, :rabbit_policy, :delete, [
+      vhost,
+      key,
+      Helpers.cli_acting_user()
+    ])
   end
 
-  def usage, do: "clear_policy [-p <vhost>] <key>"
+  def usage, do: "clear_policy [--vhost <vhost>] <name>"
 
+  def usage_additional() do
+    [
+      ["<name>", "Name of policy to clear (remove)"]
+    ]
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.parameters()
+    ]
+  end
+
+  def help_section(), do: :policies
+
+  def description(), do: "Clears (removes) a policy"
 
   def banner([key], %{vhost: vhost}) do
     "Clearing policy \"#{key}\" on vhost \"#{vhost}\" ..."
