@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,11 +11,10 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ClearVhostLimitsCommand do
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
   use RabbitMQ.CLI.DefaultOutput
@@ -24,25 +23,28 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ClearVhostLimitsCommand do
     {args, Map.merge(%{vhost: "/"}, opts)}
   end
 
-  def validate([], _) do
-    :ok
-  end
-
-  def validate([_], _) do
-    {:validation_failure, :too_many_args}
-  end
-
-  def validate([_|_] = args, _) when length(args) > 1 do
-    {:validation_failure, :too_many_args}
-  end
+  use RabbitMQ.CLI.Core.AcceptsNoPositionalArguments
 
   def run([], %{node: node_name, vhost: vhost}) do
-    :rabbit_misc.rpc_call(node_name, :rabbit_vhost_limit, :clear, [vhost, Helpers.cli_acting_user()])
+    :rabbit_misc.rpc_call(node_name, :rabbit_vhost_limit, :clear, [
+      vhost,
+      Helpers.cli_acting_user()
+    ])
   end
 
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
-  def usage, do: "clear_vhost_limits [-p <vhost>]"
+  def usage, do: "clear_vhost_limits [--vhost <vhost>]"
+
+  def usage_doc_guides() do
+    [
+      DocGuide.virtual_hosts()
+    ]
+  end
+
+  def help_section(), do: :virtual_hosts
+
+  def description(), do: "Clears virtual host limits"
 
   def banner([], %{vhost: vhost}) do
     "Clearing vhost \"#{vhost}\" limits ..."
