@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,14 +11,14 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ExecCommand do
-  @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
+  alias RabbitMQ.CLI.Core.DocGuide
 
-  def merge_defaults(args, opts), do: {args, opts}
+  @behaviour RabbitMQ.CLI.CommandBehaviour
+
+  use RabbitMQ.CLI.Core.MergesNoDefaults
 
   def switches(), do: [offline: :boolean]
 
@@ -44,6 +44,7 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ExecCommand do
     rescue
       ex in SyntaxError ->
         {:validation_failure, "SyntaxError: " <> Exception.message(ex)}
+
       _ ->
         :ok
     end
@@ -53,14 +54,36 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ExecCommand do
     try do
       {val, _} = Code.eval_string(expr, [options: opts], __ENV__)
       {:ok, val}
-    rescue ex ->
-      {:error, Exception.message(ex)}
+    rescue
+      ex ->
+        {:error, Exception.message(ex)}
     end
   end
 
+  use RabbitMQ.CLI.DefaultOutput
+
   def formatter(), do: RabbitMQ.CLI.Formatters.Inspect
 
-  def usage, do: "exec <expr> [--offline]"
-
   def banner(_, _), do: nil
+
+  def usage, do: "exec <expression> [--offline]"
+
+  def usage_additional() do
+    [
+      ["<expression>", "Expression to evaluate"],
+      ["--offline", "disable inter-node communication"]
+    ]
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.cli(),
+      DocGuide.monitoring(),
+      DocGuide.troubleshooting()
+    ]
+  end
+
+  def help_section(), do: :operations
+
+  def description(), do: "Evaluates a snippet of Elixir code on the CLI node"
 end
