@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,20 +11,18 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ListCiphersCommand do
+  alias RabbitMQ.CLI.Core.DocGuide
+
   @behaviour RabbitMQ.CLI.CommandBehaviour
   use RabbitMQ.CLI.DefaultOutput
 
-  def merge_defaults(args, opts) do
-    {args, opts}
-  end
-  def validate(args, _) when length(args) > 0 do
-      {:validation_failure,
-       {:bad_argument, :too_many_args}}
-  end
-  def validate(_, _), do: :ok
+  def scopes(), do: [:ctl, :diagnostics]
+
+  use RabbitMQ.CLI.Core.MergesNoDefaults
+  use RabbitMQ.CLI.Core.AcceptsNoPositionalArguments
 
   def distribution(_), do: :none
 
@@ -36,6 +34,16 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ListCiphersCommand do
 
   def usage, do: "list_ciphers"
 
-  def banner(_, _), do: "Listing supported ciphers ..."
+  def usage_doc_guides() do
+    [
+      DocGuide.configuration(),
+      DocGuide.tls()
+    ]
+  end
 
+  def help_section(), do: :observability_and_health_checks
+
+  def description(), do: "Lists cipher suites supported by encoding commands"
+
+  def banner(_, _), do: "Listing supported ciphers ..."
 end
