@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -15,6 +15,7 @@
 
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ListFeatureFlagsCommand do
+  alias RabbitMQ.CLI.Core.{DocGuide, Validators}
   alias RabbitMQ.CLI.Ctl.InfoKeys
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
@@ -26,9 +27,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ListFeatureFlagsCommand do
 
   def info_keys(), do: @info_keys
 
-  def scopes(), do: [:ctl]
-  def switches(), do: [timeout: :integer]
-  def aliases(), do: [t: :timeout]
+  def scopes(), do: [:ctl, :diagnostics]
+  use RabbitMQ.CLI.Core.AcceptsDefaultSwitchesAndTimeout
 
   def merge_defaults([], opts), do: {["name", "state"], opts}
   def merge_defaults(args, opts), do: {args, opts}
@@ -40,19 +40,51 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ListFeatureFlagsCommand do
     end
   end
 
-  use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
-
-  def run([_|_] = args, %{node: node_name, timeout: time_out}) do
-    :rabbit_misc.rpc_call(node_name, :rabbit_ff_extra, :cli_info, [], time_out)
-    |> filter_by_arg(args)
+  def validate_execution_environment(args, opts) do
+    Validators.chain(
+      [
+        &Validators.rabbit_is_loaded/2,
+        &Validators.rabbit_is_running/2
+      ],
+      [args, opts]
+    )
   end
 
-  def usage, do: "list_feature_flags [<feature-flag-info-item> ...]"
+  def run([_|_] = args, %{node: node_name, timeout: timeout}) do
+    case :rabbit_misc.rpc_call(node_name, :rabbit_ff_extra, :cli_info, [], timeout) do
+      # Server does not support feature flags, consider none are available.
+      # See rabbitmq/rabbitmq-cli#344 for context. MK.
+      {:badrpc, {:EXIT, {:undef, _}}} -> []
+      {:badrpc, _} = err    -> err
+      val                   -> filter_by_arg(val, args)
+    end
+
+  end
+
+  def banner(_, _), do: "Listing feature flags ..."
+
+  def usage, do: "list_feature_flags [<column> ...]"
 
   def usage_additional() do
-    "<feature-flag-info-item> must be a member of the list [name, state, stability, provided_by, desc]."
+    [
+      ["<column>", "must be one of " <> Enum.join(Enum.sort(@info_keys), ", ")]
+    ]
   end
 
+  def usage_doc_guides() do
+    [
+      DocGuide.feature_flags()
+    ]
+  end
+
+  def help_section(), do: :feature_flags
+
+  def description(), do: "Lists feature flags"
+
+  #
+  # Implementation
+  #
+
   defp filter_by_arg(ff_info, _) when is_tuple(ff_info) do
     # tuple means unexpected data
     ff_info
@@ -68,6 +100,4 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ListFeatureFlagsCommand do
       end
     )
   end
-
-  def banner(_,_), do: "Listing feature flags ..."
 end
