@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,40 +11,47 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ListGlobalParametersCommand do
+  alias RabbitMQ.CLI.Core.DocGuide
   @behaviour RabbitMQ.CLI.CommandBehaviour
+
+  def merge_defaults(args, opts) do
+    {args, Map.merge(%{table_headers: true}, opts)}
+  end
+
+  def scopes(), do: [:ctl, :diagnostics]
+
+  use RabbitMQ.CLI.Core.AcceptsDefaultSwitchesAndTimeout
+  use RabbitMQ.CLI.Core.AcceptsNoPositionalArguments
+  use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
+
+  def run([], %{node: node_name, timeout: timeout}) do
+    :rabbit_misc.rpc_call(
+      node_name,
+      :rabbit_runtime_parameters,
+      :list_global_formatted,
+      [],
+      timeout
+    )
+  end
+
   use RabbitMQ.CLI.DefaultOutput
 
   def formatter(), do: RabbitMQ.CLI.Formatters.Table
 
-  def merge_defaults(args, opts) do
-    {args, opts}
+  def usage, do: "list_global_parameters [--no-table-headers]"
+
+  def usage_doc_guides() do
+    [
+      DocGuide.parameters()
+    ]
   end
 
-  def scopes(), do: [:ctl, :diagnostics]
-  def switches(), do: [timeout: :integer]
-  def aliases(), do: [t: :timeout]
+  def help_section(), do: :parameters
 
-  def validate([_|_], _) do
-    {:validation_failure, :too_many_args}
-  end
-  def validate([], _), do: :ok
-
-  use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
-
-  def run([], %{node: node_name, timeout: timeout}) do
-    :rabbit_misc.rpc_call(node_name,
-      :rabbit_runtime_parameters,
-      :list_global_formatted,
-      [],
-      timeout)
-  end
-
-  def usage, do: "list_global_parameters"
+  def description(), do: "Lists global runtime parameters"
 
   def banner(_, _), do: "Listing global runtime parameters ..."
-
 end
