@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,33 +11,42 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.ListUsersCommand do
-  @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
+  alias RabbitMQ.CLI.Core.DocGuide
 
-  def formatter(), do: RabbitMQ.CLI.Formatters.Table
+  @behaviour RabbitMQ.CLI.CommandBehaviour
 
   def scopes(), do: [:ctl, :diagnostics]
-  def switches(), do: [timeout: :integer]
-  def aliases(), do: [t: :timeout]
+  use RabbitMQ.CLI.Core.AcceptsDefaultSwitchesAndTimeout
 
-  def merge_defaults(args, opts), do: {args, opts}
-
-  def validate([_|_], _) do
-    {:validation_failure, :too_many_args}
+  def merge_defaults(args, opts) do
+    {args, Map.merge(%{table_headers: true}, opts)}
   end
-  def validate(_, _), do: :ok
 
+  use RabbitMQ.CLI.Core.AcceptsNoPositionalArguments
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([], %{node: node_name, timeout: timeout}) do
     :rabbit_misc.rpc_call(node_name, :rabbit_auth_backend_internal, :list_users, [], timeout)
   end
 
-  def usage, do: "list_users"
+  use RabbitMQ.CLI.DefaultOutput
 
-  def banner(_,_), do: "Listing users ..."
+  def formatter(), do: RabbitMQ.CLI.Formatters.Table
+
+  def usage, do: "list_users [--no-table-headers]"
+
+  def usage_doc_guides() do
+    [
+      DocGuide.access_control()
+    ]
+  end
+
+  def help_section(), do: :user_management
+
+  def description(), do: "List user names and tags"
+
+  def banner(_, _), do: "Listing users ..."
 end
