@@ -29,6 +29,7 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ReportCommand do
     ListQueuesCommand,
     StatusCommand
   }
+  alias RabbitMQ.CLI.Diagnostics.Commands.CommandLineArgumentsCommand
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
 
@@ -51,7 +52,7 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ReportCommand do
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([], %{node: node_name} = opts) do
-    case :rabbit_misc.rpc_call(node_name, :rabbit_vhost, :list, []) do
+    case :rabbit_misc.rpc_call(node_name, :rabbit_vhost, :list_names, []) do
       {:badrpc, _} = err ->
         err
 
@@ -61,7 +62,8 @@ defmodule RabbitMQ.CLI.Ctl.Commands.ReportCommand do
           run_command(ClusterStatusCommand, [], opts),
           run_command(EnvironmentCommand, [], opts),
           run_command(ListConnectionsCommand, info_keys(ListConnectionsCommand), opts),
-          run_command(ListChannelsCommand, info_keys(ListChannelsCommand), opts)
+          run_command(ListChannelsCommand, info_keys(ListChannelsCommand), opts),
+          run_command(CommandLineArgumentsCommand, [], opts)
         ]
 
         vhost_data =
