@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,14 +11,12 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.SetPermissionsCommand do
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
 
   def merge_defaults(args, opts) do
     {args, Map.merge(%{vhost: "/"}, opts)}
@@ -28,26 +26,50 @@ defmodule RabbitMQ.CLI.Ctl.Commands.SetPermissionsCommand do
     {:validation_failure, :not_enough_args}
   end
 
-  def validate([_|_] = args, _) when length(args) < 4 do
+  def validate([_ | _] = args, _) when length(args) < 4 do
     {:validation_failure, :not_enough_args}
   end
 
-  def validate([_|_] = args, _) when length(args) > 4 do
+  def validate([_ | _] = args, _) when length(args) > 4 do
     {:validation_failure, :too_many_args}
   end
+
   def validate(_, _), do: :ok
 
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([user, conf, write, read], %{node: node_name, vhost: vhost}) do
-    :rabbit_misc.rpc_call(node_name,
+    :rabbit_misc.rpc_call(
+      node_name,
       :rabbit_auth_backend_internal,
       :set_permissions,
       [user, vhost, conf, write, read, Helpers.cli_acting_user()]
     )
   end
 
-  def usage, do: "set_permissions [-p <vhost>] <username> <conf> <write> <read>"
+  use RabbitMQ.CLI.DefaultOutput
 
-  def banner([user|_], %{vhost: vhost}), do: "Setting permissions for user \"#{user}\" in vhost \"#{vhost}\" ..."
+  def usage, do: "set_permissions [--vhost <vhost>] <username> <conf> <write> <read>"
+
+  def usage_additional() do
+    [
+      ["<username>", "Self-explanatory"],
+      ["<conf>", "Configuration permission pattern"],
+      ["<write>", "Write permission pattern"],
+      ["<read>", "Read permission pattern"]
+    ]
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.access_control(),
+      DocGuide.virtual_hosts()
+    ]
+  end
+
+  def help_section(), do: :access_control
+  def description(), do: "Sets user permissions for a vhost"
+
+  def banner([user | _], %{vhost: vhost}),
+    do: "Setting permissions for user \"#{user}\" in vhost \"#{vhost}\" ..."
 end
