@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,32 +11,53 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.SetUserTagsCommand do
-  alias RabbitMQ.CLI.Core.Helpers
+  alias RabbitMQ.CLI.Core.{DocGuide, Helpers}
 
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
 
   def merge_defaults(args, opts), do: {args, opts}
 
-  def validate([], _), do: {:validation_failure, :not_enough_args}
+  def validate([], _) do
+    {:validation_failure, :not_enough_args}
+  end
   def validate(_, _), do: :ok
 
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([user | tags], %{node: node_name}) do
-    :rabbit_misc.rpc_call(node_name,
+    :rabbit_misc.rpc_call(
+      node_name,
       :rabbit_auth_backend_internal,
       :set_tags,
       [user, tags, Helpers.cli_acting_user()]
     )
   end
 
+  use RabbitMQ.CLI.DefaultOutput
+
   def usage, do: "set_user_tags <username> <tag> [...]"
 
+  def usage_additional() do
+    [
+      ["<username>", "Self-explanatory"],
+      ["<tags>", "Space separated list of tags"]
+    ]
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.management(),
+      DocGuide.access_control()
+    ]
+  end
+
+  def help_section(), do: :user_management
+
+  def description(), do: "Sets user tags"
+
   def banner([user | tags], _) do
     "Setting tags for user \"#{user}\" to [#{tags |> Enum.join(", ")}] ..."
   end
