@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,30 +11,41 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
-
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Ctl.Commands.TraceOffCommand do
+  alias RabbitMQ.CLI.Core.DocGuide
+
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
 
   def merge_defaults(_, opts) do
     {[], Map.merge(%{vhost: "/"}, opts)}
   end
 
-  def validate([_|_], _), do: {:validation_failure, :too_many_args}
-  def validate(_, _), do: :ok
-
+  use RabbitMQ.CLI.Core.AcceptsNoPositionalArguments
   use RabbitMQ.CLI.Core.RequiresRabbitAppRunning
 
   def run([], %{node: node_name, vhost: vhost}) do
     case :rabbit_misc.rpc_call(node_name, :rabbit_trace, :stop, [vhost]) do
-      :ok   -> {:ok, "Trace disabled for vhost #{vhost}"};
+      :ok -> {:ok, "Trace disabled for vhost #{vhost}"}
       other -> other
     end
   end
 
-  def usage, do: "trace_off [-p <vhost>]"
+  use RabbitMQ.CLI.DefaultOutput
+
+  def usage do
+    "trace_off [--vhost <vhost>]"
+  end
+
+  def usage_doc_guides() do
+    [
+      DocGuide.firehose(),
+      DocGuide.virtual_hosts()
+    ]
+  end
+
+  def help_section(), do: :virtual_hosts
 
   def banner(_, %{vhost: vhost}), do: "Stopping tracing for vhost \"#{vhost}\" ..."
 end
