@@ -45,16 +45,17 @@ defmodule RabbitMQ.CLI.DefaultOutput do
   defp normalize_output({:badrpc, {:timeout, _n}} = input), do: {:error, input}
   defp normalize_output({:badrpc, {:timeout, _n, _msg}} = input), do: {:error, input}
   defp normalize_output({:badrpc, {:EXIT, reason}}), do: {:error, reason}
+  defp normalize_output({:error, exit_code, string}) when is_integer(exit_code) do
+    {:error, exit_code, to_string(string)}
+  end
   defp normalize_output({:error, format, args})
        when (is_list(format) or is_binary(format)) and is_list(args) do
     {:error, to_string(:rabbit_misc.format(format, args))}
   end
-
+  defp normalize_output({:error, _} = input), do: input
   defp normalize_output({:error_string, string}) do
     {:error, to_string(string)}
   end
-
-  defp normalize_output({:error, _} = input), do: input
   defp normalize_output(unknown) when is_atom(unknown), do: {:error, unknown}
   defp normalize_output({unknown, _} = input) when is_atom(unknown), do: {:error, input}
   defp normalize_output(result) when not is_atom(result), do: {:ok, result}
