@@ -14,32 +14,22 @@
 ## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 defmodule RabbitMQ.CLI.Diagnostics.Commands.CipherSuitesCommand do
+  alias RabbitMQ.CLI.Core.Helpers
+
   @behaviour RabbitMQ.CLI.CommandBehaviour
-  use RabbitMQ.CLI.DefaultOutput
 
   def merge_defaults(args, opts) do
-    format_opts = case opts do
-      %{openssl_format: true} -> %{format: "openssl", all: false};
-      %{}                     -> %{format: "erlang", all: false}
-    end
-    {args, Map.merge(format_opts,
-                     Map.delete(case_insensitive_format(opts), :openssl_format))}
+    {args, Map.merge(%{all: false, format: "openssl"}, Helpers.case_insensitive_format(opts))}
   end
 
-  defp case_insensitive_format(%{format: format} = opts) do
-    %{ opts | format: String.downcase(format) }
-  end
-  defp case_insensitive_format(opts), do: opts
-
   def switches(), do: [timeout: :integer,
-                       openssl_format: :boolean,
                        format: :string,
                        all: :boolean]
   def aliases(), do: [t: :timeout]
 
   def validate(_, %{format: format})
       when format != "openssl" and format != "erlang" and format != "map" do
-    {:validation_failure, {:bad_argument, "Format must be one of: openssl, erlang or map"}}
+    {:validation_failure, {:bad_argument, "Format should be either openssl, erlang or map"}}
   end
   def validate(args, _) when length(args) > 0 do
     {:validation_failure, :too_many_args}
@@ -60,6 +50,8 @@ defmodule RabbitMQ.CLI.Diagnostics.Commands.CipherSuitesCommand do
     :rabbit_misc.rpc_call(node_name, mod, function, args, timeout)
   end
 
+  use RabbitMQ.CLI.DefaultOutput
+
   def banner([], %{format: "openssl"}),  do: "Listing available cipher suites in OpenSSL format"
   def banner([], %{format: "erlang"}), do: "Listing available cipher suites in Erlang term format"
   def banner([], %{format: "map"}), do: "Listing available cipher suites in map format"
@@ -68,13 +60,12 @@ defmodule RabbitMQ.CLI.Diagnostics.Commands.CipherSuitesCommand do
 
   def description(), do: "Lists cipher suites enabled by default. To list all available cipher suites, add the --all argument."
 
-  def usage, do: "cipher_suites [--format (openssl | erlang | map)] [--all]"
+  def usage, do: "cipher_suites [--format <openssl | erlang | map>] [--all]"
 
   def usage_additional() do
     [
       ["--format", "output format to use: openssl, erlang or map"],
-      ["--all", "list all available suites"],
-      ["--openssl-format", "use OpenSSL format. Deprecated: use --format=openssl instead"]
+      ["--all", "list all available suites"]
     ]
   end
 
