@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 ## The Original Code is RabbitMQ.
 ##
 ## The Initial Developer of the Original Code is GoPivotal, Inc.
-## Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+## Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 alias RabbitMQ.CLI.Formatters.FormatterHelpers
 
 defmodule RabbitMQ.CLI.Formatters.Inspect do
@@ -19,21 +19,27 @@ defmodule RabbitMQ.CLI.Formatters.Inspect do
 
   def format_output(output, _) do
     case is_binary(output) do
-      true  -> output;
+      true -> output
       false -> inspect(output)
     end
   end
 
   def format_stream(stream, options) do
-    elements = Stream.scan(stream, :empty,
-                           FormatterHelpers.without_errors_2(
-                            fn(element, previous) ->
-                              separator = case previous do
-                                :empty -> "";
-                                _      -> ","
-                              end
-                              format_element(element, separator, options)
-                            end))
+    elements =
+      Stream.scan(
+        stream,
+        :empty,
+        FormatterHelpers.without_errors_2(fn element, previous ->
+          separator =
+            case previous do
+              :empty -> ""
+              _ -> ","
+            end
+
+          format_element(element, separator, options)
+        end)
+      )
+
     Stream.concat([["["], elements, ["]"]])
   end
 
