@@ -1,7 +1,7 @@
 ## The contents of this file are subject to the Mozilla Public License
 ## Version 1.1 (the "License"); you may not use this file except in
 ## compliance with the License. You may obtain a copy of the License
-## at http://www.mozilla.org/MPL/
+## at https://www.mozilla.org/MPL/
 ##
 ## Software distributed under the License is distributed on an "AS IS"
 ## basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -16,6 +16,8 @@
 defmodule RabbitMQ.CLI.Formatters.PrettyTable do
   @behaviour RabbitMQ.CLI.FormatterBehaviour
 
+  alias RabbitMQ.CLI.Formatters.FormatterHelpers
+
   require Record
   import Record
 
@@ -30,7 +32,7 @@ defmodule RabbitMQ.CLI.Formatters.PrettyTable do
     # Flatten for list_consumers
     entries_with_keys = Stream.flat_map(stream,
       fn([first | _] = element) ->
-        case Keyword.keyword?(first) or is_map(first) do
+        case FormatterHelpers.proplist?(first) or is_map(first) do
           true  -> element;
           false -> [element]
         end
