@@ -19,7 +19,7 @@ defmodule RabbitMQCtl.MixfileBase do
   def project do
     [
       app: :rabbitmqctl,
-      version: "3.7.17",
+      version: "3.8.0-dev",
       elixir: ">= 1.7.0 and < 1.10.0",
       build_embedded: Mix.env == :prod,
       start_permanent: Mix.env == :prod,
@@ -38,7 +38,9 @@ defmodule RabbitMQCtl.MixfileBase do
     [applications: [:logger],
      env: [scopes: ['rabbitmq-plugins': :plugins,
                     rabbitmqctl: :ctl,
-                    'rabbitmq-diagnostics': :diagnostics]]
+                    'rabbitmq-diagnostics': :diagnostics,
+                    'rabbitmq-queues': :queues,
+                    'rabbitmq-upgrade': :upgrade]]
     ]
     |> add_modules(Mix.env)
   end
@@ -102,7 +104,8 @@ defmodule RabbitMQCtl.MixfileBase do
 
       {:amqp, "~> 1.2.0", only: :test},
       {:dialyxir, "~> 0.5", only: :test, runtime: false},
-      {:temp, "~> 0.4", only: :test}
+      {:temp, "~> 0.4", only: :test},
+      {:x509, "~> 0.7", only: :test}
     ]
 
     rabbitmq_deps = case System.get_env("DEPS_DIR") do
