@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -72,10 +72,10 @@ merge_defaults(A, Opts) ->
     {A, maps:merge(#{only_down => false}, Opts)}.
 
 banner(_, #{node := Node, only_down := true}) ->
-    erlang:iolist_to_binary([<<"Listing federation links, which are down on node ">>,
+    erlang:iolist_to_binary([<<"Listing federation links which are down on node ">>,
                              atom_to_binary(Node, utf8), <<"...">>]);
 banner(_, #{node := Node, only_down := false}) ->
-    erlang:iolist_to_binary([<<"Listing federation links of node ">>,
+    erlang:iolist_to_binary([<<"Listing federation links on node ">>,
                              atom_to_binary(Node, utf8), <<"...">>]).
 
 run(_Args, #{node := Node, only_down := OnlyDown}) ->
