@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -78,7 +78,7 @@ run([Id], #{node := Node}) ->
         {badrpc, _} = Error ->
             Error;
         not_found ->
-            {error, <<"Link with provided ID was not found">>};
+            {error, <<"Link with the given ID was not found">>};
         Obj ->
             Upstream = proplists:get_value(upstream, Obj),
             Supervisor = proplists:get_value(supervisor, Obj),
