@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -16,6 +16,7 @@
 
 -module(rabbit_federation_link_util).
 
+-include_lib("rabbit/include/amqqueue.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 -include("rabbit_federation.hrl").
 
@@ -95,8 +96,8 @@ get_connection_name(#upstream{name = UpstreamName},
     connection_name(UpstreamName, PolicyName);
 
 get_connection_name(#upstream{name = UpstreamName},
-    #upstream_params{x_or_q = Resource}) when is_record(Resource, amqqueue)->
-    Policy = Resource#amqqueue.policy,
+    #upstream_params{x_or_q = Resource}) when ?is_amqqueue(Resource) ->
+    Policy = amqqueue:get_policy(Resource),
     PolicyName = proplists:get_value(name, Policy),
     connection_name(UpstreamName, PolicyName);
 
