@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -123,12 +123,14 @@ identity(#entry{key       = {#resource{virtual_host = VHost,
                                        kind         = Type,
                                        name         = XorQNameBin},
                              UpstreamName, UXorQNameBin},
-                id = Id}) ->
+                id = Id,
+                upstream = #upstream{consumer_tag = ConsumerTag}}) ->
     case Type of
         exchange -> [{exchange,          XorQNameBin},
                      {upstream_exchange, UXorQNameBin}];
         queue    -> [{queue,             XorQNameBin},
-                     {upstream_queue,    UXorQNameBin}]
+                     {upstream_queue,    UXorQNameBin},
+                     {consumer_tag,      ConsumerTag}]
     end ++ [{type,      Type},
             {vhost,     VHost},
             {upstream,  UpstreamName},
