@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -17,6 +17,7 @@
 -module(rabbit_federation_upstream).
 
 -include("rabbit_federation.hrl").
+-include_lib("rabbit/include/amqqueue.hrl").
 -include_lib("amqp_client/include/amqp_client.hrl").
 
 -export([federate/1, for/1, for/2, params_to_string/1, to_params/2]).
@@ -61,7 +62,7 @@ upstreams(XorQ) ->
 params_table(SafeURI, XorQ) ->
     Key = case XorQ of
               #exchange{} -> <<"exchange">>;
-              #amqqueue{} -> <<"queue">>
+              Q when ?is_amqqueue(Q) -> <<"queue">>
           end,
     [{<<"uri">>,          longstr, SafeURI},
      {Key,                longstr, name(XorQ)}].
@@ -135,6 +136,7 @@ from_upstream_or_set(US, Name, U, XorQ) ->
     #upstream{uris            = URIs,
               exchange_name   = bget(exchange,          US, U, name(XorQ)),
               queue_name      = bget(queue,             US, U, name(XorQ)),
+              consumer_tag    = bget('consumer-tag',    US, U, <<"federation-link-", Name/binary>>),
               prefetch_count  = bget('prefetch-count',  US, U, ?DEF_PREFETCH),
               reconnect_delay = bget('reconnect-delay', US, U, 5),
               max_hops        = bget('max-hops',        US, U, 1),
@@ -167,5 +169,5 @@ a2b(A) -> list_to_binary(atom_to_list(A)).
 with_name(#upstream{exchange_name = XNameBin}, VHostBin, X = #exchange{}) ->
     X#exchange{name = rabbit_misc:r(VHostBin, exchange, XNameBin)};
 
-with_name(#upstream{queue_name = QNameBin}, VHostBin, Q = #amqqueue{}) ->
-    Q#amqqueue{name = rabbit_misc:r(VHostBin, queue, QNameBin)}.
+with_name(#upstream{queue_name = QNameBin}, VHostBin, Q) when ?is_amqqueue(Q) ->
+    amqqueue:set_name(Q, rabbit_misc:r(VHostBin, queue, QNameBin)).
