@@ -19,11 +19,13 @@ var KNOWN_ARGS = {'alternate-exchange':        {'short': 'AE',  'type': 'string'
                   'x-expires':                 {'short': 'Exp', 'type': 'int'},
                   'x-max-length':              {'short': 'Lim', 'type': 'int'},
                   'x-max-length-bytes':        {'short': 'Lim B', 'type': 'int'},
+                  'x-delivery-limit':          {'short': 'DlL', 'type': 'int'},
                   'x-overflow':                {'short': 'Ovfl', 'type': 'string'},
                   'x-dead-letter-exchange':    {'short': 'DLX', 'type': 'string'},
                   'x-dead-letter-routing-key': {'short': 'DLK', 'type': 'string'},
                   'x-queue-master-locator':    {'short': 'ML', 'type': 'string'},
-                  'x-max-priority':            {'short': 'Pri', 'type': 'int'}};
+                  'x-max-priority':            {'short': 'Pri', 'type': 'int'},
+                  'x-single-active-consumer':  {'short': 'SAC', 'type': 'boolean'}};
 
 // Things that are like arguments that we format the same way in listings.
 var IMPLICIT_ARGS = {'durable':         {'short': 'D',    'type': 'boolean'},
@@ -55,7 +57,7 @@ var NAVIGATION = {'Overview':    ['#/',            "management"],
 var CHART_RANGES = {'global': [], 'basic': []};
 var ALL_CHART_RANGES = {};
 
-var COLUMNS =
+var ALL_COLUMNS =
     {'exchanges' :
      {'Overview': [['type',                 'Type',                   true],
                    ['features',             'Features (with policy)', true],
@@ -64,7 +66,8 @@ var COLUMNS =
       'Message rates': [['rate-in',         'rate in',                true],
                         ['rate-out',        'rate out',               true]]},
      'queues' :
-     {'Overview': [['features',             'Features (with policy)', true],
+     {'Overview': [['type',                 'Type',                   true],
+                   ['features',             'Features (with policy)', true],
                    ['features_no_policy',   'Features (no policy)',   false],
                    ['policy',               'Policy',                 false],
                    ['consumers',            'Consumer count',         false],
@@ -95,15 +98,16 @@ var COLUMNS =
                        ['acks-uncommitted', 'Acks uncommitted', false]],
       'Message rates': [['rate-publish',   'publish',            true],
                         ['rate-confirm',   'confirm',            true],
-                        ['rate-return',    'return (mandatory)', false],
+                        ['rate-unroutable-drop',    'unroutable (drop)', true],
+                        ['rate-unroutable-return',    'unroutable (return)', false],
                         ['rate-deliver',   'deliver / get',      true],
                         ['rate-redeliver', 'redelivered',        false],
                         ['rate-ack',       'ack',                true]]},
      'connections':
      {'Overview': [['user',   'User name', true],
                    ['state',  'State',     true]],
-      'Details': [['ssl',            'SSL / TLS',      true],
-                  ['ssl_info',       'SSL Details',    false],
+      'Details': [['ssl',            'TLS',      true],
+                  ['ssl_info',       'TLS details',    false],
                   ['protocol',       'Protocol',       true],
                   ['channels',       'Channels',       true],
                   ['channel_max',    'Channel max',    false],
@@ -116,7 +120,9 @@ var COLUMNS =
                   ['connected_at', 'Connected at', false]]},
 
      'vhosts':
-     {'Overview': [['cluster-state',   'Cluster state',  false]],
+     {'Overview': [['cluster-state',   'Cluster state',  false],
+                   ['description',   'Description',  false],
+                   ['tags',   'Tags',  false]],
       'Messages': [['msgs-ready',      'Ready',          true],
                    ['msgs-unacked',    'Unacknowledged', true],
                    ['msgs-total',      'Total',          true]],
@@ -134,8 +140,37 @@ var COLUMNS =
                   ['info',      'Info',         true],
                   ['reset_stats',     'Reset stats',        true]]}};
 
+var DISABLED_STATS_COLUMNS =
+    {'exchanges' :
+     {'Overview': [['type',                 'Type',                   true],
+                   ['features',             'Features (with policy)', true],
+                   ['features_no_policy',   'Features (no policy)',   false],
+                   ['policy',               'Policy',                 false]]},
+     'queues' :
+     {'Overview': [['type',                 'Type',                   true],
+                   ['features',             'Features (with policy)', true],
+                   ['features_no_policy',   'Features (no policy)',   false],
+                   ['policy',               'Policy',                 false],
+                   ['state',                'State',                  true]],
+      'Messages': [['msgs-ready',      'Ready',          true],
+                   ['msgs-unacked',    'Unacknowledged', true],
+                   ['msgs-ram',        'In memory',      false],
+                   ['msgs-persistent', 'Persistent',     false],
+                   ['msgs-total',      'Total',          true]]},
+     'connections':
+     {'Overview': [['user',   'User name', true],
+                   ['state',  'State',     true]]},
+
+     'vhosts':
+     {'Overview': [['cluster-state',   'Cluster state',  false]]}};
+
+var COLUMNS;
+
 // All help ? popups
 var HELP = {
+    'delivery-limit':
+      'The number of allowed unsuccessful delivery attempts. Once a message has been delivered unsucessfully this many times it will be dropped or dead-lettered, depending on the queue configuration.',
+
     'exchange-auto-delete':
       'If yes, the exchange will delete itself after at least one queue or exchange has been bound to this one, and then all queues or exchanges have been unbound.',
 
@@ -157,6 +192,12 @@ var HELP = {
     'queue-max-length-bytes':
       'Total body size for ready messages a queue can contain before it starts to drop them from its head.<br/>(Sets the "<a target="_blank" href="https://rabbitmq.com/maxlength.html">x-max-length-bytes</a>" argument.)',
 
+    'queue-max-in-memory-length':
+      'How many (ready) messages a quorum queue can contain in memory before it starts storing them on disk only.<br/>(Sets the x-max-in-memory-length argument.)',
+
+    'queue-max-in-memory-bytes':
+      'Total body size for ready messages a quorum queue can contain in memory before it starts storing them on disk only.<br/>(Sets the x-max-in-memory-bytes argument.)',
+
     'queue-auto-delete':
       'If yes, the queue will delete itself after at least one consumer has connected, and then all consumers have disconnected.',
 
@@ -166,6 +207,9 @@ var HELP = {
     'queue-dead-letter-routing-key':
       'Optional replacement routing key to use when a message is dead-lettered. If this is not set, the message\'s original routing key will be used.<br/>(Sets the "<a target="_blank" href="https://rabbitmq.com/dlx.html">x-dead-letter-routing-key</a>" argument.)',
 
+    'queue-single-active-consumer':
+      'If set, makes sure only one consumer at a time consumes from the queue and fails over to another registered consumer in case the active one is cancelled or dies.<br/>(Sets the "<a target="_blank" href="https://rabbitmq.com/consumers.html#single-active-consumer">x-single-active-consumer</a>" argument.)',
+
     'queue-max-priority':
       'Maximum number of priority levels for the queue to support; if not set, the queue will not support message priorities.<br/>(Sets the "<a target="_blank" href="https://rabbitmq.com/priority.html">x-max-priority</a>" argument.)',
 
@@ -173,11 +217,14 @@ var HELP = {
       'Set the queue into lazy mode, keeping as many messages as possible on disk to reduce RAM usage; if not set, the queue will keep an in-memory cache to deliver messages as fast as possible.<br/>(Sets the "<a target="_blank" href="https://www.rabbitmq.com/lazy-queues.html">x-queue-mode</a>" argument.)',
 
     'queue-overflow':
-      'Sets the <a target="_blank" href="https://www.rabbitmq.com/maxlength.html#overflow-behaviour">queue overflow behaviour</a>. This determines what happens to messages when the maximum length of a queue is reached. Valid values are <code>drop-head</code> or <code>reject-publish</code>.',
+      'Sets the <a target="_blank" href="https://www.rabbitmq.com/maxlength.html#overflow-behaviour">queue overflow behaviour</a>. This determines what happens to messages when the maximum length of a queue is reached. Valid values are <code>drop-head</code>, <code>reject-publish</code> or <code>reject-publish-dlx</code>. The quorum queue type only supports <code>drop-head</code>.',
 
     'queue-master-locator':
        'Set the queue into master location mode, determining the rule by which the queue master is located when declared on a cluster of nodes.<br/>(Sets the "<a target="_blank" href="https://www.rabbitmq.com/ha.html">x-queue-master-locator</a>" argument.)',
 
+    'queue-type':
+       'Set the queue type, determining the type of queue to use: raft-based high availability or classic queue. Valid values are <code>quorum</code> or <code>classic</code>. It defaults to <code>classic<code>. <br/>',
+
     'queue-messages':
       '<p>Message counts.</p><p>Note that "in memory" and "persistent" are not mutually exclusive; persistent messages can be in memory as well as on disc, and transient messages can be paged out if memory is tight. Non-durable queues will consider all messages to be transient.</p>',
 
@@ -370,6 +417,8 @@ var HELP = {
         <dd>Rate at which messages requiring acknowledgement are being delivered in response to basic.get.</dd>\
         <dt>Get (auto ack)</dt>\
         <dd>Rate at which messages not requiring acknowledgement are being delivered in response to basic.get.</dd>\
+        <dt>Get (empty)</dt>\
+        <dd>Rate at which empty queues are hit in response to basic.get.</dd>\
         <dt>Return</dt>\
         <dd>Rate at which basic.return is sent to publishers for unroutable messages published with the \'mandatory\' flag set.</dd>\
         <dt>Disk read</dt>\
@@ -417,9 +466,16 @@ var HELP = {
 
     'filter-regex' :
     'Whether to enable regular expression matching. Both string literals \
-    and regular expressions are matched in a case-insensitive manner.<br/></br/> \
+    and regular expressions are matched in a case-insensitive manner.<br/><br/> \
     (<a href="https://developer.mozilla.org/en/docs/Web/JavaScript/Guide/Regular_Expressions" target="_blank">Regular expression reference</a>)',
 
+    'consumer-active' :
+    'Whether the consumer is active or not, i.e. whether the consumer can get messages from the queue. \
+    When single active consumer is enabled for the queue, only one consumer at a time is active. \
+    When single active consumer is disabled for the queue, consumers are active by default. \
+    For a quorum queue, a consumer can be inactive because its owning node is suspected down. <br/><br/> \
+    (<a href="https://www.rabbitmq.com/consumers.html#active-consumer" target="_blank">Documentation</a>)',
+
     'plugins' :
     'Note that only plugins which are both explicitly enabled and running are shown here.',
 
@@ -535,7 +591,9 @@ var is_user_policymaker;         // ...user is not a policymaker
 var user_monitor;                // ...user cannot monitor
 var nodes_interesting;           // ...we are not in a cluster
 var vhosts_interesting;          // ...there is only one vhost
+var queue_type;
 var rabbit_versions_interesting; // ...all cluster nodes run the same version
+var disable_stats;               // ...disable all stats, management only mode
 
 // Extensions write to this, the dispatcher maker reads it
 var dispatcher_modules = [];
@@ -594,9 +652,15 @@ function setup_global_vars() {
         }
     }
     vhosts_interesting = JSON.parse(sync_get('/vhosts')).length > 1;
+
+    queue_type = "classic";
     current_vhost = get_pref('vhost');
     exchange_types = overview.exchange_types;
 
+    disable_stats = overview.disable_stats;
+    enable_queue_totals = overview.enable_queue_totals;
+    COLUMNS = disable_stats?DISABLED_STATS_COLUMNS:ALL_COLUMNS;
+    
     setup_chart_ranges(overview.sample_retention_policies);
 }
 
@@ -708,3 +772,7 @@ var chart_data = {};
 // whenever a UI requests a page that doesn't exist
 // because things were deleted between refreshes
 var last_page_out_of_range_error = 0;
+
+var enable_uaa;
+var uaa_client_id;
+var uaa_location;
