@@ -1,6 +1,17 @@
 $(document).ready(function() {
-    replace_content('outer', format('login', {}));
-    start_app_login();
+    if (enable_uaa) {
+        get(uaa_location + "/info", "application/json", function(req) {
+            if (req.status !== 200) {
+                replace_content('outer', format('login_uaa', {}));
+                replace_content('login-status', '<p class="warning">' + uaa_location + " does not appear to be a running UAA instance or may not have a trusted SSL certificate"  + '</p> <button id="loginWindow" onclick="uaa_login_window()">Single Sign On</button>');
+            } else {
+                replace_content('outer', format('login_uaa', {}));
+            }
+        });
+    } else {
+        replace_content('outer', format('login', {}));
+        start_app_login();
+    }
 });
 
 function dispatcher_add(fun) {
@@ -58,6 +69,15 @@ function login_route_with_path() {
     window.location.replace(location);
 }
 
+function getParameterByName(name) {
+    var match = RegExp('[#&]' + name + '=([^&]*)').exec(window.location.hash);
+    return match && decodeURIComponent(match[1].replace(/\+/g, ' '));
+}
+
+function getAccessToken() {
+    return getParameterByName('access_token');
+}
+
 function start_app_login() {
     app = new Sammy.Application(function () {
         this.get('#/', function() {});
@@ -70,19 +90,58 @@ function start_app_login() {
         this.get('#/login/:username/:password', login_route);
         this.get(/\#\/login\/(.*)/, login_route_with_path);
     });
-    app.run();
-    if (get_cookie_value('auth') != null) {
-        check_login();
+    if (enable_uaa) {
+        var token = getAccessToken();
+        if (token != null) {
+            set_auth_pref(uaa_client_id + ':' + token);
+            store_pref('uaa_token', token);
+            check_login();
+        } else if(has_auth_cookie_value()) {
+            check_login();
+        };
+    } else {
+        app.run();
+        if (get_cookie_value('auth') != null) {
+            check_login();
+        }
     }
 }
 
+
+function uaa_logout_window() {
+    uaa_invalid = true;
+    uaa_login_window();
+}
+
+function uaa_login_window() {
+    var redirect;
+    if (window.location.hash != "") {
+        redirect = window.location.href.split(window.location.hash)[0];
+    } else {
+        redirect = window.location.href
+    };
+    var loginRedirectUrl;
+    if (uaa_invalid) {
+        loginRedirectUrl = Singular.properties.uaaLocation + '/logout.do?client_id=' + Singular.properties.clientId + '&redirect=' + redirect;
+    } else {
+        loginRedirectUrl = Singular.properties.uaaLocation + '/oauth/authorize?response_type=token&client_id=' + Singular.properties.clientId + '&redirect_uri=' + redirect;
+    };
+    window.open(loginRedirectUrl, "LOGIN_WINDOW");
+}
+
 function check_login() {
     user = JSON.parse(sync_get('/whoami'));
     if (user == false) {
         // clear a local storage value used by earlier versions
         clear_pref('auth');
+        clear_pref('uaa_token');
         clear_cookie_value('auth');
-        replace_content('login-status', '<p>Login failed</p>');
+        if (enable_uaa) {
+            uaa_invalid = true;
+            replace_content('login-status', '<button id="loginWindow" onclick="uaa_login_window()">Log out</button>');
+        } else {
+            replace_content('login-status', '<p>Login failed</p>');
+        }
     }
     else {
         hide_popup_warn();
@@ -138,12 +197,20 @@ function start_app() {
     // just leave the history here.
     //Sammy.HashLocationProxy._interval = null;
 
-
     app = new Sammy.Application(dispatcher);
     app.run();
+
     var url = this.location.toString();
+    var hash = this.location.hash;
+    var pathname = this.location.pathname;
     if (url.indexOf('#') == -1) {
         this.location = url + '#/';
+    } else if (hash.indexOf('#token_type') != - 1 && pathname == '/') {
+        // This is equivalent to previous `if` clause when uaa authorisation is used.
+        // Tokens are passed in the url hash, so the url always contains a #.
+        // We need to check the current path is `/` and token is present,
+        // so we can redirect to `/#/`
+        this.location = url.replace(/#token_type.+/gi, "#/");
     }
 }
 
@@ -560,7 +627,11 @@ function submit_import(form) {
                 vhost_part = '/' + esc(vhost_name);
             }
 
-            var form_action = "/definitions" + vhost_part + '?auth=' + get_cookie_value('auth');
+            if (enable_uaa) {
+                var form_action = "/definitions" + vhost_part + '?token=' + get_pref('uaa_token');
+            } else {
+                var form_action = "/definitions" + vhost_part + '?auth=' + get_cookie_value('auth');
+            };
             var fd = new FormData();
             fd.append('file', file);
             with_req('POST', form_action, fd, function(resp) {
@@ -600,9 +671,15 @@ function postprocess() {
     $('#download-definitions').on('click', function() {
             var idx = $("select[name='vhost-download'] option:selected").index();
             var vhost = ((idx <=0 ) ? "" : "/" + esc($("select[name='vhost-download'] option:selected").val()));
+        if (enable_uaa) {
             var path = 'api/definitions' + vhost + '?download=' +
                 esc($('#download-filename').val()) +
-                '&auth=' + get_cookie_value('auth');
+                '&token=' + get_pref('uaa_token');
+            } else {
+                var path = 'api/definitions' + vhost + '?download=' +
+                    esc($('#download-filename').val()) +
+                    '&auth=' + get_cookie_value('auth');
+            };
             window.location = path;
             setTimeout('app.run()');
             return false;
@@ -1114,10 +1191,14 @@ function has_auth_cookie_value() {
 }
 
 function auth_header() {
-    if(has_auth_cookie_value()) {
-        return "Basic " + decodeURIComponent(get_cookie_value('auth'));
+    if(has_auth_cookie_value() && enable_uaa) {
+        return "Bearer " + decodeURIComponent(get_pref('uaa_token'));
     } else {
-        return null;
+        if(has_auth_cookie_value()) {
+            return "Basic " + decodeURIComponent(get_cookie_value('auth'));
+        } else {
+            return null;
+        }
     }
 }
 
@@ -1149,6 +1230,19 @@ function with_req(method, path, body, fun) {
     req.send(body);
 }
 
+function get(url, accept, callback) {
+  var req = new XMLHttpRequest();
+  req.open("GET", url);
+  req.setRequestHeader("Accept", accept);
+  req.send();
+
+  req.onreadystatechange = function() {
+    if (req.readyState == XMLHttpRequest.DONE) {
+      callback(req);
+    }
+  };
+}
+
 function sync_get(path) {
     return sync_req('GET', [], path);
 }
@@ -1355,6 +1449,14 @@ function collapse_multifields(params0) {
             }
         }
     }
+    if (params.hasOwnProperty('queuetype')) {
+        delete params['queuetype'];
+        params['arguments']['x-queue-type'] = queue_type;
+        if (queue_type == 'quorum') {
+            params['durable'] = true;
+            params['auto_delete'] = false;
+        }
+    }
     return params;
 }
 
@@ -1537,6 +1639,35 @@ function rename_multifield(params, from, to) {
     return new_params;
 }
 
+function select_queue_type(queuetype) {
+    queue_type = queuetype.value;
+    update();
+}
+
+function is_quorum(queue) {
+    if (queue["arguments"]) {
+        if (queue["arguments"]["x-queue-type"]) {
+            return queue["arguments"]["x-queue-type"] === "quorum";
+        } else {
+            return false;
+        }
+    } else {
+        return false;
+    }
+}
+
+function is_classic(queue) {
+    if (queue["arguments"]) {
+        if (queue["arguments"]["x-queue-type"]) {
+            return queue["arguments"]["x-queue-type"] === "classic";
+        } else {
+            return true;
+        }
+    } else {
+        return true;
+    }
+}
+
 function ensure_queues_chart_range() {
     var range = get_pref('chart-range');
     // Note: the queues page uses the 'basic' range type
@@ -1595,4 +1726,3 @@ function get_chart_range_type(arg) {
     console.log('[WARNING]: range type not found for arg: ' + arg);
     return 'basic';
 }
-
