@@ -298,8 +298,8 @@ id_lookup(Type, List) ->
 
 overview(User, Ranges, Interval) ->
     VHosts = case User of
-                 all -> rabbit_vhost:list();
-                 _   -> rabbit_mgmt_util:list_visible_vhosts(User)
+                 all -> rabbit_vhost:list_names();
+                 _   -> rabbit_mgmt_util:list_visible_vhosts_names(User)
              end,
     DataLookup = get_data_from_nodes({rabbit_mgmt_data, overview_data,
                                       [User, Ranges, VHosts]}),
@@ -346,7 +346,8 @@ event_queue() ->
 
 consumers_stats(VHost) ->
     Data =  get_data_from_nodes({rabbit_mgmt_data, consumer_data, [VHost]}),
-    Consumers = [V || {_,V} <- maps:to_list(Data)],
+    Consumers = rabbit_mgmt_data_compat:fill_consumer_active_fields(
+                  [V || {_,V} <- maps:to_list(Data)]),
     ChPids = [ pget(channel_pid, Con)
                || Con <- Consumers, [] =:= pget(channel_details, Con)],
     ChDets = get_channel_detail_lookup(ChPids),
@@ -379,7 +380,9 @@ detail_queue_stats(Ranges, Objs, Interval) ->
        QueueData = maps:get(Id, DataLookup),
        Props = maps:get(queue_stats, QueueData),
        Stats = queue_stats(QueueData, Ranges, Interval),
-       Consumers = [{consumer_details, maps:get(consumer_stats, QueueData)}],
+       ConsumerStats = rabbit_mgmt_data_compat:fill_consumer_active_fields(
+                         maps:get(consumer_stats, QueueData)),
+       Consumers = [{consumer_details, ConsumerStats}],
        StatsD = [{deliveries,
                   detail_stats(QueueData, channel_queue_stats_deliver_stats,
                                deliver_get, second(Id), Ranges, Interval)},
@@ -456,6 +459,42 @@ format_range(Data, Key, Range0, Interval) ->
    rabbit_mgmt_stats:format_range(Range0, Now, Table, Interval, InstantRateFun,
                                   SamplesFun).
 
+%% basic.get-empty metric
+fetch_slides(Ele, Key, Data)
+  when Key =:= channel_queue_stats_deliver_stats orelse
+       Key =:= channel_stats_deliver_stats orelse
+       Key =:= queue_stats_deliver_stats orelse
+       Key =:= vhost_stats_deliver_stats orelse
+       (is_tuple(Key) andalso
+        (element(1, Key) =:= channel_queue_stats_deliver_stats orelse
+         element(1, Key) =:= channel_stats_deliver_stats orelse
+         element(1, Key) =:= queue_stats_deliver_stats orelse
+         element(1, Key) =:= vhost_stats_deliver_stats)) ->
+    case element(Ele, maps:get(Key, Data)) of
+        not_found -> [];
+        Slides when is_list(Slides) ->
+            [rabbit_mgmt_data_compat:fill_get_empty_queue_metric(S)
+             || S <- Slides, not_found =/= S];
+        Slide ->
+            [rabbit_mgmt_data_compat:fill_get_empty_queue_metric(Slide)]
+    end;
+%% drop_unroutable metric
+fetch_slides(Ele, Key, Data)
+  when Key =:= channel_stats_fine_stats orelse
+       Key =:= channel_exchange_stats_fine_stats orelse
+       Key =:= vhost_stats_fine_stats orelse
+       (is_tuple(Key) andalso
+        (element(1, Key) =:= channel_stats_fine_stats orelse
+         element(1, Key) =:= channel_exchange_stats_fine_stats orelse
+         element(1, Key) =:= vhost_stats_fine_stats)) ->
+    case element(Ele, maps:get(Key, Data)) of
+        not_found -> [];
+        Slides when is_list(Slides) ->
+            [rabbit_mgmt_data_compat:fill_drop_unroutable_metric(S)
+             || S <- Slides, not_found =/= S];
+        Slide ->
+            [rabbit_mgmt_data_compat:fill_drop_unroutable_metric(Slide)]
+    end;
 fetch_slides(Ele, Key, Data) ->
     case element(Ele, maps:get(Key, Data)) of
         not_found -> [];
@@ -561,7 +600,9 @@ detail_channel_stats(Ranges, Objs, Interval) ->
          ChannelData = maps:get(Id, DataLookup),
          Props = maps:get(channel_stats, ChannelData),
          Stats = channel_stats(ChannelData, Ranges, Interval),
-         Consumers = [{consumer_details, maps:get(consumer_stats, ChannelData)}],
+         ConsumerStats = rabbit_mgmt_data_compat:fill_consumer_active_fields(
+                           maps:get(consumer_stats, ChannelData)),
+         Consumers = [{consumer_details, ConsumerStats}],
          StatsD = [{publishes,
                     detail_stats(ChannelData, channel_exchange_stats_fine_stats,
                                  fine_stats, first(Id), Ranges, Interval)},
