@@ -1,14 +1,14 @@
 %% the contents of this file are subject to the mozilla public license
 %% version 1.1 (the "license"); you may not use this file except in
 %% compliance with the license. you may obtain a copy of the license at
-%% http://www.mozilla.org/mpl/
+%% https://www.mozilla.org/mpl/
 %%
 %% software distributed under the license is distributed on an "as is"
 %% basis, without warranty of any kind, either express or implied. see the
 %% license for the specific language governing rights and limitations
 %% under the license.
 %%
-%% copyright (c) 2016 pivotal software, inc.  all rights reserved.
+%% Copyright (c) 2016-2018 Pivotal Software, Inc. All rights reserved.
 
 -module(rabbit_mgmt_db_cache).
 
@@ -92,7 +92,7 @@ start_link(Key) ->
 %%%===================================================================
 
 init([]) ->
-    Mult = application:get_env(rabbitmg_management, management_db_cache_multiplier,
+    Mult = application:get_env(rabbitmq_management, management_db_cache_multiplier,
                                ?DEFAULT_MULT),
     {ok, #state{data = none,
                 args = [],
@@ -102,14 +102,17 @@ handle_call({fetch, FetchFun, FunArgs}, _From,
             #state{data = CachedData, args = Args,
                    multiplier = Mult, timer_ref = Ref} = State) when
      CachedData =:= none orelse Args =/= FunArgs ->
-    _ = timer:cancel(Ref),
+    case Ref of
+        R when is_reference(R) -> _ = erlang:cancel_timer(R);
+        _ -> ok
+    end,
 
     try timer:tc(FetchFun, FunArgs) of
         {Time, Data} ->
             case trunc(Time / 1000 * Mult) of
                 0 -> {reply, {ok, Data}, ?RESET_STATE(State)}; % no need to cache that
                 T ->
-                    {ok, TimerRef} = timer:send_after(T, self(), purge_cache),
+                    TimerRef = erlang:send_after(T, self(), purge_cache),
                     {reply, {ok, Data}, State#state{data = Data,
                                                     timer_ref = TimerRef,
                                                     args = FunArgs}}
