@@ -29,14 +29,16 @@ build_routes(Ignore) ->
     ManagementApp = module_app(?MODULE),
     Prefix = rabbit_mgmt_util:get_path_prefix(),
     RootIdxRtes = build_root_index_routes(Prefix, ManagementApp),
-    ApiRdrRte = build_static_index_html_route(Prefix, "/api"),
-    CliRdrRte = build_static_index_html_route(Prefix, "/cli"),
+    ApiRdrRte = build_redirect_route("/api", Prefix ++ "/api/index.html"),
+    CliRdrRte = build_redirect_route("/cli", Prefix ++ "/cli/index.html"),
+    StatsRdrRte1 = build_redirect_route("/stats", Prefix ++ "/api/index.html"),
+    StatsRdrRte2 = build_redirect_route("/doc/stats.html", Prefix ++ "/api/index.html"),
     MgmtRdrRte = {"/mgmt", rabbit_mgmt_wm_redirect, "/"},
     LocalPaths = [{module_app(M), "www"} || M <- modules(Ignore)],
     LocalStaticRte = {"/[...]", rabbit_mgmt_wm_static, LocalPaths},
     % NB: order is significant in the routing list
     Routes0 = build_module_routes(Ignore) ++
-        [ApiRdrRte, CliRdrRte, MgmtRdrRte, LocalStaticRte],
+        [ApiRdrRte, CliRdrRte, MgmtRdrRte, StatsRdrRte1, StatsRdrRte2, LocalStaticRte],
     Routes1 = maybe_add_path_prefix(Routes0, Prefix),
     % NB: ensure the root routes are first
     Routes2 = RootIdxRtes ++ Routes1,
@@ -48,10 +50,8 @@ build_root_index_routes(Prefix, ManagementApp) ->
     [{"/", rabbit_mgmt_wm_redirect, Prefix ++ "/"},
      {Prefix, cowboy_static, root_idx_file(ManagementApp)}].
 
-build_static_index_html_route("", Path) ->
-    {Path, rabbit_mgmt_wm_redirect, Path ++ "/index.html"};
-build_static_index_html_route(Prefix, Path) ->
-    {Path, rabbit_mgmt_wm_redirect, Prefix ++ Path ++ "/index.html"}.
+build_redirect_route(Path, Location) ->
+    {Path, rabbit_mgmt_wm_redirect, Location}.
 
 root_idx_file(ManagementApp) ->
     {priv_file, ManagementApp, "www/index.html"}.
@@ -172,5 +172,6 @@ dispatcher() ->
      {"/healthchecks/node",                                    rabbit_mgmt_wm_healthchecks, []},
      {"/healthchecks/node/:node",                              rabbit_mgmt_wm_healthchecks, []},
      {"/reset",                                                rabbit_mgmt_wm_reset, []},
-     {"/reset/:node",                                          rabbit_mgmt_wm_reset, []}
+     {"/reset/:node",                                          rabbit_mgmt_wm_reset, []},
+     {"/auth",                                                 rabbit_mgmt_wm_auth, []}
     ].
