@@ -38,8 +38,8 @@ content_types_provided(ReqData, Context) ->
 
 to_json(ReqData, Context) ->
     try
-        rabbit_mgmt_util:reply_list_or_paginate(augmented(ReqData, Context),
-            ReqData, Context)
+        Connections = do_connections_query(ReqData, Context),
+        rabbit_mgmt_util:reply_list_or_paginate(Connections, ReqData, Context)
     catch
         {error, invalid_range_parameters, Reason} ->
             rabbit_mgmt_util:bad_request(iolist_to_binary(Reason), ReqData, Context)
@@ -52,3 +52,12 @@ augmented(ReqData, Context) ->
     rabbit_mgmt_util:filter_conn_ch_list(
       rabbit_mgmt_db:get_all_connections(
         rabbit_mgmt_util:range_ceil(ReqData)), ReqData, Context).
+
+do_connections_query(ReqData, Context) ->
+    case rabbit_mgmt_util:disable_stats(ReqData) of
+        false ->
+            augmented(ReqData, Context);
+        true ->
+            rabbit_mgmt_util:filter_tracked_conn_list(rabbit_connection_tracking:list(),
+                                                      ReqData, Context)
+    end.
