@@ -50,12 +50,18 @@ resource_exists(ReqData, Context) ->
 
 to_json(ReqData, Context) ->
     try
-        [Q] = rabbit_mgmt_db:augment_queues(
-                [queue(ReqData)], rabbit_mgmt_util:range_ceil(ReqData),
-                full),
-        Payload = rabbit_mgmt_format:clean_consumer_details(
-                    rabbit_mgmt_format:strip_pids(Q)),
-        rabbit_mgmt_util:reply(ensure_defaults(Payload), ReqData, Context)
+        case rabbit_mgmt_util:disable_stats(ReqData) of
+            false ->
+                [Q] = rabbit_mgmt_db:augment_queues(
+                        [queue(ReqData)], rabbit_mgmt_util:range_ceil(ReqData),
+                        full),
+                Payload = rabbit_mgmt_format:clean_consumer_details(
+                            rabbit_mgmt_format:strip_pids(Q)),
+                rabbit_mgmt_util:reply(ensure_defaults(Payload), ReqData, Context);
+            true ->
+                rabbit_mgmt_util:reply(rabbit_mgmt_format:strip_pids(queue(ReqData)),
+                                       ReqData, Context)
+        end
     catch
         {error, invalid_range_parameters, Reason} ->
             rabbit_mgmt_util:bad_request(iolist_to_binary(Reason), ReqData, Context)
