@@ -19,7 +19,7 @@
 -export([init/2, resource_exists/2, to_json/2,
          content_types_provided/2, content_types_accepted/2,
          is_authorized/2, allowed_methods/2, accept_content/2,
-         delete_resource/2, id/1, put_vhost/3]).
+         delete_resource/2, id/1, put_vhost/5]).
 -export([variances/2]).
 
 -import(rabbit_misc, [pget/2]).
@@ -49,10 +49,16 @@ resource_exists(ReqData, Context) ->
 
 to_json(ReqData, Context) ->
     try
-        rabbit_mgmt_util:reply(
-          hd(rabbit_mgmt_db:augment_vhosts(
-               [rabbit_vhost:info(id(ReqData))], rabbit_mgmt_util:range(ReqData))),
-          ReqData, Context)
+        Id = id(ReqData),
+        case rabbit_mgmt_util:disable_stats(ReqData) of
+            false ->
+                rabbit_mgmt_util:reply(
+                  hd(rabbit_mgmt_db:augment_vhosts(
+                       [rabbit_vhost:info(Id)], rabbit_mgmt_util:range(ReqData))),
+                  ReqData, Context);
+            true ->
+                rabbit_mgmt_util:reply(rabbit_vhost:info(Id), ReqData, Context)
+        end
     catch
         {error, invalid_range_parameters, Reason} ->
             rabbit_mgmt_util:bad_request(iolist_to_binary(Reason), ReqData, Context)
@@ -62,16 +68,18 @@ accept_content(ReqData0, Context = #context{user = #user{username = Username}})
     Name = id(ReqData0),
     rabbit_mgmt_util:with_decode(
       [], ReqData0, Context,
-      fun(_, VHost, ReqData) ->
-              Trace = rabbit_mgmt_util:parse_bool(maps:get(tracing, VHost, undefined)),
-              case put_vhost(Name, Trace, Username) of
-                  ok               ->
-                      {true, ReqData, Context};
-                  {error, timeout} = E ->
-                      rabbit_mgmt_util:internal_server_error(
-                        "Timed out while waiting for the vhost to initialise", E,
-                        ReqData0, Context)
-              end
+      fun(_, BodyMap, ReqData) ->
+        Trace = rabbit_mgmt_util:parse_bool(maps:get(tracing, BodyMap, undefined)),
+        Description = maps:get(description, BodyMap, <<"">>),
+        Tags = maps:get(tags, BodyMap, <<"">>),
+        case put_vhost(Name, Description, Tags, Trace, Username) of
+            ok ->
+                {true, ReqData, Context};
+            {error, timeout} = E ->
+                rabbit_mgmt_util:internal_server_error(
+                  "Timed out while waiting for the vhost to initialise", E,
+                  ReqData0, Context)
+        end
       end).
 
 delete_resource(ReqData, Context = #context{user = #user{username = Username}}) ->
@@ -89,44 +97,10 @@ is_authorized(ReqData, Context) ->
 %%--------------------------------------------------------------------
 
 id(ReqData) ->
-    rabbit_mgmt_util:id(vhost, ReqData).
+    case rabbit_mgmt_util:id(vhost, ReqData) of
+      [Value] -> Value;
+      Value   -> Value
+    end.
 
-put_vhost(Name, Trace, Username) ->
-    Result = case rabbit_vhost:exists(Name) of
-        true  -> ok;
-        false -> rabbit_vhost:add(Name, Username),
-                 %% wait for up to 45 seconds for the vhost to initialise
-                 %% on all nodes
-                 case rabbit_vhost:await_running_on_all_nodes(Name, 45000) of
-                     ok               ->
-                         maybe_grant_full_permissions(Name, Username),
-                         case Trace of
-                             true      -> rabbit_trace:start(Name);
-                             false     -> rabbit_trace:stop(Name);
-                             undefined -> ok
-                         end;
-                     {error, timeout} ->
-                         {error, timeout}
-                 end
-    end,
-    case Trace of
-        true      -> rabbit_trace:start(Name);
-        false     -> rabbit_trace:stop(Name);
-        undefined -> ok
-    end,
-    Result.
-
-
-%% when definitions are loaded on boot, Username here will be ?INTERNAL_USER,
-%% which does not actually exist
-maybe_grant_full_permissions(_Name, ?INTERNAL_USER) ->
-    ok;
-maybe_grant_full_permissions(Name, Username) ->
-    U = rabbit_auth_backend_internal:lookup_user(Username),
-    maybe_grant_full_permissions(U, Name, Username).
-
-maybe_grant_full_permissions({ok, _}, Name, Username) ->
-    rabbit_auth_backend_internal:set_permissions(
-      Username, Name, <<".*">>, <<".*">>, <<".*">>, Username);
-maybe_grant_full_permissions(_, _Name, _Username) ->
-    ok.
+put_vhost(Name, Description, Tags, Trace, Username) ->
+    rabbit_vhost:put_vhost(Name, Description, Tags, Trace, Username).
