@@ -57,12 +57,22 @@ is_authorized(ReqData, Context) ->
 %%--------------------------------------------------------------------
 
 augment(Basic, ReqData) ->
-    rabbit_mgmt_db:augment_vhosts(Basic, rabbit_mgmt_util:range(ReqData)).
+    case rabbit_mgmt_util:disable_stats(ReqData) of
+        false ->
+            rabbit_mgmt_db:augment_vhosts(Basic, rabbit_mgmt_util:range(ReqData));
+        true ->
+            Basic
+    end.
 
 augmented(ReqData, #context{user = User}) ->
-    rabbit_mgmt_db:augment_vhosts(
-      [rabbit_vhost:info(V) || V <- rabbit_mgmt_util:list_visible_vhosts(User)],
-      rabbit_mgmt_util:range(ReqData)).
+    case rabbit_mgmt_util:disable_stats(ReqData) of
+        false ->            
+            rabbit_mgmt_db:augment_vhosts(
+              [rabbit_vhost:info(V) || V <- rabbit_mgmt_util:list_visible_vhosts(User)],
+              rabbit_mgmt_util:range(ReqData));
+        true ->
+            [rabbit_vhost:info(V) || V <- rabbit_mgmt_util:list_visible_vhosts(User)]
+    end.
 
 basic() ->
     rabbit_vhost:info_all([name]).
