@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -85,11 +85,13 @@ gc_channels() ->
 gc_queues() ->
     Queues = rabbit_amqqueue:list_names(),
     GbSet = gb_sets:from_list(Queues),
+    LocalQueues = rabbit_amqqueue:list_local_names(),
+    LocalGbSet = gb_sets:from_list(LocalQueues),
     gc_entity(queue_stats_publish, GbSet),
-    gc_entity(queue_stats, GbSet),
-    gc_entity(queue_msg_stats, GbSet),
-    gc_entity(queue_process_stats, GbSet),
-    gc_entity(queue_msg_rates, GbSet),
+    gc_entity(queue_stats, LocalGbSet),
+    gc_entity(queue_msg_stats, LocalGbSet),
+    gc_entity(queue_process_stats, LocalGbSet),
+    gc_entity(queue_msg_rates, LocalGbSet),
     gc_entity(queue_stats_deliver_stats, GbSet),
     gc_process_and_entity(channel_queue_stats_deliver_stats_queue_index, GbSet),
     gc_process_and_entity(consumer_stats_queue_index, GbSet),
