@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 -module(rabbit_mgmt_metrics_collector).
 
@@ -254,14 +254,14 @@ aggregate_entry({Id, Metrics}, NextStats, Ops0,
     Entry = ?channel_stats(Id, Ftd),
     Ops = insert_op(channel_stats, Id, Entry, Ops0),
     {NextStats, Ops, State};
-aggregate_entry({{Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, 0},
+aggregate_entry({{Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, DropUnroutable, 0},
                 NextStats, Ops0,
                 #state{table = channel_exchange_metrics,
                        policies = {BPolicies, DPolicies, GPolicies},
                        rates_mode = RatesMode,
                        lookup_exchange = ExchangeFun} = State) ->
-    Stats = ?channel_stats_fine_stats(Publish0, Confirm, ReturnUnroutable),
-    {Publish, _, _} = Diff = get_difference(Id, Stats, State),
+    Stats = ?channel_stats_fine_stats(Publish0, Confirm, ReturnUnroutable, DropUnroutable),
+    {Publish, _, _, _} = Diff = get_difference(Id, Stats, State),
 
     Ops1 = insert_entry_ops(channel_stats_fine_stats, Ch, true, Diff, Ops0,
                             BPolicies),
@@ -282,13 +282,13 @@ aggregate_entry({{Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, 0},
                    Ops2
            end,
     {insert_old_aggr_stats(NextStats, Id, Stats), Ops3, State};
-aggregate_entry({{_Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, 1},
+aggregate_entry({{_Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, DropUnroutable, 1},
                 NextStats, Ops0,
                 #state{table = channel_exchange_metrics,
                        policies = {_BPolicies, DPolicies, GPolicies},
                        lookup_exchange = ExchangeFun} = State) ->
-    Stats = ?channel_stats_fine_stats(Publish0, Confirm, ReturnUnroutable),
-    {Publish, _, _} = Diff = get_difference(Id, Stats, State),
+    Stats = ?channel_stats_fine_stats(Publish0, Confirm, ReturnUnroutable, DropUnroutable),
+    {Publish, _, _, _} = Diff = get_difference(Id, Stats, State),
     Ops1 = insert_entry_ops(vhost_stats_fine_stats, vhost(X), true, Diff, Ops0,
                             GPolicies),
     Ops2 = case ExchangeFun(X) of
@@ -302,14 +302,15 @@ aggregate_entry({{_Ch, X} = Id, Publish0, Confirm, ReturnUnroutable, 1},
     rabbit_core_metrics:delete(channel_exchange_metrics, Id),
     {NextStats, Ops2, State};
 aggregate_entry({{Ch, Q} = Id, Get, GetNoAck, Deliver, DeliverNoAck,
-                     Redeliver, Ack, 0}, NextStats, Ops0,
+                     Redeliver, Ack, GetEmpty, 0}, NextStats, Ops0,
                 #state{table = channel_queue_metrics,
                        policies = {BPolicies, DPolicies, GPolicies},
                        rates_mode = RatesMode,
                        lookup_queue = QueueFun} = State) ->
     Stats = ?vhost_stats_deliver_stats(Get, GetNoAck, Deliver, DeliverNoAck,
                                        Redeliver, Ack,
-                                       Deliver + DeliverNoAck + Get + GetNoAck),
+                                       Deliver + DeliverNoAck + Get + GetNoAck,
+                                       GetEmpty),
     Diff = get_difference(Id, Stats, State),
 
     Ops1 = insert_entry_ops(vhost_stats_deliver_stats, vhost(Q), true, Diff,
@@ -332,13 +333,14 @@ aggregate_entry({{Ch, Q} = Id, Get, GetNoAck, Deliver, DeliverNoAck,
            end,
      {insert_old_aggr_stats(NextStats, Id, Stats), Ops3, State};
 aggregate_entry({{_, Q} = Id, Get, GetNoAck, Deliver, DeliverNoAck,
-                     Redeliver, Ack, 1}, NextStats, Ops0,
+                     Redeliver, Ack, GetEmpty, 1}, NextStats, Ops0,
                 #state{table = channel_queue_metrics,
                        policies = {BPolicies, _, GPolicies},
                        lookup_queue = QueueFun} = State) ->
     Stats = ?vhost_stats_deliver_stats(Get, GetNoAck, Deliver, DeliverNoAck,
                                        Redeliver, Ack,
-                                       Deliver + DeliverNoAck + Get + GetNoAck),
+                                       Deliver + DeliverNoAck + Get + GetNoAck,
+                                       GetEmpty),
     Diff = get_difference(Id, Stats, State),
 
     Ops1 = insert_entry_ops(vhost_stats_deliver_stats, vhost(Q), true, Diff,
@@ -401,7 +403,8 @@ aggregate_entry({Id, Reductions}, NextStats, Ops0,
     Ops = insert_entry_ops(channel_process_stats, Id, false,
                            Entry, Ops0, BPolicies),
     {NextStats, Ops, State};
-aggregate_entry({Id, Exclusive, AckRequired, PrefetchCount, Args},
+aggregate_entry({Id, Exclusive, AckRequired, PrefetchCount,
+                 Active, ActivityStatus, Args},
                 NextStats, Ops0,
                 #state{table = consumer_created} = State) ->
     case ets:lookup(consumer_stats, Id) of
@@ -409,10 +412,28 @@ aggregate_entry({Id, Exclusive, AckRequired, PrefetchCount, Args},
             Fmt = rabbit_mgmt_format:format([{exclusive, Exclusive},
                                              {ack_required, AckRequired},
                                              {prefetch_count, PrefetchCount},
+                                             {active, Active},
+                                             {activity_status, ActivityStatus},
                                              {arguments, Args}], {[], false}),
             Entry = ?consumer_stats(Id, Fmt),
             Ops = insert_with_index_op(consumer_stats, Id, Entry, Ops0),
-            {NextStats, Ops ,State};
+            {NextStats, Ops , State};
+        [{_K, V}] ->
+            CurrentActive = proplists:get_value(active, V, undefined),
+            case Active =:= CurrentActive of
+                false ->
+                    Fmt = rabbit_mgmt_format:format([{exclusive, Exclusive},
+                                                     {ack_required, AckRequired},
+                                                     {prefetch_count, PrefetchCount},
+                                                     {active, Active},
+                                                     {activity_status, ActivityStatus},
+                                                     {arguments, Args}], {[], false}),
+                    Entry = ?consumer_stats(Id, Fmt),
+                    Ops = insert_with_index_op(consumer_stats, Id, Entry, Ops0),
+                    {NextStats, Ops , State};
+                _ ->
+                    {NextStats, Ops0, State}
+            end;
         _ ->
             {NextStats, Ops0, State}
     end;
@@ -591,6 +612,10 @@ sum_entry({A0, A1}, {B0, B1}) ->
     {B0 + A0, B1 + A1};
 sum_entry({A0, A1, A2}, {B0, B1, B2}) ->
     {B0 + A0, B1 + A1, B2 + A2};
+sum_entry({A0, A1, A2, A3}, {B0, B1, B2, B3}) ->
+    {B0 + A0, B1 + A1, B2 + A2, B3 + A3};
+sum_entry({A0, A1, A2, A3, A4}, {B0, B1, B2, B3, B4}) ->
+    {B0 + A0, B1 + A1, B2 + A2, B3 + A3, B4 + A4};
 sum_entry({A0, A1, A2, A3, A4, A5}, {B0, B1, B2, B3, B4, B5}) ->
     {B0 + A0, B1 + A1, B2 + A2, B3 + A3, B4 + A4, B5 + A5};
 sum_entry({A0, A1, A2, A3, A4, A5, A6}, {B0, B1, B2, B3, B4, B5, B6}) ->
@@ -604,6 +629,10 @@ difference({A0, A1}, {B0, B1}) ->
     {B0 - A0, B1 - A1};
 difference({A0, A1, A2}, {B0, B1, B2}) ->
     {B0 - A0, B1 - A1, B2 - A2};
+difference({A0, A1, A2, A3}, {B0, B1, B2, B3}) ->
+    {B0 - A0, B1 - A1, B2 - A2, B3 - A3};
+difference({A0, A1, A2, A3, A4}, {B0, B1, B2, B3, B4}) ->
+    {B0 - A0, B1 - A1, B2 - A2, B3 - A3, B4 - A4};
 difference({A0, A1, A2, A3, A4, A5}, {B0, B1, B2, B3, B4, B5}) ->
     {B0 - A0, B1 - A1, B2 - A2, B3 - A3, B4 - A4, B5 - A5};
 difference({A0, A1, A2, A3, A4, A5, A6}, {B0, B1, B2, B3, B4, B5, B6}) ->
