@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2016 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 
 -module('Elixir.RabbitMQ.CLI.Ctl.Commands.ListMqttConnectionsCommand').
 
@@ -35,9 +35,7 @@
          help_section/0]).
 
 formatter() -> 'Elixir.RabbitMQ.CLI.Formatters.Table'.
-
 scopes() -> [ctl, diagnostics].
-
 switches() -> [{verbose, boolean}].
 aliases() -> [{'V', verbose}].
 
@@ -72,13 +70,16 @@ usage_doc_guides() ->
     [?MQTT_GUIDE_URL].
 
 run(Args, #{node := NodeName,
-                    timeout := Timeout,
-                    verbose := Verbose}) ->
+            timeout := Timeout,
+            verbose := Verbose}) ->
     InfoKeys = case Verbose of
         true  -> ?INFO_ITEMS;
         false -> 'Elixir.RabbitMQ.CLI.Ctl.InfoKeys':prepare_info_keys(Args)
     end,
-    Nodes = 'Elixir.RabbitMQ.CLI.Core.Helpers':nodes_in_cluster(NodeName),
+
+    %% a node uses the Raft-based collector to list connections, which knows about all connections in the cluster
+    %% so no need to reach out to all the nodes
+    Nodes = [NodeName],
 
     'Elixir.RabbitMQ.CLI.Ctl.RpcStream':receive_list_items(
         NodeName,
