@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -18,7 +18,7 @@
 
 -export([info/2, initial_state/2, initial_state/5,
          process_frame/2, amqp_pub/2, amqp_callback/2, send_will/1,
-         close_connection/1]).
+         close_connection/1, handle_pre_hibernate/0]).
 
 %% for testing purposes
 -export([get_vhost_username/1, get_vhost/3, get_vhost_from_user_mapping/2,
@@ -31,6 +31,7 @@
 -define(APP, rabbitmq_mqtt).
 -define(FRAME_TYPE(Frame, Type),
         Frame = #mqtt_frame{ fixed = #mqtt_frame_fixed{ type = Type }}).
+-define(MAX_TOPIC_PERMISSION_CACHE_SIZE, 12).
 
 initial_state(Socket, SSLLoginName) ->
     RealSocket = rabbit_net:unwrap_socket(Socket),
@@ -127,10 +128,10 @@ process_request(?CONNECT,
                         rabbit_log_connection:error("MQTT login failed: no credentials provided~n"),
                         {?CONNACK_CREDENTIALS, PState1};
                     {invalid_creds, {undefined, Pass}} when is_list(Pass) ->
-                        rabbit_log_connection:error("MQTT login failed: no user username is provided"),
+                        rabbit_log_connection:error("MQTT login failed: no username is provided"),
                         {?CONNACK_CREDENTIALS, PState1};
                     {invalid_creds, {User, undefined}} when is_list(User) ->
-                        rabbit_log_connection:error("MQTT login failed for ~p: no password provided", [User]),
+                        rabbit_log_connection:error("MQTT login failed for user '~p': no password provided", [User]),
                         {?CONNACK_CREDENTIALS, PState1};
                     {UserBin, PassBin} ->
                         case process_login(UserBin, PassBin, ProtoVersion, PState1) of
@@ -138,43 +139,62 @@ process_request(?CONNECT,
                                 {SessionPresent0, PState2} = maybe_clean_sess(PState1),
                                 {{?CONNACK_ACCEPT, SessionPresent0}, PState2};
                             {?CONNACK_ACCEPT, Conn, VHost, AState} ->
-                                RetainerPid = rabbit_mqtt_retainer_sup:child_for_vhost(VHost),
-                                link(Conn),
-                                {ok, Ch} = amqp_connection:open_channel(Conn),
-                                link(Ch),
-                                amqp_channel:enable_delivery_flow_control(Ch),
-                                ok = rabbit_mqtt_collector:register(ClientId, self()),
-                                Prefetch = rabbit_mqtt_util:env(prefetch),
-                                #'basic.qos_ok'{} = amqp_channel:call(
-                                  Ch, #'basic.qos'{prefetch_count = Prefetch}),
-                                rabbit_mqtt_reader:start_keepalive(self(), Keepalive),
-                                PState3 = PState1#proc_state{
-                                            will_msg   = make_will_msg(Var),
-                                            clean_sess = CleanSess,
-                                            channels   = {Ch, undefined},
-                                            connection = Conn,
-                                            client_id  = ClientId,
-                                            retainer_pid = RetainerPid,
-                                            auth_state = AState},
-                                {SessionPresent1, PState4} = maybe_clean_sess(PState3),
-                                {{?CONNACK_ACCEPT, SessionPresent1}, PState4};
-                            ConnAck ->
-                                {ConnAck, PState1}
+                                case rabbit_mqtt_collector:register(ClientId, self()) of
+                                  ok ->
+                                    RetainerPid = rabbit_mqtt_retainer_sup:child_for_vhost(VHost),
+                                    link(Conn),
+                                    {ok, Ch} = amqp_connection:open_channel(Conn),
+                                    link(Ch),
+                                    amqp_channel:enable_delivery_flow_control(Ch),
+                                    Prefetch = rabbit_mqtt_util:env(prefetch),
+                                    #'basic.qos_ok'{} = amqp_channel:call(
+                                      Ch, #'basic.qos'{prefetch_count = Prefetch}),
+                                    rabbit_mqtt_reader:start_keepalive(self(), Keepalive),
+                                    PState3 = PState1#proc_state{
+                                                will_msg   = make_will_msg(Var),
+                                                clean_sess = CleanSess,
+                                                channels   = {Ch, undefined},
+                                                connection = Conn,
+                                                client_id  = ClientId,
+                                                retainer_pid = RetainerPid,
+                                                auth_state = AState},
+                                    {SessionPresent1, PState4} = maybe_clean_sess(PState3),
+                                    {{?CONNACK_ACCEPT, SessionPresent1}, PState4};
+                                  %% e.g. this node was removed from the MQTT cluster members
+                                  {error, _} = Err ->
+                                    rabbit_log_connection:error("MQTT cannot accept a connection: "
+                                                                "client ID tracker is unavailable: ~p", [Err]),
+                                    %% ignore all exceptions, we are shutting down
+                                    catch amqp_connection:close(Conn),
+                                    {?CONNACK_SERVER, PState1};
+                                  {timeout, _} ->
+                                    rabbit_log_connection:error("MQTT cannot accept a connection: "
+                                                                "client ID registration timed out"),
+                                    %% ignore all exceptions, we are shutting down
+                                    catch amqp_connection:close(Conn),
+                                    {?CONNACK_SERVER, PState1}
+                                end;
+                            ConnAck -> {ConnAck, PState1}
                         end
                 end
         end,
     {ReturnCode, SessionPresent} = case Return of
-                                       {?CONNACK_ACCEPT, _} ->
-                                           Return;
-                                       Return ->
-                                           {Return, false}
+                                       {?CONNACK_ACCEPT, Bool} -> {?CONNACK_ACCEPT, Bool};
+                                       Other                   -> {Other, false}
                                    end,
     SendFun(#mqtt_frame{fixed    = #mqtt_frame_fixed{type = ?CONNACK},
                         variable = #mqtt_frame_connack{
                                       session_present = SessionPresent,
                                       return_code = ReturnCode}},
             PState5),
-    {ok, PState5};
+    case ReturnCode of
+      ?CONNACK_ACCEPT      -> {ok, PState5};
+      ?CONNACK_CREDENTIALS -> {error, unauthenticated, PState5};
+      ?CONNACK_AUTH        -> {error, unauthorized, PState5};
+      ?CONNACK_SERVER      -> {error, unavailable, PState5};
+      ?CONNACK_INVALID_ID  -> {error, invalid_client_id, PState5};
+      ?CONNACK_PROTO_VER   -> {error, unsupported_protocol_version, PState5}
+    end;
 
 process_request(?PUBACK,
                 #mqtt_frame{
@@ -559,11 +579,11 @@ process_login(UserBin, PassBin, ProtoVersion,
                             ?CONNACK_AUTH
                     end;
                 {error, {auth_failure, Explanation}} ->
-                    rabbit_log_connection:error("MQTT login failed for ~p auth_failure: ~s~n",
+                    rabbit_log_connection:error("MQTT login failed for user '~p' auth_failure: ~s~n",
                         [binary_to_list(UserBin), Explanation]),
                     ?CONNACK_CREDENTIALS;
                 {error, access_refused} ->
-                    rabbit_log_connection:warning("MQTT login failed for ~p access_refused "
+                    rabbit_log_connection:warning("MQTT login failed for user '~p': access_refused "
                     "(vhost access not allowed)~n",
                         [binary_to_list(UserBin)]),
                     ?CONNACK_AUTH;
@@ -575,7 +595,7 @@ process_login(UserBin, PassBin, ProtoVersion,
                     ?CONNACK_AUTH
             end;
         false ->
-            rabbit_log_connection:error("MQTT login failed for ~p auth_failure: vhost ~s does not exist~n",
+            rabbit_log_connection:error("MQTT login failed for user '~p' auth_failure: vhost ~s does not exist~n",
                 [binary_to_list(UserBin), VHost]),
             ?CONNACK_CREDENTIALS
     end.
@@ -716,7 +736,8 @@ supported_subs_qos(?QOS_1) -> ?QOS_1;
 supported_subs_qos(?QOS_2) -> ?QOS_1.
 
 delivery_mode(?QOS_0) -> 1;
-delivery_mode(?QOS_1) -> 2.
+delivery_mode(?QOS_1) -> 2;
+delivery_mode(?QOS_2) -> 2.
 
 %% different qos subscriptions are received in different queues
 %% with appropriate durability and timeout arguments
@@ -878,19 +899,23 @@ close_connection(PState = #proc_state{ connection = Connection,
         undefined -> ok;
         _         -> ok = rabbit_mqtt_collector:unregister(ClientId, self())
     end,
-    %% ignore noproc or other exceptions to avoid debris
+    %% ignore noproc or other exceptions, we are shutting down
     catch amqp_connection:close(Connection),
     PState #proc_state{ channels   = {undefined, undefined},
                         connection = undefined }.
 
-% NB: check_*: MQTT spec says we should ack normally, ie pretend there
-% was no auth error, but here we are closing the connection with an error. This
-% is what happens anyway if there is an authorization failure at the AMQP level.
+handle_pre_hibernate() ->
+    erase(topic_permission_cache),
+    ok.
+
+%% NB: check_*: MQTT spec says we should ack normally, ie pretend there
+%% was no auth error, but here we are closing the connection with an error. This
+%% is what happens anyway if there is an authorization failure at the AMQP 0-9-1 client level.
 
 check_publish(TopicName, Fn, PState) ->
   case check_topic_access(TopicName, write, PState) of
     ok -> Fn();
-    _ -> {err, unauthorized, PState}
+    _ -> {error, unauthorized, PState}
   end.
 
 check_subscribe([], Fn, _) ->
@@ -899,7 +924,7 @@ check_subscribe([], Fn, _) ->
 check_subscribe([#mqtt_topic{name = TopicName} | Topics], Fn, PState) ->
   case check_topic_access(TopicName, read, PState) of
     ok -> check_subscribe(Topics, Fn, PState);
-    _ -> {err, unauthorized, PState}
+    _ -> {error, unauthorized, PState}
   end.
 
 check_topic_access(TopicName, Access,
@@ -909,28 +934,46 @@ check_topic_access(TopicName, Access,
                         exchange = Exchange,
                         client_id = ClientId,
                         mqtt2amqp_fun = Mqtt2AmqpFun }) ->
-  Resource = #resource{virtual_host = VHost,
-                       kind = topic,
-                       name = Exchange},
-  RoutingKey = Mqtt2AmqpFun(TopicName),
-  Context = #{routing_key  => RoutingKey,
-              variable_map => #{
-                  <<"username">>  => Username,
-                  <<"vhost">>     => VHost,
-                  <<"client_id">> => rabbit_data_coercion:to_binary(ClientId)
-              }
-  },
+    Cache =
+        case get(topic_permission_cache) of
+            undefined -> [];
+            Other     -> Other
+        end,
 
-  try rabbit_access_control:check_topic_access(User, Resource, Access, Context) of
-    R -> R
-  catch
-    _:{amqp_error, access_refused, Msg, _} ->
-      rabbit_log:error("operation resulted in an error (access_refused): ~p~n", [Msg]),
-        {error, access_refused};
-    _:Error ->
-      rabbit_log:error("~p~n", [Error]),
-        {error, access_refused}
-  end.
+    Key = {TopicName, Username, ClientId, VHost, Exchange, Access},
+    case lists:member(Key, Cache) of
+        true ->
+            ok;
+        false ->
+            Resource = #resource{virtual_host = VHost,
+                                 kind = topic,
+                                 name = Exchange},
+
+            RoutingKey = Mqtt2AmqpFun(TopicName),
+            Context = #{routing_key  => RoutingKey,
+                        variable_map => #{
+                                          <<"username">>  => Username,
+                                          <<"vhost">>     => VHost,
+                                          <<"client_id">> => rabbit_data_coercion:to_binary(ClientId)
+                                         }
+                       },
+
+            try rabbit_access_control:check_topic_access(User, Resource, Access, Context) of
+                ok ->
+                    CacheTail = lists:sublist(Cache, ?MAX_TOPIC_PERMISSION_CACHE_SIZE - 1),
+                    put(topic_permission_cache, [Key | CacheTail]),
+                    ok;
+                R ->
+                    R
+            catch
+                _:{amqp_error, access_refused, Msg, _} ->
+                    rabbit_log:error("operation resulted in an error (access_refused): ~p~n", [Msg]),
+                    {error, access_refused};
+                _:Error ->
+                    rabbit_log:error("~p~n", [Error]),
+                    {error, access_refused}
+            end
+    end.
 
 info(consumer_tags, #proc_state{consumer_tags = Val}) -> Val;
 info(unacked_pubs, #proc_state{unacked_pubs = Val}) -> Val;
