@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -11,7 +11,7 @@
 %% The Original Code is RabbitMQ.
 %%
 %% The Initial Developer of the Original Code is GoPivotal, Inc.
-%% Copyright (c) 2007-2017 Pivotal Software, Inc.  All rights reserved.
+%% Copyright (c) 2007-2019 Pivotal Software, Inc.  All rights reserved.
 %%
 
 -module(rabbit_mqtt_reader).
@@ -24,7 +24,7 @@
 
 -export([start_link/2]).
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
-         code_change/3, terminate/2]).
+         code_change/3, terminate/2, handle_pre_hibernate/1]).
 
 -export([conserve_resources/3, start_keepalive/2]).
 
@@ -107,10 +107,18 @@ handle_call(Msg, From, State) ->
 handle_cast(duplicate_id,
             State = #state{ proc_state = PState,
                             conn_name  = ConnName }) ->
-    rabbit_log_connection:warning("MQTT disconnecting duplicate client id ~p (~p)~n",
-                 [rabbit_mqtt_processor:info(client_id, PState), ConnName]),
+    rabbit_log_connection:warning("MQTT disconnecting client ~p with duplicate id '~s'~n",
+                 [ConnName, rabbit_mqtt_processor:info(client_id, PState)]),
     {stop, {shutdown, duplicate_id}, State};
 
+handle_cast(decommission_node,
+            State = #state{ proc_state = PState,
+                            conn_name  = ConnName }) ->
+    rabbit_log_connection:warning("MQTT disconnecting client ~p with id '~s' as its node is about"
+                                  " to be decommissioned~n",
+                 [ConnName, rabbit_mqtt_processor:info(client_id, PState)]),
+    {stop, {shutdown, decommission_node}, State};
+
 handle_cast(Msg, State) ->
     {stop, {mqtt_unexpected_cast, Msg}, State}.
 
@@ -189,6 +197,10 @@ terminate(Reason, State) ->
     maybe_emit_stats(State),
     do_terminate(Reason, State).
 
+handle_pre_hibernate(State) ->
+    rabbit_mqtt_processor:handle_pre_hibernate(),
+    {hibernate, State}.
+
 do_terminate({network_error, {ssl_upgrade_error, closed}, ConnStr}, _State) ->
     rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: connection closed~n",
        [ConnStr]);
@@ -256,8 +268,9 @@ log_tls_alert(Alert, ConnStr) ->
     rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: alert ~s~n",
        [ConnStr, Alert]).
 
-log_new_connection(#state{conn_name = ConnStr}) ->
-    rabbit_log_connection:info("accepting MQTT connection ~p (~s)~n", [self(), ConnStr]).
+log_new_connection(#state{conn_name = ConnStr, proc_state = PState}) ->
+    rabbit_log_connection:info("accepting MQTT connection ~p (~s, client id: ~s)~n",
+                               [self(), ConnStr, rabbit_mqtt_processor:info(client_id, PState)]).
 
 process_received_bytes(<<>>, State = #state{proc_state = ProcState,
                                             received_connect_frame = false}) ->
@@ -287,25 +300,43 @@ process_received_bytes(Bytes,
                       State #state{ parse_state = PS,
                                     proc_state = ProcState1,
                                     connection = ConnPid });
+                %% PUBLISH and more
+                {error, unauthorized = Reason, ProcState1} ->
+                    rabbit_log_connection:error("MQTT connection ~s is closing due to an authorization failure~n", [ConnStr]),
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)};
+                %% CONNECT frames only
+                {error, unauthenticated = Reason, ProcState1} ->
+                    rabbit_log_connection:error("MQTT connection ~s is closing due to an authentication failure~n", [ConnStr]),
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)};
+                %% CONNECT frames only
+                {error, invalid_client_id = Reason, ProcState1} ->
+                    rabbit_log_connection:error("MQTT cannot accept connection ~s: client uses an invalid ID~n", [ConnStr]),
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)};
+                %% CONNECT frames only
+                {error, unsupported_protocol_version = Reason, ProcState1} ->
+                    rabbit_log_connection:error("MQTT cannot accept connection ~s: incompatible protocol version~n", [ConnStr]),
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)};
+                {error, unavailable = Reason, ProcState1} ->
+                    rabbit_log_connection:error("MQTT cannot accept connection ~s due to an internal error or unavailable component~n",
+                        [ConnStr]),
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)};
                 {error, Reason, ProcState1} ->
-                    rabbit_log_connection:info("MQTT protocol error ~p for connection ~s~n",
-                        [Reason, ConnStr]),
+                    rabbit_log_connection:error("MQTT protocol error on connection ~s: ~p~n",
+                        [ConnStr, Reason]),
                     {stop, {shutdown, Reason}, pstate(State, ProcState1)};
                 {error, Error} ->
-                    rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~s~n",
-                        [Error, ConnStr]),
+                    rabbit_log_connection:error("MQTT detected a framing error on connection ~s: ~p~n",
+                        [ConnStr, Error]),
                     {stop, {shutdown, Error}, State};
                 {stop, ProcState1} ->
-                    {stop, normal, pstate(State, ProcState1)};
-                {err, unauthorized = Reason, ProcState1} ->
-                    {stop, {shutdown, Reason}, pstate(State, ProcState1)}
+                    {stop, normal, pstate(State, ProcState1)}
             end;
         {error, {cannot_parse, Error, Stacktrace}} ->
-            rabbit_log_connection:error("MQTT cannot parse frame for connection '~s', unparseable payload: ~p, error: {~p, ~p} ~n",
+            rabbit_log_connection:error("MQTT cannot parse a frame on connection '~s', unparseable payload: ~p, error: {~p, ~p} ~n",
                 [ConnStr, Bytes, Error, Stacktrace]),
             {stop, {shutdown, Error}, State};
         {error, Error} ->
-            rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~s~n",
+            rabbit_log_connection:error("MQTT detected a framing error on connection ~s: ~p~n",
                 [ConnStr, Error]),
             {stop, {shutdown, Error}, State}
     end.
