@@ -1,7 +1,7 @@
 %%   The contents of this file are subject to the Mozilla Public License
 %%   Version 1.1 (the "License"); you may not use this file except in
 %%   compliance with the License. You may obtain a copy of the License at
-%%   http://www.mozilla.org/MPL/
+%%   https://www.mozilla.org/MPL/
 %%
 %%   Software distributed under the License is distributed on an "AS IS"
 %%   basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -21,6 +21,7 @@
 
 start(_Type, _StartArgs) ->
     rabbit_peer_discovery_httpc:maybe_configure_proxy(),
+    rabbit_peer_discovery_httpc:maybe_configure_inet6(),
     rabbit_peer_discovery_common_sup:start_link().
 
 stop(_State) ->
