@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -44,7 +44,9 @@
 -type stringifyable() :: atom() | binary() | string() | integer().
 
 
--spec getenv(Key :: string()) -> string() | false.
+-spec getenv(Key :: string() | undefined) -> string() | false.
+getenv(undefined) ->
+  false;
 getenv(Key) ->
   process_getenv_value(Key, os:getenv(Key)).
 
