@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -50,7 +50,8 @@ init() ->
     %% we cannot start this plugin yet since it depends on the rabbit app,
     %% which is in the process of being started by the time this function is called
     application:load(rabbitmq_peer_discovery_common),
-    rabbit_peer_discovery_httpc:maybe_configure_proxy().
+    rabbit_peer_discovery_httpc:maybe_configure_proxy(),
+    rabbit_peer_discovery_httpc:maybe_configure_inet6().
 
 -spec list_nodes() -> {ok, {Nodes :: list(), NodeType :: rabbit_types:node_type()}} | {error, Reason :: string()}.
 
@@ -269,7 +270,8 @@ build_registration_body() ->
   Payload3 = registration_body_maybe_add_address(Payload2),
   Payload4 = registration_body_add_port(Payload3),
   Payload5 = registration_body_maybe_add_check(Payload4),
-  registration_body_maybe_add_tag(Payload5).
+  Payload6 = registration_body_maybe_add_tag(Payload5),
+  registration_body_maybe_add_meta(Payload6).
 
 -spec registration_body_add_id() -> list().
 registration_body_add_id() ->
@@ -356,6 +358,27 @@ registration_body_maybe_add_tag(Payload, Cluster, Tags) ->
   lists:append(Payload, [{'Tags', [rabbit_data_coercion:to_atom(Cluster)] ++ [rabbit_data_coercion:to_atom(X) || X <- Tags]}]).
 
 
+-spec registration_body_maybe_add_meta(Payload :: list()) -> list().
+registration_body_maybe_add_meta(Payload) ->
+  M = ?CONFIG_MODULE:config_map(?BACKEND_CONFIG_KEY),
+  ClusterName = get_config_key(cluster_name, M),
+  Meta = ?UTIL_MODULE:as_list(get_config_key(consul_svc_meta, M)),
+  registration_body_maybe_add_meta(Payload, ClusterName, Meta).
+
+-spec registration_body_maybe_add_meta(Payload :: list(),
+                                       ClusterName :: string(),
+                                       Meta :: list()) -> list().
+registration_body_maybe_add_meta(Payload, "default", []) ->
+  Payload;
+registration_body_maybe_add_meta(Payload, "default", Meta) ->
+  lists:append(Payload, [{<<"meta">>, Meta}]);
+registration_body_maybe_add_meta(Payload, _ClusterName, []) ->
+  Payload;
+registration_body_maybe_add_meta(Payload, ClusterName, Meta) ->
+  Merged = maps:to_list(maps:merge(#{<<"cluster">> => rabbit_data_coercion:to_binary(ClusterName)}, maps:from_list(Meta))),
+  lists:append(Payload, [{<<"meta">>, Merged}]).
+
+
 -spec validate_addr_parameters(false | true, false | true) -> false | true.
 validate_addr_parameters(false, true) ->
     rabbit_log:warning("The parameter CONSUL_SVC_ADDR_NODENAME" ++
@@ -517,7 +540,7 @@ create_session(Name, TTL) ->
 %% @end
 %%--------------------------------------------------------------------
 -spec consul_session_create(Query, Headers, Body) -> {ok, term()} | {error, Reason::string()} when
-      Query :: [autocluster_httpc:query_component()],
+      Query :: list(),
       Headers :: [{string(), string()}],
       Body :: term().
 consul_session_create(Query, Headers, Body) ->
