@@ -44,7 +44,8 @@ init() ->
     %% we cannot start this plugin yet since it depends on the rabbit app,
     %% which is in the process of being started by the time this function is called
     application:load(rabbitmq_peer_discovery_common),
-    ?HTTPC_MODULE:maybe_configure_proxy().
+    ?HTTPC_MODULE:maybe_configure_proxy(),
+    ?HTTPC_MODULE:maybe_configure_inet6().
 
 
 -spec list_nodes() -> {ok, {Nodes :: list(), NodeType :: rabbit_types:node_type()}} | {error, Reason :: string()}.
@@ -190,7 +191,11 @@ node_path(Map) ->
 extract_nodes([], Nodes) -> Nodes;
 extract_nodes([H|T], Nodes) ->
   M = ?CONFIG_MODULE:config_map(?BACKEND_CONFIG_KEY),
-  extract_nodes(T, lists:append(Nodes, [get_node_from_key(maps:get(<<"key">>, H), M)])).
+  ToAppend = case get_node_from_key(maps:get(<<"key">>, H), M) of
+                 {error, none} -> [];
+                 Name          -> [Name]
+             end,
+  extract_nodes(T, lists:append(Nodes, ToAppend)).
 
 %% @doc Return the list of erlang nodes
 %% @end
