@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License at
-%% http://www.mozilla.org/MPL/
+%% https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
@@ -23,7 +23,7 @@
 -include("rabbit_peer_discovery_k8s.hrl").
 
 -export([init/0, list_nodes/0, supports_registration/0, register/0, unregister/0,
-         post_registration/0, lock/1, unlock/1]).
+         post_registration/0, lock/1, unlock/1, randomized_startup_delay_range/0]).
 
 -ifdef(TEST).
 -compile(export_all).
@@ -47,7 +47,8 @@ init() ->
     %% we cannot start this plugin yet since it depends on the rabbit app,
     %% which is in the process of being started by the time this function is called
     application:load(rabbitmq_peer_discovery_common),
-    rabbit_peer_discovery_httpc:maybe_configure_proxy().
+    rabbit_peer_discovery_httpc:maybe_configure_proxy(),
+    rabbit_peer_discovery_httpc:maybe_configure_inet6().
 
 -spec list_nodes() -> {ok, {Nodes :: list(), NodeType :: rabbit_types:node_type()}}.
 
@@ -65,7 +66,9 @@ list_nodes() ->
 -spec supports_registration() -> boolean().
 
 supports_registration() ->
-    false.
+    %% see rabbitmq-peer-discovery-aws#17,
+    %%     rabbitmq-peer-discovery-k8s#23
+    true.
 
 
 -spec register() -> ok.
@@ -91,9 +94,18 @@ lock(_Node) ->
 unlock(_Data) ->
     ok.
 
+-spec randomized_startup_delay_range() -> {integer(), integer()}.
+
+randomized_startup_delay_range() ->
+    %% Pods in a stateful set are initialized one by one,
+    %% so RSD is not really necessary for this plugin.
+    %% See https://www.rabbitmq.com/cluster-formation.html#peer-discovery-k8s for details.
+    {0, 2}.
+
 %%
 %% Implementation
 %%
+
 -spec get_config_key(Key :: atom(), Map :: #{atom() => peer_discovery_config_value()})
                     -> peer_discovery_config_value().
 
@@ -144,7 +156,7 @@ maybe_ready_address(Subset) ->
     maps:get(<<"addresses">>, Subset, []).
 
 %% @doc Return a list of nodes
-%%    see http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_endpoints
+%%    see https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_endpoints
 %% @end
 %%
 -spec extract_node_list(term()) -> [binary()].
