@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -38,7 +38,7 @@
 %% We make sure the sharded queues are created when
 %% RabbitMQ starts.
 maybe_shard_exchanges() ->
-    [maybe_shard_exchanges(V) || V <- rabbit_vhost:list()],
+    [maybe_shard_exchanges(V) || V <- rabbit_vhost:list_names()],
     ok.
 
 maybe_shard_exchanges(VHost) ->
@@ -94,7 +94,7 @@ declare_queue(XName, Durable, N, Node) ->
     QBin = make_queue_name(exchange_bin(XName), a2b(Node), N),
     QueueName = rabbit_misc:r(v(XName), queue, QBin),
     try rabbit_amqqueue:declare(QueueName, Durable, false, [], none,
-                                ?SHARDING_USER, Node) of
+                                ?SHARDING_USER, {ignore_location, Node}) of
         {_Reply, _Q} ->
             ok
     catch
