@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -47,12 +47,12 @@ usage() ->
 usage_doc_guides() ->
     [?SHOVEL_GUIDE_URL].
 
-help_section() ->
-    {plugin, shovel}.
-
 description() ->
     <<"Displays status of Shovel on a node">>.
 
+help_section() ->
+    {plugin, shovel}.
+
 flags() ->
     [].
 
@@ -66,7 +66,7 @@ merge_defaults(A,O) ->
     {A, O}.
 
 banner(_, #{node := Node}) ->
-    erlang:iolist_to_binary([<<"Shovel status of node ">>,
+    erlang:iolist_to_binary([<<"Shovel status on node ">>,
                              atom_to_binary(Node, utf8)]).
 
 run(_Args, #{node := Node}) ->
