@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -194,7 +194,7 @@ validate_params_user(#amqp_params_direct{virtual_host = VHost},
                      User = #user{username = Username}) ->
     case rabbit_vhost:exists(VHost) andalso
         (catch rabbit_access_control:check_vhost_access(
-                 User, VHost, undefined)) of
+                 User, VHost, undefined, #{})) of
         ok -> ok;
         _  -> {error, "user \"~s\" may not connect to vhost \"~s\"",
                   [Username, VHost]}
