@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -90,7 +90,7 @@ filter_vhost_req(List, ReqData) ->
 %% static shovels do not have a vhost, so only allow admins (not
 %% monitors) to see them.
 filter_vhost_user(List, _ReqData, #context{user = User = #user{tags = Tags}}) ->
-    VHosts = rabbit_mgmt_util:list_login_vhosts(User, undefined),
+    VHosts = rabbit_mgmt_util:list_login_vhosts_names(User, undefined),
     [I || I <- List, case pget(vhost, I) of
                          undefined -> lists:member(administrator, Tags);
                          VHost     -> lists:member(VHost, VHosts)
