@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -504,7 +504,7 @@ cancel_subscription({ok, ConsumerTag, Description}, Frame,
 %% thus we don't have to clean it up.
 tidy_canceled_subscription(ConsumerTag, _Subscription,
                            undefined, State = #proc_state{subscriptions = Subs}) ->
-    Subs1 = dict:erase(ConsumerTag, Subs),
+    Subs1 = maps:remove(ConsumerTag, Subs),
     ok(State#proc_state{subscriptions = Subs1});
 
 %% Client-initiated cancelations will pass an actual frame
@@ -822,9 +822,15 @@ send_delivery(Delivery = #'basic.deliver'{consumer_tag = ConsumerTag},
                        [ConsumerTag],
                        State)
     end,
-    amqp_channel:notify_received(DeliveryCtx),
+    notify_received(DeliveryCtx),
     NewState.
 
+notify_received(undefined) ->
+  %% no notification for quorum queues
+  ok;
+notify_received(DeliveryCtx) ->
+  %% notification for flow control
+  amqp_channel:notify_received(DeliveryCtx).
 
 send_method(Method, Channel, State) ->
     amqp_channel:call(Channel, Method),
@@ -851,7 +857,10 @@ close_connection(State = #proc_state{connection = none}) ->
 close_connection(State = #proc_state{connection = Connection}) ->
     %% ignore noproc or other exceptions to avoid debris
     catch amqp_connection:close(Connection),
-    State#proc_state{channel = none, connection = none, subscriptions = none}.
+    State#proc_state{channel = none, connection = none, subscriptions = none};
+close_connection(undefined) ->
+    rabbit_log:debug("~s:close_connection: undefined state", [?MODULE]),
+    #proc_state{channel = none, connection = none, subscriptions = none}.
 
 %%----------------------------------------------------------------------------
 %% Reply-To
