@@ -1,7 +1,7 @@
 %% The contents of this file are subject to the Mozilla Public License
 %% Version 1.1 (the "License"); you may not use this file except in
 %% compliance with the License. You may obtain a copy of the License
-%% at http://www.mozilla.org/MPL/
+%% at https://www.mozilla.org/MPL/
 %%
 %% Software distributed under the License is distributed on an "AS IS"
 %% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -23,6 +23,7 @@
          code_change/3, terminate/2]).
 -export([start_heartbeats/2]).
 -export([info/2]).
+-export([ssl_login_name/2]).
 
 -include("rabbit_stomp.hrl").
 -include("rabbit_stomp_frame.hrl").
@@ -157,6 +158,12 @@ handle_info(#'basic.ack'{delivery_tag = Tag, multiple = IsMulti}, State) ->
                                                                  IsMulti,
                                                                  ProcState),
     {noreply, processor_state(NewProcState, State), hibernate};
+handle_info({Delivery = #'basic.deliver'{},
+             Message = #amqp_msg{}},
+             State) ->
+    %% receiving a message from a quorum queue
+    %% no delivery context
+    handle_info({Delivery, Message, undefined}, State);
 handle_info({Delivery = #'basic.deliver'{},
              #amqp_msg{props = Props, payload = Payload},
              DeliveryCtx},
