@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -122,8 +122,6 @@ ets_tables(_OldTables) ->
         end,
     lists:filtermap(F, ets:all()).
 
-
-table_info(Table) when not is_atom(Table) -> undefined;
 table_info(Table) ->
     TableInfo = ets:info(Table),
     map_table_info(Table, TableInfo).
