@@ -1,7 +1,7 @@
 %%  The contents of this file are subject to the Mozilla Public License
 %%  Version 1.1 (the "License"); you may not use this file except in
 %%  compliance with the License. You may obtain a copy of the License
-%%  at http://www.mozilla.org/MPL/
+%%  at https://www.mozilla.org/MPL/
 %%
 %%  Software distributed under the License is distributed on an "AS IS"
 %%  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
@@ -46,6 +46,12 @@ full_path(Name0) ->
 %%--------------------------------------------------------------------
 
 file_info(Name) ->
-    {ok, Info} = file:read_file_info(full_path(Name)),
-    [{name, list_to_binary(Name)},
-     {size, Info#file_info.size}].
+    Size = case file:read_file_info(full_path(Name)) of
+               {ok, Info} ->
+                   Info#file_info.size;
+               {error, Error} ->
+                   rabbit_log:warning("error getting file info for ~s: ~p",
+                                      [Name, Error]),
+                   0
+           end,
+    [{name, list_to_binary(Name)}, {size, Size}].
